/*
 * Created on Jan 27, 2005
 */
package proxyconfig;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

/**
 * @author danjo
 */
public class HelpButton extends JButton implements ActionListener, HyperlinkListener {

    private String helpFileName;
    private String toolTipText;
    private String helpTitle;
    private JEditorPane  mainEditorPane;
    
    public HelpButton (String cmd, ActionListener listener) {

        setActionCommand(cmd);
        addActionListener(listener);
        setFont(new Font("Arial",Font.PLAIN, 10));
        setText("?");
        setBackground(ProxyConfig.HELP_BUTTON_COLOR);;
    }

    /**
     * @param helpFileName The name of the file that contains the help information. 
     * @param toolTipText The tool tip for the help button.
     * @param helpTitle The title of the window that shows the help window.
     */
    public HelpButton (String helpFileName, String toolTipText, String helpTitle) {
        setIcon (new ImageIcon (Toolkit.getDefaultToolkit().getImage(getClass().getResource("/Help24.gif"))));

        this.helpFileName = helpFileName;
        this.toolTipText = toolTipText;
        this.helpTitle = helpTitle;

		if (this.toolTipText != null) {	
	    	setToolTipText (this.toolTipText);
		}

		setPreferredSize(new Dimension (24,24));
		setMaximumSize(new Dimension (24,24));
        //setFont(new Font("Arial",Font.PLAIN, 10));
        //setText("?");
		addActionListener ( this );
    }

    public Dimension getPreferredSize () {
        return new Dimension (24,24);
    }
    public Dimension getMaximumSize () {
        return new Dimension (24,24);
    }
    public void actionPerformed (ActionEvent ae) {
        URL helpURL = null;
        try {
            mainEditorPane = new JEditorPane ();
           
            helpURL = getClass().getResource("/" + this.helpFileName);
            mainEditorPane.setPage (helpURL);
            mainEditorPane.setEditable(false);
            mainEditorPane.addHyperlinkListener(this);
            
            JScrollPane scrollPane = new JScrollPane (mainEditorPane);
	    	scrollPane.setPreferredSize (new Dimension (400,400));
	    	JOptionPane.showMessageDialog ( null, 
					scrollPane,
					helpTitle,
					JOptionPane.INFORMATION_MESSAGE
				);
        } catch (IOException ioe) {
	    	System.out.println ( "Unable to display help information " + helpURL + " : " + ioe);
	    	ioe.printStackTrace();
		}
    }

    public void hyperlinkUpdate(HyperlinkEvent evt) {
        if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            try {
              mainEditorPane.setPage(evt.getURL());        
            } catch (Exception e) {
                System.out.println ("Unable to open the URL: " + evt.getURL());
                e.printStackTrace();
            } 
          }        
    }
}
