/*
 * Created on May 25, 2005
 */
package proxyconfig;

import proxyconfig.certificate.CertificateWizard;
import proxyconfig.mynames.MyName;
import proxyconfig.ocd.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.net.InetAddress;
import java.util.ListIterator;

/**
 * @author danjo
 */
public class WizardNamePanel extends JPanel implements ActionListener {

    ProxyConfig proxyConfig;
    OCD overlay;
    JTextField nameTF;
    MyName myName;
    
    WizardNamePanel(OCD overlay, ProxyConfig proxyConfig) {
        
        this.proxyConfig = proxyConfig;
        this.overlay = overlay;
        
        setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));
        setBackground (ProxyConfig.WHITE_COLOR);
        
        add (Box.createVerticalGlue());
        
        JPanel horizPanel = new JPanel();
        horizPanel.setBackground(proxyconfig.ProxyConfig.WHITE_COLOR);
        add(horizPanel);
        
        horizPanel.setLayout(new BoxLayout(horizPanel, BoxLayout.X_AXIS));
        
        horizPanel.add (Box.createHorizontalGlue());
        
        JEditorPane msgLB = new JEditorPane ( "text/html",
                "<div align=\"left\"><font face=\"arial\" color=\"blue\" size=\"5\">" +
                	"Set the name of this machine on this overlay: " + overlay.id  +
                "</font>"
        	);
        msgLB.setPreferredSize(new Dimension(600,60));
        msgLB.setMaximumSize(new Dimension(600,60));
        
        horizPanel.add(msgLB);
        horizPanel.add (Box.createHorizontalGlue());
        
        add (Box.createRigidArea (new Dimension(10,10)));
        
        horizPanel = new JPanel();
        horizPanel.setBackground(proxyconfig.ProxyConfig.WHITE_COLOR);
        add(horizPanel);
        
        horizPanel.setLayout(new BoxLayout(horizPanel, BoxLayout.X_AXIS));
        
        horizPanel.add (Box.createHorizontalGlue());
        nameTF = new JTextField (30);
        nameTF.setBackground (ProxyConfig.INPUT_BOX_COLOR);
        nameTF.setMaximumSize (new Dimension(400,30));
        try {
            InetAddress localAddress = InetAddress.getLocalHost ();
        	String tmpName = localAddress.getHostName() + "." + overlay.id;
        	nameTF.setText(tmpName);
        	
        	MyName tmpMyName = new MyName ();
            tmpMyName.active = true;
            tmpMyName.name = tmpName;
            tmpMyName.ocd = overlay;
            tmpMyName.ocdId = overlay.id;
            
            // Some overlays might want to set some special properties for a name
            // if it is shown as part of the NameWizard.
            overlay.initNameInNameWizard (tmpMyName);
            
            boolean nameExistsFlag = false;
            
            //go through my name list to see if same name already exists.  If not, add it.
            ListIterator nameItr = proxyConfig.proxyConfigData.myNameList.listIterator();
            while (nameItr.hasNext()) {
                MyName currName = (MyName) nameItr.next();
                if (currName.name.equals (tmpName)) {
                    nameExistsFlag = true;
                    tmpMyName.active = true;
                    tmpMyName = currName;
                    break;
                }
            }
            
            if (nameExistsFlag == false) {
                proxyConfig.proxyConfigData.myNameList.add(tmpMyName);
                proxyConfig.changeListener.setChanged(true);
            } 
            
            myName = tmpMyName;
        	
        } catch (Exception e) {
            e.printStackTrace();
        }
        horizPanel.add (nameTF);
        horizPanel.add (Box.createHorizontalGlue());

        add (Box.createVerticalGlue());
        
        horizPanel = new JPanel();
        horizPanel.setBackground(proxyconfig.ProxyConfig.WHITE_COLOR);
        add(horizPanel);
        
        horizPanel.setLayout(new BoxLayout(horizPanel, BoxLayout.X_AXIS));
        
        horizPanel.add (Box.createHorizontalGlue());
        JButton certBT = new JButton ("Obtain Certificate");
        certBT.addActionListener (this);
        certBT.setActionCommand ("OBTAIN_CERT");
        
        horizPanel.add (certBT);
        horizPanel.add (Box.createHorizontalGlue());

        
        add (Box.createVerticalGlue());
    }
    
    public void actionPerformed (ActionEvent ae) {
        String cmd = ae.getActionCommand();
        if (cmd.equals ("OBTAIN_CERT")) {
            CertificateWizard certWiz = new CertificateWizard(proxyConfig, nameTF.getText());
            if (certWiz.finishFlag) {
                
                if (certWiz.changeListener.hasChanged()) {
                    myName.certFileName = certWiz.getCertificateFileName();
                    myName.privKeyFileName = certWiz.getPrivKeyFileName();
                    
                    if (myName.certFileName != null && myName.privKeyFileName != null) {
                    
                        File tmpCertFile = new File (myName.certFileName);
                        String certMsg = "";
                        if (! tmpCertFile.exists()) {
                            certMsg += "Certificate file " + tmpCertFile.getPath() + " does not exist.\n";
                            myName.certOKFlag = MyName.CERT_NOT_FOUND;
                        }
                	
                        File tmpPrivKeyFile = new File (myName.privKeyFileName);
                        if (! tmpPrivKeyFile.exists()) {
                            certMsg += "Private key file " + tmpPrivKeyFile.getPath() + " does not exist.\n";
                            myName.certOKFlag = MyName.CERT_NOT_FOUND;
                        }
                	
                        if ( ! certMsg.equals("")) {
                            certMsg += "\nSince the certificate for name " + myName.name + " has not been " 
                	    			+ "correctly setup, \npeople will not be able to contact you on this name securely."
                	    			+ "\nDo you want to specify the certificate again?";
                	    
                        } else {
                            // Check validity of certificates
                            myName.certOKFlag = MyName.CERT_VALID;
	
                        }
                    
                    }
                    
                }
            }
            	
        	proxyConfig.changeListener.setChanged(true);
        }
    }
    
    void update () {
        if (!overlay.isActive) {
            proxyConfig.proxyConfigData.myNameList.remove(myName);
        } else {
            myName.name = nameTF.getText();
        }
    }
    
}
