/*
 * Created on Jan 22, 2005
 */
package proxyconfig.i3;

import javax.swing.table.AbstractTableModel;

import proxyconfig.outboundconnections.OutboundRecord;

import java.awt.Toolkit;
import java.util.*;
/**
 * @author danjo
 * This class implements the TableModel to display Records 
 */
public class I3AddressBookTableModel extends AbstractTableModel {

	/** The list of records to displayed in the table */
	private ArrayList recordList;
	
	/** The label for the columns */
	private String[] columns;
	
	public I3AddressBookTableModel(String[] columns) {
		
		recordList = new ArrayList();
		this.columns = columns;
		 	
	}
	
	public int getRowCount() {
		
		return recordList.size();
	}

	public int getColumnCount() {
		
		return columns.length;		
	}
	
	public String getColumnName(int columnIndex) {
		return columns[columnIndex];
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		
		I3AddressBookEntry currRec = (I3AddressBookEntry) recordList.get(rowIndex);
		
		switch (columnIndex) {
			case 0:
			    return currRec.name;
			
			case 1:
			    return currRec.i3Id;
			  
			default:
			    return "error";
		}
	}
	
	/** Add a record to the table */
	public void addRecord(I3AddressBookEntry rec) {
	    
	    recordList.add(rec);
	    fireTableDataChanged();
	}
	
	
	public void deleteRecord(I3AddressBookEntry rec) {
	    recordList.remove(rec);
	    fireTableDataChanged();
	}
	
	/** Add all the records in the specified list to the table */
	public void addRecords(List rList) {
		
		ListIterator itr = rList.listIterator();
		
		while (itr.hasNext()) {
		    I3AddressBookEntry tempRec = (I3AddressBookEntry) itr.next();
			recordList.add(tempRec);
		}
	    fireTableDataChanged();
	}
	/** Remove all records */
	public void clear() {
		recordList.clear();
		fireTableDataChanged();
	}
	
	/** Returns the specified record 
	 * Class cast errors possible here. 
	 * @param recIndex the Index of the Record in the records list
	 * @param row the selected row (may be same as recIndex, not sure)
	 */
	public I3AddressBookEntry getRecord(int recIndex) {
	
		if (recIndex >= 0 && recIndex < recordList.size()) {
		    I3AddressBookEntry tempRec = (I3AddressBookEntry) recordList.get(recIndex);
		    return tempRec;
		}
		
		return null;
	}

	/** Returns a list of all records contained in this table */
	public ArrayList getRecordList() {
		return new ArrayList(recordList);
	}

	/** Returns the inded of the specified Record in the records list */
	public int indexOf(OutboundRecord rec) {
		return recordList.indexOf(rec);
	}


}
