/*
 * Created on May 17, 2005
 */
package proxyconfig.i3;

import proxyconfig.mynames.*;
import proxyconfig.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.*;
import javax.swing.event.*;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import proxyconfig.ProxyConfig;
import proxyconfig.mynames.OCDNameEditPanel;

/**
 * @author danjo
 */
public class I3NameEditPanel extends OCDNameEditPanel implements ActionListener, ItemListener {

    public static final String TRIGGER_TYPE_NORMAL = "Direction connection";
    
    public static final String TRIGGER_TYPE_MIDDLEBOX = "Connect through middlebox";

    //public static final String TRIGGER_TYPE_PRIVATE_ID = "Private ID";
    
    JComboBox triggerTypeCombo;
    JLabel i3NameStackLB;
    JTextField i3NameStackTF;
    
    I3TriggerLimitPanel triggerRateLimitPanel;
    
    public I3NameEditPanel(MyName rec) {
        
        setBorder (BorderFactory.createTitledBorder(BorderFactory.createLineBorder (Color.BLACK),"i3"));
        setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
    	//Construct the GUI
		Insets insets = new Insets(2,2,2,2);
		GridBagLayout gbag = new GridBagLayout();
		setLayout(gbag);

		GridBagConstraints gc = new GridBagConstraints();
		gc.weighty = 0.0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.insets = insets;
		gc.gridx = GridBagConstraints.REMAINDER;
		gc.anchor = GridBagConstraints.WEST;

		
		JLabel tempLabel;

		// The preferred Dimension of all the input fields - mainly used to enforce a common height
		Dimension minDim = new Dimension(50, 25);
		
		tempLabel = new JLabel("Trigger Type");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		add(tempLabel);

		triggerTypeCombo = new JComboBox();
		triggerTypeCombo.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		triggerTypeCombo.setMinimumSize(minDim);
		triggerTypeCombo.addItemListener (this);
		gc.gridx = 1;
		gc.gridy = 0;
		gc.weightx = 1.0;
		gbag.setConstraints(triggerTypeCombo, gc);
		add(triggerTypeCombo);

		triggerRateLimitPanel = new I3TriggerLimitPanel(rec, new InputChangeListener() /*dummy */);
		gc.gridx = 0;
		gc.gridy = 1;
		gc.gridwidth = 2;
		gbag.setConstraints (triggerRateLimitPanel, gc);
		add (triggerRateLimitPanel);
		gc.gridwidth = 1;
				
		i3NameStackLB = new JLabel("Middleboxes");
		i3NameStackLB.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		i3NameStackLB.setVisible(false);
		gc.gridx = 0;
		gc.gridy = 3;
		gc.weightx = 0.0;
		gbag.setConstraints(i3NameStackLB, gc);
		add(i3NameStackLB);

		i3NameStackTF = new JTextField (100);
		i3NameStackTF.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		i3NameStackTF.setVisible(false);
		i3NameStackTF.setBackground (ProxyConfig.INPUT_BOX_COLOR);
		gc.gridx = 1;
		gc.gridy = 3;
		gc.weightx = 0.0;
		gbag.setConstraints(i3NameStackTF, gc);
		add(i3NameStackTF);

		
		triggerTypeCombo.addItem (TRIGGER_TYPE_NORMAL);
		triggerTypeCombo.addItem (TRIGGER_TYPE_MIDDLEBOX);
		//triggerTypeCombo.addItem (TRIGGER_TYPE_PRIVATE_ID);

		if (rec != null) {
		    
		    String selTrigType = (String) rec.ocdSpecificProperties.get(I3OCD.I3_TRIGGER_TYPE);
		    
		    String tmpNameStack = (String) rec.ocdSpecificProperties.get(I3OCD.I3_NAME_STACK);
		    if (tmpNameStack != null && ! tmpNameStack.trim().equals ("")) {
		        triggerTypeCombo.setSelectedItem(TRIGGER_TYPE_MIDDLEBOX);
	        	i3NameStackTF.setText ((String) rec.ocdSpecificProperties.get( (I3OCD.I3_NAME_STACK)));
	    		
		    } else {
		        triggerTypeCombo.setSelectedItem (TRIGGER_TYPE_NORMAL);
		    }
		}
		
    }

    public void actionPerformed (ActionEvent ae) {
    
    	String cmd = ae.getActionCommand();
    
    }
    
    
    public String checkFields() {
        String retStr = "";
        
        String trigType = (String) triggerTypeCombo.getSelectedItem();
        
        if (trigType.equals (TRIGGER_TYPE_MIDDLEBOX)) {
            if (i3NameStackTF.getText().trim().equals("")) {
                retStr += "Please specify the name of at least one middle box";
            } 
        }
        
        retStr += triggerRateLimitPanel.checkFields();
        
        return retStr;
    }
    
    public void update (MyName rec) {

        // This function should not be called for any of the private trigger myNames
        // as the trigger type is hard coded to public here.
        
        rec.ocdSpecificProperties.clear();
        
        String tmpTrigType = (String) triggerTypeCombo.getSelectedItem();
        
        rec.ocdSpecificProperties.put (I3OCD.I3_TRIGGER_TYPE, "public");
        
        if (tmpTrigType.equals (TRIGGER_TYPE_MIDDLEBOX)) {
            rec.ocdSpecificProperties.put (I3OCD.I3_NAME_STACK, i3NameStackTF.getText());
        }
        
        triggerRateLimitPanel.update (rec);
    }

    /* (non-Javadoc)
     * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
     */
    public void itemStateChanged(ItemEvent e) {
        String tmpType = (String) triggerTypeCombo.getSelectedItem();
        
        if (tmpType == null) {
            return;
        }
        
        i3NameStackLB.setVisible(false);
        i3NameStackTF.setVisible (false);
        
        if (tmpType.equals (TRIGGER_TYPE_MIDDLEBOX)) {
            
            i3NameStackTF.setVisible(true);
            i3NameStackLB.setVisible(true);
        }
    }


}