/*
 * Created on Oct 22, 2005
 */
package proxyconfig.i3;

import proxyconfig.*;
import proxyconfig.certificate.*;
import proxyconfig.mynames.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.border.*;

import proxyconfig.HelpButton;
import proxyconfig.InputChangeListener;
import proxyconfig.ProxyConfig;
import proxyconfig.ProxyConfigData;
import proxyconfig.ProxyConfigFileReader;
import proxyconfig.ocd.*;
import proxyconfig.mynames.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.File;
import java.io.IOException;
import java.net.*;

/**
 * @author danjo
 */
public class NATServerWizardDialog extends WizardDialog implements ActionListener, DocumentListener {

    public InputChangeListener changeListener = new InputChangeListener(); 

    /**
     * The id of the i3 overlay to be used by this wizard
     */
    public static String WIZ_I3_ID = "pli3";
    
    ProxyConfig proxyConfig;

    /** This is set to true if the cancel button is pressed. */
    private boolean cancelFlag = false;
   
    private JTextField nameTF;
    private JEditorPane nameDetailsPane;

    private I3TriggerLimitPanel triggerRateLimitPanel;
    
    private JEditorPane securityEP;
    
    private JPanel namePanel;
    private JPanel securityPanel;
    
    private boolean certificateSetFlag = false;
    private String nameCertificateSet = null;
    private String certFileName = null;
    private String privKeyFileName = null;
    
    private int currentStep = 1;
 
    private JPanel resourcePanel;
    
    private JButton nextBT, prevBT;
    
    private JButton certWizBT;
    
    /** The MyName object being processed by this wizard instance. */
    private MyName currName;
    private boolean newNameFlag = false;
    
    private ArrayList existingNamesList = null;
   

    private JRadioButton udpRB, tcpRB;
    private JPanel connectionPanel;


    public NATServerWizardDialog(ProxyConfig proxyConfig) {
        
        super(proxyConfig, "Server Behind NAT Wizard");

        
        this.proxyConfig = proxyConfig;
        
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        
    	int xInsets = 200;
    	int yInsets = 225;
    	
    	//Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	Dimension screenSize = new Dimension(1024,768);
    	
    	setResizable(false);
    	
    	setBounds(xInsets, yInsets, screenSize.width - xInsets*2, screenSize.height - yInsets * 2);
        
        Container contents = getContentPane();
        
        contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
        contents.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
        JPanel tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tmpPanel);
        
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));

        JEditorPane tmpLabel = new JEditorPane("text/html",
                 	"<font size=\"7\" color=\"blue\"><b>Server behind NAT Wizard</b></font");
        tmpLabel.setEditable(false);
        tmpLabel.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(tmpLabel);
        tmpLabel.setMaximumSize(new Dimension(200,50));

        tmpPanel.add( (Box.createHorizontalGlue()));
            
        contents.add(Box.createRigidArea(new Dimension(20,20)));
        
        // main panel start
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout (new BoxLayout (mainPanel, BoxLayout.Y_AXIS));
        contents.add (mainPanel);

        // name panel - start
        namePanel = new JPanel ();
        namePanel.setLayout (new BoxLayout (namePanel, BoxLayout.Y_AXIS));
        namePanel.setBorder (BorderFactory.createTitledBorder("STEP 1 : Name your machine"));
        mainPanel.add (namePanel);
	
        tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        namePanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));

        tmpPanel.add( (new JLabel ("Choose an i3 name for your machine:")));
       
        tmpPanel.add (Box.createRigidArea (new Dimension(5,5)));
        
    	nameTF = new JTextField(30);
    	nameTF.setMaximumSize (new Dimension(200,25));
    	nameTF.getDocument().addDocumentListener(this);
        nameTF.getDocument().addDocumentListener(changeListener);
        
    	tmpPanel.add (nameTF);
		tmpPanel.add (new HelpButton ("helpsnippets/NATServerWizardDialog_SetName.html", null, "Help"));

		tmpPanel.add(Box.createHorizontalGlue());
       
    	tmpPanel = new JPanel();
		tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
		namePanel.add (tmpPanel);
		tmpPanel.add (Box.createHorizontalGlue());
	
		nameDetailsPane = new JEditorPane ("text/html", "");
		nameDetailsPane.setBackground (ProxyConfig.BACKGROUND_COLOR);
		nameDetailsPane.setEditable (false);
		updateNameDetailsPane();
	
		tmpPanel.add (nameDetailsPane);	
		tmpPanel.add (Box.createHorizontalGlue());

		// namepanel end
	

		// security panel - start
        securityPanel = new JPanel ();
        securityPanel.setLayout (new BoxLayout (securityPanel, BoxLayout.Y_AXIS));
        securityPanel.setBorder (BorderFactory.createTitledBorder("STEP 2 : Secure your server's name"));
        mainPanel.add (securityPanel);

    
        tmpPanel = new JPanel();
		tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
		securityPanel.add (tmpPanel);
		tmpPanel.add (Box.createHorizontalGlue());
	
		securityEP = new JEditorPane ("text/html", "");
		securityEP.setBackground (ProxyConfig.BACKGROUND_COLOR);
		securityEP.setEditable (false);
		securityEP.setPreferredSize ( new Dimension (900,150));
		securityEP.setMaximumSize ( new Dimension (900,150));
		tmpPanel.add (securityEP);
		tmpPanel.add (Box.createHorizontalGlue());
		
		
        tmpPanel = new JPanel();
		tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
		securityPanel.add (tmpPanel);

		certWizBT = new JButton ("Secure It");
		certWizBT.setActionCommand ("SECURE_IT");
		certWizBT.addActionListener(this);

		tmpPanel.add (Box.createHorizontalGlue());
		tmpPanel.add (certWizBT);
		tmpPanel.add (new HelpButton ("helpsnippets/NATServerWizardDialog_Security.html", null, "Help"));
		tmpPanel.add (Box.createHorizontalGlue());

        // security panel - end
		
	// connection protocol panel - start	
	connectionPanel = new JPanel();
        connectionPanel.setLayout (new BoxLayout (connectionPanel, BoxLayout.Y_AXIS));
        connectionPanel.setBorder (BorderFactory.createTitledBorder("STEP 4 : Choose Connection Protocol"));
        mainPanel.add (connectionPanel);

	tmpPanel = new JPanel();
	tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
	tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
	connectionPanel.add (tmpPanel);
        tmpPanel.add( (new JLabel ("Do you want to contact the i3 servers using TCP or UDP?")));
	tmpPanel.add (new HelpButton ("helpsnippets/NATServerWizardDialog_Connection.html", null, "Help"));
	tmpPanel.add (Box.createHorizontalGlue());
	
	tmpPanel = new JPanel();
	tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
	tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
	connectionPanel.add (tmpPanel);
	ButtonGroup protocolBTGroup = new ButtonGroup();
	udpRB = new JRadioButton ("UDP");
	udpRB.addChangeListener (changeListener);
	protocolBTGroup.add (udpRB);
	tmpPanel.add (udpRB);
	tmpPanel.add (Box.createHorizontalGlue());

	tmpPanel = new JPanel();
	tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
	tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
	connectionPanel.add (tmpPanel);
	tcpRB = new JRadioButton ("TCP");
	tcpRB.setSelected (true);
	tcpRB.addChangeListener (changeListener);
	protocolBTGroup.add (tcpRB);
	tmpPanel.add (tcpRB);
	tmpPanel.add (Box.createRigidArea (new Dimension (5,5)));
	tmpPanel.add (new JLabel ("<html><i>(Select TCP if your firewall blocks incoming UDP packets)</i></html>"));
	tmpPanel.add (Box.createHorizontalGlue());

	tmpPanel = new JPanel();
	tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
	tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
	connectionPanel.add (tmpPanel);
        tmpPanel.add( (new JLabel ("This selection affects ALL the names of your machines.")));
	tmpPanel.add (Box.createHorizontalGlue());
	
	// connection protocol panel - end
        
        // resourcePanel - start
        resourcePanel = new JPanel ();
        resourcePanel.setLayout (new BoxLayout (resourcePanel, BoxLayout.Y_AXIS));
        resourcePanel.setBorder (BorderFactory.createTitledBorder("STEP 3 : Manage the resources of your server"));
        mainPanel.add (resourcePanel);

        tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
	tmpPanel.setLayout (new BoxLayout (tmpPanel, BoxLayout.X_AXIS));
        resourcePanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));
        tmpPanel.add( (new JLabel ("Limit the incoming bandwidth and number of connections:")));
		tmpPanel.add (new HelpButton ("helpsnippets/NATServerWizardDialog_ResourceManagement.html", null, "Help"));
        tmpPanel.add(Box.createHorizontalGlue());
        
        tmpPanel.add (Box.createRigidArea (new Dimension(5,5)));

        triggerRateLimitPanel = new I3TriggerLimitPanel(null, changeListener);
        resourcePanel.add (triggerRateLimitPanel);
        // resourcePanel - end

        mainPanel.add (Box.createVerticalGlue());
        
        // main panel end
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);

        contents.add(Box.createRigidArea(new Dimension(5,5)));
        contents.add(buttonPanel);

        nextBT = new JButton ("Next Step");
        nextBT.setActionCommand( ("NEXT_STEP"));
        nextBT.addActionListener (this);

        prevBT = new JButton  ("Previous Step");
        prevBT.setActionCommand ("PREV_STEP");
        prevBT.addActionListener (this);
     
        JButton cancelBT = new JButton ("Cancel");
        cancelBT.setActionCommand ("CANCEL");
        cancelBT.addActionListener (this);
        
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(prevBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(nextBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(cancelBT);
        buttonPanel.add(Box.createHorizontalGlue());

        
        mainPanel.add(Box.createVerticalGlue());

	loadOCDDetails();
	
        loadExistingNames();
        changeListener.setEnabled(true);
        currentStep = 1;
        makeStepVisible ();

    }
    
    private void loadOCDDetails() {
	I3OCD tmpOCD = (I3OCD) proxyConfig.proxyConfigData.getOverlayById (WIZ_I3_ID);
	if (tmpOCD != null) {
		if (tmpOCD.i3UseTCP) {
			tcpRB.setSelected(true);
			udpRB.setSelected (false);
		} else {
			tcpRB.setSelected (false);
			udpRB.setSelected (true);
		}	
	}
    }

    /** Offer the user a choice of editing existing names if any.
     *  If no names already exist or the user chooses to start a new name,
     *  we create a new MyName object for this wizard instance.
     */
    private void loadExistingNames() {
        Iterator itr = proxyConfig.proxyConfigData.myNameList.iterator();
        existingNamesList = new ArrayList();
        while (itr.hasNext()) {
            MyName tmpName = (MyName) itr.next();

            if (tmpName.ocdId.equals (WIZ_I3_ID)) {

                existingNamesList.add(tmpName);
            }
        }
        
        if (existingNamesList.size() > 0) {
            ExistingNameCheckDialog chkDialog = new ExistingNameCheckDialog (proxyConfig, existingNamesList);
            chkDialog.show();
            if (chkDialog.createNewFlag) {
                initNewName();
            } else {
                this.currName = chkDialog.getSelectedName();
            }
        } else {
            initNewName();
        }
        setGUIFromCurrMyName ();
    }
    
    private void setGUIFromCurrMyName () {
        nameTF.setText (this.currName.name);
        triggerRateLimitPanel.setMyName (this.currName);
    }

    private void initNewName () {
        
        this.newNameFlag = true;
        // Construct the MyName object
        this.currName = new MyName();
        this.currName.active = true;
        this.currName.ocd = proxyConfig.proxyConfigData.getOverlayById(WIZ_I3_ID);
        this.currName.ocdId = this.currName.ocd.id;
        this.currName.ocdSpecificProperties.put (I3OCD.I3_TRIGGER_TYPE, "public");
        
    	InetAddress localAddress;
    	try {
        	localAddress = InetAddress.getLocalHost ();
         	String nameSuggestion = localAddress.getHostName();
         	nameSuggestion = nameSuggestion + "." + WIZ_I3_ID;
            
         	this.currName.name = nameSuggestion;        
    	} catch (UnknownHostException e) {
        	// TODO Auto-generated catch block
        	System.out.println ("i3 NAT Server Wizard: Unable to construct name suggestion.\n");
        	e.printStackTrace();
    	}

    }
    
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();
        
        if (cmd.equals("NEXT_STEP")) {
            handleNextStep();
            return;
   
        } else if (cmd.equals ("PREV_STEP")) {
            
            currentStep --;
            makeStepVisible ();
            
        } else if (cmd.equals ("SECURE_IT")) {
            handleSecureIt ();
            
        } else if (cmd.equals("CANCEL")) {
            
            if (changeListener.hasChanged()) {
                
                int response = JOptionPane.showConfirmDialog(null, 
                       "Are you sure you want to quit without saving the configuration?", "Warning", JOptionPane.YES_NO_OPTION);
                if (response == JOptionPane.NO_OPTION) {
                    return;
                }
            }
           
	    this.cancelFlag = true; 
            setVisible(false);

            // We are done
            //System.exit (0);
        }
        
    }

    private void handleSecureIt () {
        String tmpName = nameTF.getText().trim();
   	    CertificateWizard certWiz = new CertificateWizard(proxyConfig, tmpName);
        if (certWiz.finishFlag) {
            
            if (certWiz.changeListener.hasChanged()) {
                proxyConfig.changeListener.setChanged(true);
                // Make sure that certificates were create correctly.
                if (!certificateExists(tmpName)) {
                    JOptionPane.showMessageDialog(
                            proxyConfig, 
                            "There was some error in certificate allocation.\n" +
                            "Your allocated private key and certificate files could not be found.",
                            "Error", 
                            JOptionPane.ERROR_MESSAGE);                    
                }
            }
        }
        updateSecurityPanel();
    }
    private void handleNextStep () {
        switch (currentStep) {
        
        	case 1: // Specify name
        	{
        	    // Check if the name is of the correct form.
        	    String tmpName = nameTF.getText();
        	    if (tmpName == null || tmpName.trim().equals("")) {
                    JOptionPane.showMessageDialog(
                            proxyConfig, 
                            "Please specify a name for your machine.", 
                            "Error", 
                            JOptionPane.ERROR_MESSAGE);
                    ProxyConfig.beep();
                    return;
        	    }
        	    
        	    if ( !tmpName.endsWith("." + WIZ_I3_ID)) {
                    JOptionPane.showMessageDialog(
                            proxyConfig, 
                            "Please specify a name for your machine that ends with . " + WIZ_I3_ID + ".", 
                            "Error", 
                            JOptionPane.ERROR_MESSAGE);
                    ProxyConfig.beep();
                    return;
        	    }
        	    
        	    if ( newNameFlag && nameAlreadyExists(tmpName)) {
        	        
                    JOptionPane.showMessageDialog(
                            proxyConfig, 
                            "The specified name already exists.  You cannot add it again.\n" +
                            "Restart the wizard and choose edit existing name if you want\n" +
                            "to modify the details of this name.", 
                            "Error", 
                            JOptionPane.ERROR_MESSAGE);
                    ProxyConfig.beep();
                    return;        	        
        	    }
        	    
        	    currentStep ++;
        	    makeStepVisible();
        	    return;
        	}
        	
        	case 2: //security
        	{    
        	    
        	    currentStep++;
        	    makeStepVisible();
        	    return;
        	}

		case 3: // resource management
		{
			currentStep++;
			makeStepVisible();
			return;
		}
        	
        	case 4: // tcp or udp
        	{
                if (checkFields()) {
                    
                      
                    this.currName.name = nameTF.getText();
                    triggerRateLimitPanel.update (this.currName);
                    
                    if (newNameFlag) {
                        proxyConfig.proxyConfigData.addMyName(this.currName);
                    }
                    fillCertificateDetails(this.currName);
		
		    I3OCD tmpOCD = (I3OCD) proxyConfig.proxyConfigData.getOverlayById (WIZ_I3_ID);
		    if (tmpOCD != null) {
		    	tmpOCD.i3UseTCP = tcpRB.isSelected();
		    }

                    proxyConfig.changeListener.setChanged(true);
                    proxyConfig.saveConfigFile();
                      
                    showConfigSuccessMsg();
                      
                    setVisible (false);
                    // We are done
                    //System.exit (0);
                  }        	    
        	}        
        }
    }
    
    private boolean nameAlreadyExists(String testName) {
        Iterator itr = existingNamesList.iterator();
        while (itr.hasNext()) {
            MyName tmpName = (MyName) itr.next();
            if (tmpName.name.equals (testName)) {
                return true;
            }
        }
        return false;
    }
    
    public boolean checkFields () {
        
        String msg = "";
        
        if (nameTF.getText().trim().equals("")) {
            
            msg += "Please specify a name for the web server.\n";
        }
        
        msg += triggerRateLimitPanel.checkFields();
        
        if (! msg.equals ("")) {
            JOptionPane.showMessageDialog(proxyConfig, msg, "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        } else {
            return true;
        }
        

    }
     
    
    public boolean hasChanged() {
        
        return changeListener.hasChanged();
    }
    
    public boolean isOK() {
        
        return true;
    }


    /* (non-Javadoc)
     * @see proxyconfig.WizardDialog#update()
     */
    public void update() {
        // TODO Auto-generated method stub
        
    }

    private void updateNameDetailsPane () {
        if (nameDetailsPane == null || nameTF == null) {
            return;
        }
        if (nameTF.getText() != null) {
            String name_OCALA_clients = nameTF.getText();
            String name_native_clients = nameTF.getText() + ".ocalaproxy.net:8040";
            nameDetailsPane.setText (
                    	"<font size=\"3\">Clients which run OCALA can address your machine as <b>" + name_OCALA_clients + "</b>." +
                    	"<br>" +
                    	"Clients which do not run OCALA can address your machine as <b>" + name_native_clients + "</b> <i>(HTTP access only)</i>" +
                    	"</font>"
                    	
                    );
        } else {
            nameDetailsPane.setText("");
        }
    }

    public void updateSecurityPanel () {
        
        if (nameTF == null || nameTF.getText() == null || nameTF.getText().trim().equals("")) {
            // This condition will probably NEVER be triggered.
            securityEP.setText("You haven't selected a valid name that can be secured.");
            certWizBT.setEnabled (false);
            return;
        }
        
        String tmpName = nameTF.getText();
        if (certificateExists (tmpName)) {
            
            securityEP.setText (
                   "The name <b>" + tmpName + "</b> is <font color=\"green\">SECURED</font>." +
                   		"Other persons can now contact you in a secure fashion if they choose to."
                  );
            certWizBT.setEnabled (false);
            return;
        
        } else {
            // certificate does not exist.
            securityEP.setText (
                    "The name <b>" + tmpName + "</b> is <font color=\"red\">NOT SECURED</font>."
                   );
            certWizBT.setEnabled (true);

        }
    }
    
    
    private boolean certificateExists (String tmpName) {
        File newCertFile = new File ("config_files/certificates/" + tmpName + ".cert");
        File newPrivKeyFile = new File ("config_files/certificates/" + tmpName + ".privkey");
        File newPubKeyFile = new File ("config_files/certificates/" + tmpName + ".pubkey");
        if (newCertFile.exists() && newPrivKeyFile.exists() && newPubKeyFile.exists()) {
            return true;
        } else {
            return false;
        }
    }
    
    private boolean fillCertificateDetails (MyName myNameToBeFilled) {
        if (certificateExists (myNameToBeFilled.name)) {
            myNameToBeFilled.privKeyFileName = "config_files/certificates/" + myNameToBeFilled.name + ".privkey";
            myNameToBeFilled.certFileName = "config_files/certificates/" + myNameToBeFilled.name + ".cert";
            myNameToBeFilled.certOKFlag = MyName.CERT_VALID;
            return true;
        } else {
            myNameToBeFilled.privKeyFileName = null;
            myNameToBeFilled.certFileName = null;
            myNameToBeFilled.certOKFlag = MyName.CERT_NOT_FOUND;
            return false;
        }
    }

    /* (non-Javadoc)
     * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
     */
    public void changedUpdate(DocumentEvent e) {
        // TODO Auto-generated method stub
        updateNameDetailsPane();
        
    }


    /* (non-Javadoc)
     * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
     */
    public void insertUpdate(DocumentEvent e) {
        // TODO Auto-generated method stub
        updateNameDetailsPane();        
    }


    /* (non-Javadoc)
     * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
     */
    public void removeUpdate(DocumentEvent e) {
        // TODO Auto-generated method stub
        updateNameDetailsPane();
    }

    private void makeStepVisible () {
        if (currentStep == 1) {
            namePanel.setVisible (true);
            securityPanel.setVisible (false);
            resourcePanel.setVisible (false);
	    connectionPanel.setVisible(false);
            prevBT.setEnabled (false);
            nextBT.setEnabled (true);
            nextBT.setText ("Next Step");
        } else if (currentStep == 2) {
            namePanel.setVisible (false);
            updateSecurityPanel();
            securityPanel.setVisible (true);
	    connectionPanel.setVisible(false);
            resourcePanel.setVisible (false);
            prevBT.setEnabled (true);
            nextBT.setEnabled (true);
            nextBT.setText ("Next Step");
            
        } else if (currentStep == 3) {
            namePanel.setVisible (false);
            securityPanel.setVisible (false);
	    connectionPanel.setVisible(false);
            resourcePanel.setVisible (true);
            prevBT.setEnabled (true);
            nextBT.setEnabled (true);
            nextBT.setText ("Next Step");

        } else if (currentStep == 4) {
            namePanel.setVisible (false);
            securityPanel.setVisible (false);
	    connectionPanel.setVisible(true);
            resourcePanel.setVisible (false);
            prevBT.setEnabled (true);
            nextBT.setEnabled (true);
            nextBT.setText ("Finish");
       
        } else {
            System.out.println ("Invalid step  "  + currentStep);
         }
    }
    
    private void showConfigSuccessMsg () {
        URL helpURL = null;
        
        String nameMsg;
        String tmpName = nameTF.getText();
        if (certificateExists(tmpName)) {
            nameMsg = "<b>" + tmpName + "</b> and secured it.";  
        } else {
            nameMsg = "<b>" + tmpName + "</b>.  " +
            "Please note that you have <font color=\"red\">NOT SECURED</font> your name."
            ;
        }
        
        String testingMsg = "<h3>Testing</h3>\n";
	
	String iconMsg = "";
        if (proxyConfig.isWindows()) {
            testingMsg += "<p>You can test your setup by pinging yourself.  Open a command prompt " +
            				"and execute <b>ping " + tmpName + "</b>.  You should see valid ping responses." +
            				"<p><b>NOTE 1:</b> Ping must be enabled on your machine for this test to succeed. " +
            				"Most Windows installations have ping turned off by default.  To enable ping, " +
            				"please follow the instructions at <a href=\"http://nic.phys.ethz.ch/readme/164\">http://nic.phys.ethz.ch/readme/164</a> " +
            				"or search the web for alternate instructions.";

	    //iconMsg = "<p>Look out for the <img src=\"helpsnippets\\images\\ProxyMonitor_icon.gif\"> and <img src=\"http://img.com.com/b.gif\"> icons in the Windows System Tray to make sure OCALA Proxy is running and <i>i3</i> is active.";
	    
        } else {
            testingMsg += "<p>You can test your setup by pinging yourself.  Open a shell and " +
            			 "execute <b>ping " + tmpName + "</b>. You should see valid ping responses." +
            			 "<p><b>NOTE 1:</b> Your firewall must be configured to not block pings for this test to succeed.";
            
        }
	if (udpRB.isSelected()) {
        	testingMsg += "<p><b>NOTE 2:</b> This test will fail if your firewall blocks incoming UDP packets on the port " +
        				"<i>i3</i> listens on (58748 by default).  Please add a rule to your fireall configuration that allows these packets in."
        				;
	}
        String msgStr = "<html><body" +
        				"<b><h2><font color=\"blue\">Congratulations!</font></h2></b>" +
        				"<p>You have successfuly set up the name " + nameMsg +
        				"Your machine is all set to receive connections once OCALA proxy has been started.<br><br>" +
        				"Please note that OCALA does not start up any server (eg: httpd or sshd) by itself.  " +
        				"So don't forget to start up the services you want to be remotely accessible. <br>" +
        				iconMsg +
					testingMsg + 
        				"</body></html>";
        JEditorPane mainEditorPane = new JEditorPane ("text/html" ,msgStr);
        mainEditorPane.setEditable(false); 
	
        JScrollPane scrollPane = new JScrollPane (mainEditorPane);
	    scrollPane.setPreferredSize (new Dimension (400,600));
	scrollPane.getViewport().setViewPosition (new Point (0,0));
	JOptionPane.showMessageDialog ( proxyConfig, 
					scrollPane,
					"Configuration Successful",
					JOptionPane.INFORMATION_MESSAGE
				);
        
    }

    public boolean isRestartRequired() {

	    if (cancelFlag) {
		    return false;
	    }
	    if (!changeListener.hasChanged()) {
		    return false;
	    }
	    return true;
    }
}
