/*
 * Created on May 17, 2005
 */
package proxyconfig.outboundconnections;

import javax.swing.*;


import proxyconfig.ProxyConfig;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;

/**
 * @author danjo
 */ 
public class HopPanel extends JPanel implements ActionListener {

    DefaultListModel listModel;
    JList hopList;
    ProxyConfig proxyConfig;
    
    HopPanel (ProxyConfig proxyConfig) {
        
        this.proxyConfig = proxyConfig;
        
        setLayout (new BorderLayout());
        
        listModel = new DefaultListModel();
        hopList = new JList (listModel);
        
        add(new JScrollPane (hopList), BorderLayout.CENTER);
        
        JPanel buttonPanel = new JPanel ();
        buttonPanel.setBackground (proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        buttonPanel.setLayout (new BoxLayout (buttonPanel, BoxLayout.Y_AXIS));
        buttonPanel.add(Box.createVerticalGlue());
        
        
        add(buttonPanel, BorderLayout.EAST);
        
        URL addBTURL = getClass().getResource("/newRecBT.gif");
        JButton addBT = new JButton(new ImageIcon(Toolkit.getDefaultToolkit().getImage(addBTURL)));
        addBT.setToolTipText("Add new hop");
        addBT.setActionCommand ("ADD_HOP");
        addBT.addActionListener (this);
        buttonPanel.add (addBT);
        
        URL delBTURL = getClass().getResource("/delRecBT.gif");
        JButton delBT = new JButton(new ImageIcon(Toolkit.getDefaultToolkit().getImage(delBTURL)));
        delBT.setToolTipText("Delete selected hop");
        delBT.setActionCommand ("DEL_HOP");
        delBT.addActionListener (this);
        buttonPanel.add (delBT);

        
        URL upBTURL = getClass().getResource("/upBT.gif");
        JButton upBT = new JButton(new ImageIcon(Toolkit.getDefaultToolkit().getImage(upBTURL)));
        upBT.setActionCommand ("MOVE_UP");
        upBT.setToolTipText("Move selected hop up");
        upBT.addActionListener (this);
        buttonPanel.add (upBT);
        
        URL downBTURL = getClass().getResource("/downBT.gif");
        JButton downBT = new JButton(new ImageIcon(Toolkit.getDefaultToolkit().getImage(downBTURL)));
        downBT.setActionCommand ("MOVE_DOWN");
        downBT.setToolTipText("Move selected hop down");
        downBT.addActionListener (this);
        buttonPanel.add (downBT);
        
        buttonPanel.add(Box.createVerticalGlue());
        
    }
    
    public void actionPerformed (ActionEvent ae) {
        String cmd  = ae.getActionCommand();
        
        if (cmd.equals ("ADD_HOP")) {
            HopInfoEditPanel hopEditPanel = new HopInfoEditPanel(null, proxyConfig);
            
            do {
                ProxyConfig.changeOptionPaneBackground(Color.WHITE);
            
                int retVal = JOptionPane.showConfirmDialog(
                    				null, hopEditPanel, "Enter New Hop Details", 
                    				JOptionPane.OK_CANCEL_OPTION);
                if (retVal == JOptionPane.OK_OPTION) {
                
                    HopInfo currRecord = new HopInfo();

                    if (hopEditPanel.updateRecord(currRecord)) {
                        proxyConfig.proxyConfigData.outboundList.add(currRecord);
                        listModel.addElement(currRecord);
                        proxyConfig.changeListener.setChanged(true);
                        break;
                    } else {
                        
                        String tmpAvailSuffixes = proxyConfig.proxyConfigData.availableOCDSuffixes();
           
                        int retVal2 = JOptionPane.showConfirmDialog(proxyConfig, 
                                "The specified hop name does not belong to any of the available overlays.\n" +
                                "The hop name suffix should end with " + tmpAvailSuffixes + ".\n" +
                                "Retry?",
                                "Invalid hop name",
                                JOptionPane.YES_NO_OPTION
                               );
                        if (retVal2 == JOptionPane.NO_OPTION) {
                            break;
                        }
                    }
                    
                } else {
                    break;
                }
            } while (true);
            
            ProxyConfig.restoreOptionPaneBackground();


            
        } else if (cmd.equals ("DEL_HOP")) {
            HopInfo currRecord = (HopInfo) hopList.getSelectedValue();
            if (currRecord != null) {
                int retVal = JOptionPane.showConfirmDialog(null, 
                        "Are you sure you want to delete the Hop " + currRecord + " ?", 
                        "Confirm Delete", JOptionPane.YES_NO_OPTION);
                if (retVal == JOptionPane.YES_OPTION) {
                    listModel.removeElement(currRecord);
                    proxyConfig.changeListener.setChanged(true);

                }
                
            } else {
                ProxyConfig.beep();
            }

        } else if (cmd.equals ("MOVE_UP")) {
            
            HopInfo currHop = (HopInfo) hopList.getSelectedValue();
            if (currHop == null) {
                ProxyConfig.beep();
                return;
            }
            
            int currIndex = hopList.getSelectedIndex();
            
            currIndex --;
            if (currIndex < 0) {
                ProxyConfig.beep();
                return;
            }
            
            listModel.removeElement(currHop);
            listModel.add(currIndex, currHop);
            hopList.setSelectedIndex(currIndex);
            
            proxyConfig.changeListener.setChanged(true);
            
        } else if (cmd.equals ("MOVE_DOWN")) {

            HopInfo currHop = (HopInfo) hopList.getSelectedValue();
            if (currHop == null) {
                ProxyConfig.beep();
                return;
            }
            
            int currIndex = hopList.getSelectedIndex();
            
            currIndex ++;
            if (currIndex > listModel.size() - 1 ) {
                ProxyConfig.beep();
                return;
            }
            
            listModel.removeElement(currHop);
            listModel.add(currIndex, currHop);
            hopList.setSelectedIndex(currIndex);
            
            proxyConfig.changeListener.setChanged(true);
            
        }
    }
    
    void addHop (HopInfo hop) {
        listModel.addElement(hop);
    }
}
