/*
 * Created on Apr 12, 2006
 */
package proxyconfig.rsptunnelocd;

import java.awt.Container;
import java.awt.Dimension;
import java.net.*;
import java.util.*;

import javax.swing.*;

import proxyconfig.*;
import proxyconfig.mynames.MyName;
import proxyconfig.outboundconnections.*;
import java.awt.event.*;
import javax.swing.event.*;

import java.awt.*;
import java.io.*;

/**
 * @author danjo
 */
public class SecureConnectionWizardDialog extends WizardDialog implements ActionListener{
    public InputChangeListener changeListener = new InputChangeListener(); 

    ProxyConfig proxyConfig;

    JTabbedPane tabPane;
    
    JButton enableBT;
    
    JTextField nameTF;
    
  
    JEditorPane statusLB;
    
    JLabel gatewayDownLB;
    
    JRadioButton gatewayDown_dropRB, gatewayDown_nodropRB;

    static final String GATEWAY_LIST_FILE = "SecureConnection.Gateways.txt";
    
    boolean secureConnectionEnabledFlag;
    
    URLPatternTableModel urlPatternTableModel;
    JTable urlPatternTable;
    JButton addURLPatternBT, editURLPatternBT, delURLPatternBT;
    
    private JPanel urlPanel;
    private JEditorPane urlTableLB;
    
    static final String RSP_TUNNEL_OCD_ID = "rspTunnelOCD";
    static final String LSIP_OCD_ID = "ipv4";
    
    public SecureConnectionWizardDialog(ProxyConfig proxyConfig) {
        
        super(proxyConfig, "Secure Connection Configuration Wizard");

        
        this.proxyConfig = proxyConfig;
        
        changeListener.setEnabled(false);
       
        secureConnectionEnabledFlag = checkIfOCDActive();
        
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        
    	int xInsets = 200;
    	int yInsets =100; 
    	
    	//Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    	Dimension screenSize = new Dimension(1024,768);
    	
    	setResizable(false);
    	
    	setBounds(xInsets, yInsets, screenSize.width - xInsets*2, screenSize.height - yInsets * 2);
        
        Container contents = getContentPane();
        
        contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
        contents.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
        JPanel tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        contents.add(tmpPanel);
        
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));

        JEditorPane tmpLabel = new JEditorPane("text/html",
                 	"<font size=\"6\" color=\"blue\"><b>Configure OCALA Secure Connection</b></font");
        tmpLabel.setEditable(false);
        tmpLabel.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(tmpLabel);
        tmpLabel.setMaximumSize(new Dimension(200,70));

        tmpPanel.add( (Box.createHorizontalGlue()));
            
        contents.add(Box.createRigidArea(new Dimension(20,20)));
        
        // main panel start
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout (new BoxLayout (mainPanel, BoxLayout.Y_AXIS));
        contents.add (mainPanel);

        // step 1 start
        tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        mainPanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));
        
        tmpLabel = new JEditorPane("text/html",
                 	"<b>OCALA Secure Connection is </b>");
        tmpLabel.setEditable(false);
        tmpLabel.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(tmpLabel);
        tmpLabel.setMaximumSize(new Dimension(200,50));

        statusLB = new JEditorPane("text/html",
     									getStatusString()
     								);
        statusLB.setEditable(false);
        statusLB.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(statusLB);
        statusLB.setMaximumSize(new Dimension(100,50));

        tmpPanel.add (Box.createRigidArea(new Dimension (5,5)));
        
        enableBT = new JButton ();
        if (secureConnectionEnabledFlag) {
            enableBT.setText("Disable");
            enableBT.setActionCommand ("DISABLE_SECURE_CONNECTION");
        } else {
            enableBT.setText("Enable");
            enableBT.setActionCommand ("ENABLE_SECURE_CONNECTION");
        }
        enableBT.addActionListener(this);
        tmpPanel.add (enableBT);
		tmpPanel.add (new HelpButton ("helpsnippets/SecureConnectionWizardDialog_EnableDisable.html", null, "Help"));
        tmpPanel.add( (Box.createHorizontalGlue()));
        
        
        
        mainPanel.add (Box.createRigidArea (new Dimension(50,30)));

        // List of URL/URL Suffixes
        
        urlPanel = new JPanel();
        urlPanel.setLayout (new BoxLayout (urlPanel, BoxLayout.Y_AXIS));
        mainPanel.add (urlPanel);
        
        tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        urlPanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));
        
        urlTableLB = new JEditorPane("text/html",
                 	"<b>Connections to the following URLs/URL suffixes (eg: *.berkeley.edu) will be secured.<br>" +
                 	"Please choose the OCALA Secure Connection gateway for each URL/URL suffix." +
                 	"</b>"
                 );
        
        urlTableLB.setEditable(false);
        urlTableLB.setBackground (ProxyConfig.BACKGROUND_COLOR);
        tmpPanel.add(urlTableLB);
        urlTableLB.setMaximumSize(new Dimension(800,60));
		tmpPanel.add (new HelpButton ("helpsnippets/SecureConnectionWizardDialog_ChooseGateway.html", null, "Help"));
		tmpPanel.add (Box.createHorizontalGlue());
        String[] columns = {"URL/URL Suffix", "Gateway"};
        
        urlPatternTableModel = new URLPatternTableModel (columns);
                
        TableSorter tableSorter = new TableSorter(urlPatternTableModel);
        urlPatternTable = new JTable(tableSorter);
		tableSorter.addMouseListenerToHeaderInTable(urlPatternTable);

        urlPatternTable.setBackground(proxyconfig.ProxyConfig.WHITE_COLOR);
        urlPatternTable.setPreferredSize(new Dimension(500, 200));
        
        urlPatternTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
       
        JScrollPane scrollPane = new JScrollPane(
                	urlPatternTable,
                	JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                	JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED
                	);
        scrollPane.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        scrollPane.setPreferredSize(new Dimension(500, 200));

        urlPanel.add(scrollPane);

        LinkedList urlPatternTableButtonList = new LinkedList();
        
        addURLPatternBT = new JButton ("Add URL Pattern");
        addURLPatternBT.setActionCommand ("ADD_URL_PATTERN");
        addURLPatternBT.addActionListener (this);
        urlPatternTableButtonList.add(addURLPatternBT);
        
        editURLPatternBT = new JButton ("Edit URL Pattern");
        editURLPatternBT.setActionCommand ("EDIT_URL_PATTERN");
        editURLPatternBT.addActionListener (this);
        urlPatternTableButtonList.add(editURLPatternBT);
        
        delURLPatternBT = new JButton ("Delete URL Pattern");
        delURLPatternBT.setActionCommand ("DEL_URL_PATTERN");
        delURLPatternBT.addActionListener (this);
        urlPatternTableButtonList.add (delURLPatternBT);
        
        CenteredButtonPanel urlPatternTablePanel = new CenteredButtonPanel (
                										urlPatternTableButtonList, 
                										new Dimension (10,10)
                								);
        urlPanel.add(urlPatternTablePanel);
        
        loadURLPatterns();
        
        // main panel end
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);

        contents.add(Box.createRigidArea(new Dimension(5,5)));
        contents.add(buttonPanel);

        JButton cancelBT = new JButton("CANCEL");
        cancelBT.setActionCommand("CANCEL");
        cancelBT.addActionListener (this);

        JButton okBT = new JButton ("OK");
        okBT.setActionCommand ("OK");
        okBT.addActionListener (this);
        
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(okBT);
        buttonPanel.add(Box.createRigidArea(new Dimension(10,10)));
        buttonPanel.add(cancelBT);
        buttonPanel.add(Box.createHorizontalGlue());

        
        
        
        // Gateway Down - start
        mainPanel.add (Box.createRigidArea (new Dimension(50,20)));
        
        
        tmpPanel = new JPanel();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        mainPanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));
        
        gatewayDownLB = new JLabel ("If the chosen gateway is down :");
        tmpPanel.add (gatewayDownLB);
        tmpPanel.add (Box.createRigidArea (new Dimension (3,3)));
		tmpPanel.add (new HelpButton ("helpsnippets/SecureConnectionWizardDialog_GatewayDown.html", null, "Help"));
		tmpPanel.add (Box.createHorizontalGlue());
        
        ButtonGroup gatewayDownBTGroup = new ButtonGroup();
        
        gatewayDown_dropRB = new JRadioButton ("Drop all traffic to the secured names");
        gatewayDownBTGroup.add (gatewayDown_dropRB);
        gatewayDown_dropRB.setSelected(true);
        
        tmpPanel = new JPanel ();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        mainPanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));
        tmpPanel.add (Box.createRigidArea (new Dimension (100,10)));
        tmpPanel.add (gatewayDown_dropRB);
        tmpPanel.add (Box.createHorizontalGlue());
        
        gatewayDown_nodropRB = new JRadioButton ("All traffic to the secured names even if INSECURE!");
        gatewayDownBTGroup.add(gatewayDown_nodropRB);
        tmpPanel = new JPanel ();
        tmpPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        mainPanel.add(tmpPanel);
        tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.X_AXIS));
        tmpPanel.add (Box.createRigidArea (new Dimension (100,10)));
        tmpPanel.add (gatewayDown_nodropRB);
        tmpPanel.add (Box.createHorizontalGlue());
        
        RSPTunnelOCD ocd = (RSPTunnelOCD) proxyConfig.proxyConfigData.getOverlayById(RSP_TUNNEL_OCD_ID);
        if (ocd != null && ocd.gatewayDownPolicy != null) {
            if (ocd.gatewayDownPolicy.equals ("drop")) {
                gatewayDown_dropRB.setSelected(true);
            } else {
                gatewayDown_nodropRB.setSelected(true);
            }
        }
        
        

        mainPanel.add(Box.createVerticalGlue());
        
        enableFields(secureConnectionEnabledFlag);
        
        changeListener.setEnabled(true);
    }
    
    
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();
        
        if (cmd.equals ("ENABLE_SECURE_CONNECTION")) {
            secureConnectionEnabledFlag = true;
            enableFields (true);
            statusLB.setText(getStatusString());
            enableBT.setActionCommand( ("DISABLE_SECURE_CONNECTION"));
            enableBT.setText("Disable");
            changeListener.setChanged( true );
            enableOCD (true);
        } else if (cmd.equals ("DISABLE_SECURE_CONNECTION")) {
            secureConnectionEnabledFlag = false;
            enableFields (false);
            statusLB.setText(getStatusString());
            enableBT.setActionCommand( ("ENABLE_SECURE_CONNECTION"));
            enableBT.setText("Enable");
            changeListener.setChanged ( true );
            enableOCD (false);
        } else if (cmd.equals ("DEL_URL_PATTERN")) {
            OutboundRecord currRec = (OutboundRecord) urlPatternTableModel.getRecord (urlPatternTable.getSelectedRow());
            if (currRec != null) {
                urlPatternTableModel.deleteRecord(currRec);
                proxyConfig.proxyConfigData.delOutbound(currRec);
                changeListener.setChanged(true);
                proxyConfig.changeListener.setChanged(true);
            } else {
                ProxyConfig.beep();
            }
        } else if (cmd.equals ("ADD_URL_PATTERN")) {
           
            
            URLPatternPanel urlPatternPanel = new URLPatternPanel (urlPatternTableModel, proxyConfig, false, null);
            
            do {
                int retVal = JOptionPane.showConfirmDialog(
                		null, 
                		urlPatternPanel, 
                		"Add New URL Pattern", 
                		JOptionPane.OK_CANCEL_OPTION
                	);
                
            	if (retVal == JOptionPane.OK_OPTION) {
                	if (urlPatternPanel.checkFields()) {
                        OutboundRecord newOutboundRec = new OutboundRecord ();
                        newOutboundRec.securityProtocol = OutboundRecord.SECURITY_PROTOCOL_CUSTOM_SSL;
                        newOutboundRec.compressionAlgo = OutboundRecord.COMPRESSION_ALGO_NONE;
                        
                        HopInfo firstHop = new HopInfo();
                        firstHop.ocdId = RSP_TUNNEL_OCD_ID;
                        firstHop.ocd = proxyConfig.proxyConfigData.getOverlayById(RSP_TUNNEL_OCD_ID);
                        
                        HopInfo secondHop = new HopInfo();
                        secondHop.ocdId = LSIP_OCD_ID;
                        secondHop.ocd = proxyConfig.proxyConfigData.getOverlayById(LSIP_OCD_ID);
                        secondHop.endPointName = "";
            		secondHop.authenticateFlag = false;

                        newOutboundRec.hopList.add (firstHop);
                        newOutboundRec.hopList.add (secondHop);

                        urlPatternPanel.updateRecord (newOutboundRec);
                        
                        if (proxyConfig.proxyConfigData.addOutbound(newOutboundRec)){
                            urlPatternTableModel.addRecord(newOutboundRec);
                            changeListener.setChanged(true);
                            break;
                        } else {
                            JOptionPane.showMessageDialog(
                            		null,
                            		"The URL suffix \"" + newOutboundRec.pattern + "\" has a conflict with an existing URL pattern in \"outbound.xml\".\n" +
                            		"Please manually edit \"outbound.xml\".",
                            		"Conflicting URL suffix!",
                            		JOptionPane.ERROR_MESSAGE
                            	);
                            
                        }
                	}
            	} else {
            	    break;
            	}
            } while (true);

            
        } else if (cmd.equals ("EDIT_URL_PATTERN")) {
            
            OutboundRecord currRec = (OutboundRecord) urlPatternTableModel.getRecord (urlPatternTable.getSelectedRow());
            if (currRec == null) {
                ProxyConfig.beep();
            } else {
                URLPatternPanel urlPatternPanel = new URLPatternPanel (urlPatternTableModel, proxyConfig, true, currRec);

                do {
                    int retVal = JOptionPane.showConfirmDialog(
                 		null, 
                 		urlPatternPanel, 
                 		"Edit URL Pattern", 
                 		JOptionPane.OK_CANCEL_OPTION
                 	);
                 
                    if (retVal == JOptionPane.OK_OPTION) {
                        if (urlPatternPanel.checkFields()) {
                            urlPatternPanel.updateRecord (currRec);
                            proxyConfig.changeListener.setChanged(true);
                            changeListener.setChanged (true);
                            urlPatternTableModel.fireTableDataChanged();
                        }
                        break;
                 	} else {
                        break;
                    }
                } while (true);
            }

        } else if (cmd.equals("OK")) {
            if (checkFields()) {
              
                updateFields();
                proxyConfig.changeListener.setChanged(true);
                proxyConfig.saveConfigFile();
                
                setVisible (false);

                showConfigSuccessMsg();
                // We are done
                //System.exit (0);
            } 

            
        } else if (cmd.equals("CANCEL")) {
            
            if (changeListener.hasChanged()) {
                
                int response = JOptionPane.showConfirmDialog(null, 
                       "Are you sure you want to discard the changes made to the OCALA Secure Connection configuration?", "Warning", JOptionPane.YES_NO_OPTION);
                if (response == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            
            setVisible(false);

            // We are done
            //System.exit (0);
        }
        
    }

    
    public boolean checkFields () {
        if (secureConnectionEnabledFlag && urlPatternTableModel.getURLs() == null) {
            // no url patterns specified.
            int answer = JOptionPane.showConfirmDialog(null, 
                    "OCALA Secure Connections is enabled.  But you have not specified any URLs/URL patterns " +
                    "that will use OCALA Secure Connections.\n  This means that NO connections will be secure.\n" +
                    "Is this okay?",
                    "Warning",
                    JOptionPane.YES_NO_OPTION
                   );
            if (answer == JOptionPane.YES_OPTION) {
                return true;
            } else {
                return false;
            }
        }
        return true;
    }
     
    
    public boolean hasChanged() {
        
        return changeListener.hasChanged();
    }
    
    public boolean isOK() {
        
        return true;
    }


    /* (non-Javadoc)
     * @see proxyconfig.WizardDialog#update()
     */
    public void update() {
        // TODO Auto-generated method stub
        
    }

    private boolean isSecureConnectionActive () {
        
        return secureConnectionEnabledFlag;
    }
    
    
    private String getStatusString () {
        
        if (isSecureConnectionActive ()) {
            return "<b><font color=\"green\">ENABLED</font></b>";
        } else {
            return "<b><font color=\"red\">DISABLED</font></b>";
        }
    }
    
    /** Enable/Disable fields, depending on whether secure connection
     * is enabled or disabled.
     */
    private void enableFields (boolean enabledFlag) {
        gatewayDownLB.setEnabled (enabledFlag);
        gatewayDown_dropRB.setEnabled (enabledFlag);
        gatewayDown_nodropRB.setEnabled (enabledFlag);
        urlPatternTable.setEnabled (enabledFlag);
        addURLPatternBT.setEnabled (enabledFlag);
        editURLPatternBT.setEnabled (enabledFlag);
        delURLPatternBT.setEnabled (enabledFlag);
        urlPanel.setEnabled (enabledFlag);
        urlTableLB.setEnabled (enabledFlag);
        urlTableLB.setDisabledTextColor(Color.GRAY);
    }
    
    
    /**
     * Check if the RSPTunnelOCD is active in proxy-config.xml
     * @return
     */
    boolean checkIfOCDActive () {
        RSPTunnelOCD ocd = (RSPTunnelOCD) proxyConfig.proxyConfigData.getOverlayById(RSP_TUNNEL_OCD_ID);
        if (ocd == null) {
            return false;
        }
        if (ocd.isActive) {
            return true;
        } else {
            return false;
        }
   	}
    
    private void enableOCD (boolean enabledFlag) {
        RSPTunnelOCD ocd = (RSPTunnelOCD) proxyConfig.proxyConfigData.getOverlayById(RSP_TUNNEL_OCD_ID);
        if (ocd != null) {
            ocd.isActive = enabledFlag;
        } else {
            if (enabledFlag) {
                //Create a new RSPTunnelOCD
                ocd = new RSPTunnelOCD();
                ocd.type = "RSPTunnelOCD";
                ocd.id = RSP_TUNNEL_OCD_ID;
                ocd.isActive = true;
                ocd.libFileName_linux = "./libRSPTunnelOCD.so";
                ocd.libFileName_windows = "RSPTunnelOCD.dll";
                ocd.libFileName_apple = "libRSPTunnelOCD.dylib";
                ocd.configFile = "config_files/RSPTunnelOCD.xml";
                ocd.description = "A simple tunnel for the remote server proxy";
               
                
                proxyConfig.proxyConfigData.addOCD(ocd);   
                
            }
        }
    }
    
    void updateFields() {
        RSPTunnelOCD ocd = (RSPTunnelOCD) proxyConfig.proxyConfigData.getOverlayById(RSP_TUNNEL_OCD_ID);
        if (ocd != null) {
	  	/**
            ocd.rspDomainList.clear();
            Iterator urlItr = urlPatternTableModel.getRecordList().iterator();
            while (urlItr.hasNext()) {
                OutboundRecord currOutRec = (OutboundRecord) urlItr.next();
                if (currOutRec.hopList.size() == 2) {
                    HopInfo firstHop = (HopInfo) currOutRec.hopList.get(0);
                    if (firstHop.endPointName != null) {
                        String currGateway = firstHop.endPointName;
                        if (ocd.rspDomainList.contains (currGateway)) {
                            ocd.rspDomainList.add (currGateway);
                        }
                    }
                }
            }
	    */

            if (gatewayDown_dropRB.isSelected()) {
                ocd.gatewayDownPolicy = "drop";
            } else if (gatewayDown_nodropRB.isSelected()) {
                ocd.gatewayDownPolicy = "nodrop";
            }
        } else {
            JOptionPane.showMessageDialog (
                    null,
                    "Erorr while updating fields. Please report this scenario to OCALA support. Thanks.",
                    "Error",
                    JOptionPane.ERROR_MESSAGE
                   );
        }
        
    }
    
    void loadURLPatterns () {
        
        Iterator itr = proxyConfig.proxyConfigData.outboundList.iterator();
        while (itr.hasNext()) {
            OutboundRecord currRec = (OutboundRecord) itr.next();
            // Check if the hop info of this outbound record corresponds to 
            // a Secure Connection
            if (currRec.hopList.size() == 2) {
                
                HopInfo firstHop = (HopInfo) currRec.hopList.get(0);
                HopInfo secondHop = (HopInfo) currRec.hopList.get(1);
                if (firstHop.ocdId.equals (RSP_TUNNEL_OCD_ID) &&
                        secondHop.ocdId.equals (LSIP_OCD_ID)) {
                    urlPatternTableModel.addRecord( currRec);
                }
            }
            
        }
    }
    
    
    private void showConfigSuccessMsg () {

        String msg = "";
        if (secureConnectionEnabledFlag) {
            String targetURLs = urlPatternTableModel.getURLs();
            String targetURLStr;
            if (targetURLs != null) {
                targetURLStr = "All traffic to <b><i>" + targetURLs + "</i></b> will be secured.<br>";
            } else {
                targetURLStr = "<font color=\"red\">WARNING:</font> You have not specified the connections that are to use OCALA Secure Connections. <br>";
            }
       
       	    String iconMsg = "";
	    if (proxyConfig.isWindows()) {
		//iconMsg = "The <img src=\"images/ProxyMonitor_icon.gif\"/> and <img src=\"images/rsptunnelocd_run_icon.gif\"/> icons will appear in the Windows System Tray when OCALA proxy is running and Secure Connection is active.";
	    }

            String gatewayStr = null;
            if (gatewayDown_dropRB.isSelected()) {
                gatewayStr = "If a gateway is down, all connections using that gateway will be dropped.<br>";
            } else {
                gatewayStr = "If a gateway is down, all new connections using that gateway will be sent over the normal unsecured channel.<br>";
            }
            msg = "<html><body" +
					"<b><font size=\"4\" color=\"blue\">Congratulations!</font></b><br><br>" +	
					"You have successfully set up OCALA Secure Connections.<br><br>" +
					targetURLStr +
					gatewayStr +
					"<br><br>Don't forget to make sure that the OCALA proxy is running." +
					iconMsg +
					"</body></html>"
				;
            
        }
        JEditorPane mainEditorPane = new JEditorPane ("text/html" ,msg);
         mainEditorPane.setEditable (false);  
        JScrollPane scrollPane = new JScrollPane (mainEditorPane);
	    scrollPane.setPreferredSize (new Dimension (400,300));
	    JOptionPane.showMessageDialog ( null, 
					scrollPane,
					"Configuration Successful",
					JOptionPane.INFORMATION_MESSAGE
				);
        
    }


}
