/*
 * Created on Apr 13, 2006
 */
package proxyconfig.rsptunnelocd;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.ListIterator;

import javax.swing.*;

import proxyconfig.*;
import proxyconfig.outboundconnections.*;

/**
 * @author danjo
 */
public class URLPatternPanel extends JPanel {

    JComboBox gatewayCombo;
    JTextField urlPatternTF;
    
    URLPatternTableModel urlPatternTableModel;
    ProxyConfig proxyConfig;
   
    private boolean editModeFlag;
    OutboundRecord myRec;
    
    public URLPatternPanel(URLPatternTableModel urlPatternTableModel, ProxyConfig proxyConfig, boolean editModeFlag, OutboundRecord myRec) {
    
        this.myRec = myRec;
        this.editModeFlag = editModeFlag;
        this.urlPatternTableModel = urlPatternTableModel;
        this.proxyConfig = proxyConfig;
        
        setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));
        
        JPanel mainPanel = new JPanel();
        add(mainPanel);
    	Insets insets = new Insets(2,2,2,2);
		GridBagLayout gbag = new GridBagLayout();
		mainPanel.setLayout(gbag);

		GridBagConstraints gc = new GridBagConstraints();
		gc.weighty = 0.0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.insets = insets;
		gc.gridx = GridBagConstraints.REMAINDER;
		gc.anchor = GridBagConstraints.WEST;

		JLabel tempLabel;

		// The preferred Dimension of all the input fields - mainly used to enforce a common height
		Dimension minDim = new Dimension(200, 25);
		
		tempLabel = new JLabel("URL/URL Suffix");
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		mainPanel.add(tempLabel);

		urlPatternTF = new JTextField ();
		urlPatternTF.setMinimumSize(minDim);
		gc.gridx = 1;
		gc.gridy = 0;
		gc.weightx = 0.0;
		gbag.setConstraints(urlPatternTF, gc);
		mainPanel.add (urlPatternTF);

		tempLabel = new JLabel("Gateway to use");
		gc.gridx = 0;
		gc.gridy = 1;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		mainPanel.add(tempLabel);
		
		gatewayCombo = new JComboBox();
		gatewayCombo.setMinimumSize(minDim);
		gc.gridx = 1;
		gc.gridy = 1;
		gc.weightx = 0.0;
		gbag.setConstraints(gatewayCombo, gc);
		mainPanel.add(gatewayCombo);
				
		populateGatewayList();
		
		if (myRec != null) {
            setFields (myRec);
		}
	}
    
    boolean checkFields () {
        
        String s = urlPatternTF.getText();

        if (s == null || s.trim().equals("")) {
            JOptionPane.showMessageDialog(
            		null,
            		"The URL/URL suffix cannot be empty.  Please enter a URL/URL suffix",
            		"URL/URL Suffix missing",
            		JOptionPane.ERROR_MESSAGE
            	);
            return false;
        }
        
        if ((s != null) && (s.length() > 0)) {
            if ( !editModeFlag && urlPatternTableModel.existsPattern (s)) {
                
                JOptionPane.showMessageDialog(
                        		null,
                        		"The URL/URL suffix \"" + s + "\" already exists.\n" +
                        		"Cannot add it again.",
                        		"Duplicate!",
                        		JOptionPane.ERROR_MESSAGE
                        	);
      
                return false;
            }

            String warningMsg = null;
            warningMsg = urlPatternTableModel.conflictsRecord(s, editModeFlag,  myRec);
            if ( !warningMsg.equals( (""))) {
                JOptionPane.showMessageDialog(
                		null,
                		"The URL/URL suffix \"" + s + "\" conflicts with existing URL suffixes \"" + warningMsg + "\".\n" +
                		"Cannot add it.",
                		"Conflicting URL!",
                		JOptionPane.ERROR_MESSAGE
                	);
                
                return false;
            } 
            if (s.lastIndexOf('*') > 0) {
                JOptionPane.showMessageDialog(
                		null,
                		"The URL suffix \"" + s + "\" has a \'*\' inside the pattern.\n" +
                		"Only suffixes or full URLs are allowed.  This means that a \'*\' can occur only at the beginning, if at all.",
                		"Invalid URL suffix!",
                		JOptionPane.ERROR_MESSAGE
                	);
                return false;
            }   
        } else {
            return false;
        }
        return true;
    }

    
    void updateRecord (OutboundRecord rec) {
        
        rec.pattern = urlPatternTF.getText();
        HopInfo firstHop = (HopInfo) rec.hopList.get(0);
        firstHop.endPointName = gatewayCombo.getSelectedItem().toString();
    }


    private void populateGatewayList () {
        
        ArrayList tmpList = new ArrayList();
       
        /* Ignoring gateway list file as something as URI not hierarchical exception thrown in
	 * Windows.  In linux it is fine.
	 * 	
        // First get the list of gateways from the gateway list file.
        try {
            URL gatewayURL = getClass().getResource("/" + SecureConnectionWizardDialog.GATEWAY_LIST_FILE);
            if (gatewayURL == null) {                
                System.out.println ("Unable to find gateway list file: " + SecureConnectionWizardDialog.GATEWAY_LIST_FILE);
                return;
            }

            URI gatewayURI = new URI (gatewayURL.toString());
    	    System.out.println ("URI = " + gatewayURI.toString() + " URL =  " + gatewayURL.toString() + "\n");
            BufferedReader br = new BufferedReader (new FileReader (new File (new URI ("jar:file://ProxyConfig.jar!SecureConnection.Gateways.txt"))));
            String currLine = null;
            if (br != null) {
                while ( (currLine = br.readLine()) != null) {
                    tmpList.add (currLine);
                    gatewayCombo.addItem(currLine);
                }                
            }        
        } catch (IOException ioe) {
            System.out.println ("Unable to read gateway list: " + ioe);
            ioe.printStackTrace();            
        } catch (URISyntaxException ure) {
            System.out.println ("Unable to read gateway list: " + ure);
            ure.printStackTrace();
        }
        */
        // Now add the gateways present in the RSPTunnelOCD config file.
        // and pre-select them.
        RSPTunnelOCD ocd = (RSPTunnelOCD) proxyConfig.proxyConfigData.getOverlayById(SecureConnectionWizardDialog.RSP_TUNNEL_OCD_ID);
        if (ocd != null) {
            ListIterator domainItr = ocd.rspDomainList.listIterator();
            while (domainItr.hasNext()) {
                String currDomain = (String) domainItr.next();
                if (!tmpList.contains (currDomain)) {
                    gatewayCombo.addItem(currDomain);
                }
            }
        }
    }


    void setFields (OutboundRecord rec) {
        urlPatternTF.setText( rec.pattern);
        if (rec.hopList.size() == 2) {
            HopInfo tmpHop = (HopInfo) rec.hopList.get(0);
            gatewayCombo.setSelectedItem(tmpHop.endPointName);
        }
    } 
}
