#!/usr/bin/perl -w

use strict 'vars';
package main;

use English;




# check if /dev/net/tun exists, if not create it appropriately


# Check for a line of the form 'major# tuntab' in /etc/iproute2/rt_tables
my $found_tuntab = 0;
open RTTABLES, "/etc/iproute2/rt_tables";
while (<RTTABLES>) {
    chomp $_;
    if ($_ =~ /^#/) {
	    next;
    }
    if ($_ =~ /tuntab/) {
	    $found_tuntab = 1;
        #print "Found tuntab entry in rt_tables: $_\n";
	    last;
    }
}

# Check if the tun device exists;
my $tun_dev_name = "/dev/net/tun";
my $found_tun = 0;
if (-e $tun_dev_name) {
    $found_tun = 1;
}

my $minor_num;
if ($found_tun == 0 || $found_tuntab == 0) {

    # We need to install the tun device or the routing table entries.
    # So we need root permissions.

    # Check for root permissions.
    if ($UID != 0) {
        # No root permissions
        print "Unable to install the tun device.  Root permissions needed.\n";
        #print "Please re-run quickStart.sh or run configure_tun.pl with superuser privileges.\n\n";
        exit (2); # Exit with appropriate error code
    }   

    $minor_num = 201;
    if ( $found_tun == 0) { # Create the tun device
        print "Creating $tun_dev_name ";
        my $cmd = "mknod /dev/net/tun c 10 $minor_num";
        system($cmd);
    }

    if ($found_tuntab == 0) {  # Add routing table entry.
        print "Adding tuntab entry to rt_tables\n";
        my $cmd = "echo \'200 tuntab\' >> /etc/iproute2/rt_tables";
        #print $cmd."\n";
        system($cmd);
    }

    print "Installed tun device (minor number $minor_num)\n";
    
} else { # tun device already present and in routing table.  Do nothing
    
    my($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
        $atime, $mtime, $ctime, $blksize, $blocks) = 
	stat $tun_dev_name or die "Can't stat $tun_dev_name\n";
    $minor_num = $rdev%256;

    print "tun device (minor number $minor_num) already installed.\n";
}



exit (0);

