/**
 *  \file DNSPacket.h
 *  \brief Declaration of DNSPacket object.
 */

#ifndef DNS_PACKET_H
#define DNS_PACKET_H 1
#include "IPPacket.h"

class VirtualNode;

class DNSPacket: public IPPacket {

private:
    
public:

   /** This is used to the store the contents of the DNS request,
      * while waiting for the DNS reply.  
      * TODO Not exactly sure what it is.
      */
    char packetData[28];
    
    /** Pointer to the DNS header of this packet. */
    char *dnsHdr;

    uint16_t dnsFlags;

    int dnsNumQuest;
    
    /** The URL requested in the IP Packet */
    URL reqURL;

    /** This field is used only when legacy DNS requests are made.
      * This helps to remember the virtual node which made the request.
      * The receivedDNSReply() of the virtual node will be invoked when
      * the DNS reply is received.
      */
    VirtualNode *vn;
    
 
    DNSPacket(int pktSize, char *pktContents, IPHdr *ipHeader);
    DNSPacket(const char *);

    void getAnswer(char *buf, const IPAddress *ansIP);

    ~DNSPacket() {
        
    }

    static int makeDNSQueryPacket (char *buf, const char *requestedURL, uint16_t transactionId);

    static char * parseDNSQueryName(char *dnsQuest, char* tmpURLStr);

    static char * stripDNSPrefLastSection (char *dnsPrefStr);
    static void extractActualName (char *buf, const char *dnsPrefSection);
    static void extractFirstSection (char *buf, const char *dnsPrefSection);

    static void getPrefSection (const char *dnsPrefStr, char *prefSection);
    static bool checkPreference (const char *dnsPrefStr, char prefToCheck);


};

#endif
