/**
 *  \file FakeAddressManager.h
 *  \brief Declaration of the FakeAddressManager object.
*/

#ifndef FAKE_ADDRESS_MANAGER_H
#define FAKE_ADDRESS_MANAGER_H 1

#include "common.h"
#include "ConfigFile.h"
#include <bitset>
using namespace std;

#define DEFAULT_FAKE_ADDR_RANGE_START "10.0.0.1"
#define DEFAULT_FAKE_ADDR_RANGE_END "10.0.0.255"
//XXX: what should be a good default range end?

#define DEFAULT_FAKE_ADDR_RANGE_MASK  "255.0.0.0"

/** The maximum number of fake addresses available. (2^24)*/
#define MAX_NUM_FAKE_ADDR 16777216

/**
  * Declarations of the FakeAddressManager class.
  */
class FakeAddressManager {
private:
    
    /** The start IP of the available range of fake IPs. */
    unsigned long startFakeIP;

    /** The number of fake IPs in the available range. */
    unsigned long numIPs;

    /** The bit set to keep track of which fake IPs are currently in use. */
    bitset<MAX_NUM_FAKE_ADDR> freeIPSet;

    /** The index of the nextFree IP in the free IP bit set. */
    unsigned long nextFreeIP;
    
public:
    
    /** The starting address of the range of fake IP addresses
     * available.
     */
    char fakeAddrRangeStart[MAX_IP_ADDR_LEN];

    /** The mask corresponding to the range of fake IP addresses
     * available.
     */
    char fakeAddrRangeMask[MAX_IP_ADDR_LEN];


    /** Constructor
      */
    FakeAddressManager(string, string);

    IPAddress *getNewFakeIP();
    IPAddress *getNewFakeIP(string url);
    IPAddress* getNewRandomFakeIP();

    bool isFakeIP(string ipStr);

    bool checkAllocateFakeIP (struct in_addr requestedAddr);

    bool  allocateFakeIP(const IPAddress *);
    void releaseFakeIP(const IPAddress *ip);
};

#endif
