/**
 *  \file FakeAddressNegotiator.h
 *  \brief Declaration of the FakeAddressNegotiator class.
*/

#ifndef FAKE_ADDRESS_NEGOTIATOR
#define FAKE_ADDRESS_NEGOTIATOR

#include "common.h"
#include "debug.h"
#include "FakeAddressNegotiationBlocks.h"

class VirtualNode;
class Proxy;

#define FAKE_ADDRESS_NEGOTIATION_SUCCESS    1
#define FAKE_ADDRESS_NEGOTIATION_FAILED     2
#define FAKE_ADDRESS_NEGOTIATION_IN_PROGRESS    3

/**
  * This class negotiates the same fake ip address at 
  * the proxies at both end of the connection.
  */
class FakeAddressNegotiator {


    /** The fake IP address which was requested from the other end.
      */
    struct in_addr requestedAddr;

    /** The virtual node for which the address is being negotiated.
      */
    VirtualNode *vn;

    /** A reference to the generic proxy */
    Proxy *proxy;
   
    /** FAKE_ADDRESS_NEGOTIATION_SUCCESS, FAILED, IN_PROGRESS */
    int state;
    
public:

    FakeAddressNegotiator (VirtualNode *vn, Proxy* proxy);
   
    void initNewConnection();
    void receivedBlock (FakeAddressNegotiationBlock *blk);

    void receivedNegotiationRequest (FakeAddressNegotiationBlock * blk);
    void receivedNegotiationReplyYes (FakeAddressNegotiationBlock *blk) ;
    void receivedNegotiationReplyNo () ;

    int getState();
};

#endif
