/**
 *  \file IPPacket.h
 *  \brief Definition of an IP packet.
*/

#ifndef IP_PACKET_H
#define IP_PACKET_H 1

//WINHAHA #include <netdb.h>

#include "common.h"

class DNSPacket; //forward declaration

class IPPacket {

public:
    
    /** DNS_PACKET, I3_PACKET etc defined in packetTypes.h */
    int pktType;

    /**
     *  The actual (original) contents of the packet (as received or
     *  transmitted.
     */
    char *pktContents;

    /**
      * The length of the packet.
      */
    int pktSize;

	/** This flag is set to false to indicate the pktContents buffer should not be freed
	 * when the object is destroyed.
	 * The default value of this flag is true.
	 */
	bool freeMemFlag;

    /**
      * Pointer to the IP header of this packet.
      */
    struct IPHdr *ipHeader;

    IPPacket(int pktSize, char* pktContents);

    IPPacket();

    virtual ~IPPacket();

    char* getSrcIPAddress(char *tmpAddrBuf);
    char* getDestIPAddress(char *tmpAddrBuf);
    ipaddr_n_t getSrcIPAddress();
    ipaddr_n_t getDestIPAddress();

    portnum_n_t getSrcPortNum();
    portnum_n_t getDestPortNum();

    protocol_n_t getProtocolNum();

    uint32_t getFlowHash();

    bool isDNSRequest();

    DNSPacket *convertToDNSPacket();
    
    void printPacket();
    void dumpPacket();

	friend class Tunnel;
};

#endif
