/**
 *  \file JournalManager.h
 *  \brief Definition of the Journal Manager class.
*/

#ifndef JOURNAL_MANAGER_H
#define JOURNAL_MANAGER_H

#include "common.h"
#include "OCALA_Export.h"

class JournalSource;
class OCI;
class TimerEvent;

/**
  * This is a general purpose class to periodically dump Journal records to disk.
  */
class JournalManager {

    /** A pointer to the OCI.  This is need to schedule timer events. */
    OCI* oci;
    
    /** The source of Journal Records */
    JournalSource *journalSrc;
   
    /** The timer used to periodically invoke flushToDisk() */
    TimerEvent *timer;
   
    void doCrashRecovery ();

public:
    /** The name of the journal disk file */
    char fileName[MAX_FILENAME_LEN];

  
    OCALA_LIB_EXPORT_IMPORT JournalManager (OCI* oci, JournalSource *, char *fileName, bool recoverFromCrash);  
    OCALA_LIB_EXPORT_IMPORT void flushToDisk();

    OCALA_LIB_EXPORT_IMPORT void start();
    OCALA_LIB_EXPORT_IMPORT void stop();
};

#endif  // JOURNAL_MANAGER
