/**
 *  \file JournalSource.h
 *  \brief Definition of an interface which classes serving as sources of JournalRecord objects
 * must implement.
*/

#ifndef JOURNAL_SOURCE_H
#define JOURNAL_SOURCE_H

#include <list>
using namespace std;

#include "OCALA_Export.h"

#include "JournalRecord.h"

/**
  * This class just behaves as an interface for classes which can serve as the source of 
  * journal records - for eg: OCI, various OCDs.
  */
class OCALA_LIB_EXPORT_IMPORT JournalSource {

public:

    /**
      * Copy a list of Journal Records into the specified recList.
      */
    virtual void getJournalRecordList( list<JournalRecord*> *recList) {
        //dummy must be overloaded
    }

    /**
      * This function is called when the Journal Manager is doing crash recovery.
      * This callback to the jource of the journal record.  The Source is supposed
      * to reconstruct its state on the basis of the information contained in
      * recordStr.
      */
    virtual void recoverRecord (char *recordStr) {
        //dummy must be overloaded
    }
    

};
#endif
