/**
 *  \file LegacyAppLayer.h
 *  \brief Declaration of the layer which interacts directly with the legacy applications.
*/

#ifndef LEGACY_APP_LAYER_H
#define LEGACY_APP_LAYER_H

#include <map>
using namespace std;

#include "common.h"
class Tunnel;
class Proxy;
class IPPacket;
class OCIState;
class VirtualNode;


/**
  * This class represents the legacy applications.
  * The virtual nodes talking to the tun device are part of this layer.
  */
class LegacyAppLayer {

    Proxy* proxy;

    /** The tunnel used to communicate with the legacy apps. */
    Tunnel *tunnel;
 
    /** The hash table of Virtual nodes keyed by their URL */
    map<URL, VirtualNode*> map_VN_URL;
    
    /** The hash table of Virtual nodes keyed by their fake IP address */
    map<IPAddress, VirtualNode*> map_VN_FakeIP;

    /** Hash table of virtual nodes keyed by the flow hashes of the packets in the flow.
      * The flow hash is calculated using IPPacket::getFlowHash() function.
      */
    map<uint32_t, VirtualNode*> map_VN_FlowHash;
   
public:

    void addVN_FakeIP(VirtualNode *vn);
    void removeVN_FakeIP(VirtualNode *vn);
    VirtualNode* lookupVN_FakeIP(IPAddress);
    
    void addVN_URL(VirtualNode* vn);
    void removeVN_URL(VirtualNode *vn);
    VirtualNode* lookupVN_URL(URL);

    void addVN_FlowHash(VirtualNode* vn);
    void removeVN_FlowHash(VirtualNode *vn);
    VirtualNode* lookupVN_FlowHash(IPPacket*);
    
    void tunnelCapturedPacket(IPPacket *captIPPkt);

    LegacyAppLayer (Proxy *proxy);
    void setTunnel (Tunnel *tunnel);

    void sendToLegApp (VirtualNode *vn, char *pkt, int pktLen);
    
    VirtualNode* getNewVN (OCIState *ociState);

    void destroyVN (VirtualNode *);
};

#endif
