/**
 *  \file Params.h
 *  \brief Defines a whole lot of parameters common across various modules
 * of the proxy.
*/

/** This is the standard port to which DNS requests are made. */
#define STD_DNS_REQUEST_PORT 53

/** This is the port to be used by the proxy to contact DNS servers
  * while resolving DNS packets for legacy URLs.
  */
#define LEGACY_DNS_DEFAULT_REQUEST_PORT 55667
    
/** The name of the configuration file to be used if not specified
 * through the command line.
 */
#define DEFAULT_CONFIG_FILENAME "config_files/proxy-config.xml"

/**
  * The period (seconds) with which a Virtual Node should be checked for activity.
  */
#define VN_ACTIVE_CHECK_PERIOD 300

/** The time period (in seconds) with which journals of the various overlay networks
 * are flushed to disk.
 */
#define JOURNAL_FLUSH_PERIOD 20


/** The maximum number of certificates this overlay can possess. */
#define MAX_NUM_CERTIFICATES    15

/** The delimiters to be used to separate certificate file name:private key file name pairs 
  * in the configuration file attribute. 
  */
#define CERT_FILENAME_DELIM " \t\n"

/** The delimiter between the certificate file name and private key file name in the 
  * configuration file.
  */
#define CERT_PRIVKEY_DELIM ":"

/** Number of seconds the OCI should wait for setupSuccess or setupFailed callback from the OCD 
  * during a new connection setup.
  */
#define OCD_SETUP_TIMER    10

/**
  * The number of seconds the OCI should wait for a PathSetupSuccess or PathSetupFailed
  * message after sending an OCI PathSetup message.
  */
#define OCI_SETUP_TIMER     20

/** The normal (when no loss has been detected for a long time)
  * inter KeepAlive message period in milliseconds
  */
#define OCI_KEEPALIVE_NORMAL_SEND_INTERVAL  2 * 1000

/** The normal (when no loss has been detected for a long time)
  * number of millisecons to wait for KeepAlive reply
  */
#define OCI_KEEPALIVE_NORMAL_REPLY_INTERVAL  10 * 1000

/**
  * The period (in seconds) with which an OCIState should
  * be checked for inactivity.
  */
#define OCI_ACTIVE_CHECK_PERIOD     7200

/** How often should the OCI look for dead OCIStates */
#define OCI_STATE_CLEANUP_PERIOD    30

/** The maximum number of times we try to set up a path before giving up */
#define MAX_PATH_SETUP_ATTEMPTS 10

/** The default file name for the OCIState Journal */
#define OCI_STATE_JOURNAL_FILENAME ".ociJournal.jrnl"

