#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <arpa/inet.h>
#include <errno.h>
#include <net/if.h>

#include "debug.h"
#include "common.h"

#ifndef IPPROTO_DIVERT
#warning "Snarf_divert requires kernel divert socket support"
int
create_divert_socket(int divert_port)
{
    fprintf(stderr, "snarf_divert: no divert socket support\n");
    exit(-1);
}
#else
/*
 * This routine is vaguely FreeBSD specific.  It requires Divert
 * sockets.  A port to Linux can be found (as of Sep 2001) at
 * http://www.anr.mcnc.org/~divert/index.shtml
 */

int create_divert_socket(int divert_port)
{
    int sock;
    struct sockaddr_in divertsock;
    int rc;

    sock = socket(PF_INET, SOCK_RAW, IPPROTO_DIVERT);
    if (sock < 0) {
	perror("snarf_freebsd::create_divert_socket socket call");
	PRINT_DEBUG (DEBUG_LEVEL_FATAL, "snarf_freebsd::create_divert_socket socket call");
	return -1;
    }

    bzero(&divertsock, sizeof(divertsock));
    divertsock.sin_addr.s_addr = INADDR_ANY;
    divertsock.sin_port = htons(divert_port);
    divertsock.sin_len = sizeof(divertsock);
    divertsock.sin_family = AF_INET;

    PRINT_DEBUG (DEBUG_LEVEL_MINIMAL, "Binding raw socket to divert port %d\n", divert_port);
    rc = bind(sock, (struct sockaddr *)&divertsock,
	      sizeof(divertsock));
    if (rc) {
	perror("snarf_freebsd::create_divert_socket bind");
	PRINT_DEBUG (DEBUG_LEVEL_MINIMAL, "snarf_freebsd::create_divert_socket bind");
	close(sock);
	return -1;
    }

    return sock;
}

#endif /* IPPROTO_DIVERT */

int raw_socket() {
    int sock;
    int on = 1;

    sock = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
    if (sock == -1) {
	PRINT_DEBUG (DEBUG_LEVEL_FATAL, "socket::raw_socket socket raw create failed");
        EXIT_ON_ERROR;
    }

    if (setsockopt(sock, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on)) < 0) {
	PRINT_DEBUG (DEBUG_LEVEL_FATAL, "socket::raw_socket setsockopt HDRINCL failed");
	EXIT_ON_ERROR;
    }
    return sock;
}



