/**
 *  \file checkpoint.c
 *  \brief This file contains function to add checkpoints to the code.
 *  This is used while benchmarking the proxy.
 */

#include "checkpoint.h"

#include <string.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>


TimeStamp timestamp[MAX_NUM_EVENTS];
int event_count = 0;

void addEvent(char *payload)
{
  int ret;
  uint32_t seq;
  
  struct timeval now;
  gettimeofday(&now, NULL);

  memcpy(&seq, payload, sizeof(uint32_t));
  seq = ntohl(seq);
  
  assert(event_count < MAX_NUM_EVENTS);
  
  ret = sprintf(timestamp[event_count].event, "%ld DATA_EVENT ", seq);
  timestamp[event_count].ts = now.tv_sec * UMILLION + now.tv_usec;

  event_count++;
}


void
addCheckpoint(int type, int seq)
{
    struct timeval now;
    gettimeofday(&now, NULL);

    if (!(type == EVENT_PATH_BROKEN || type == EVENT_PATH_SETUP_COMPLETED)) {
        return;
    }
    assert(event_count < MAX_NUM_EVENTS);
    
    timestamp[event_count].ts = now.tv_sec * UMILLION + now.tv_usec;
    timestamp[event_count].seq = seq;
    timestamp[event_count].type = type;
    event_count++;
}

char* getEventName (int eventType) {

    switch (eventType) {

	case UDP_RECV:
	    return "UDP_RECV";
	case UDP_SEND_START:
	    return "UDP_SEND_START";
	case UDP_SEND_END:
	    return "UDP_SEND_END";
	case TCP_RECV_START:
	    return "TCP_RECV_START";
	case TCP_RECV_END:
	    return "TCP_RECV_END";
	case TCP_SEND_START:
	    return "TCP_SEND_START";
	case TCP_SEND_END:
	    return "TCP_SEND_END";
	case UDP_ECHO_SEND_END:
	    return "UDP_ECHO_SEND_END";
	case UDP_ECHO_SEND_START:
	    return "UDP_ECHO_SEND_START";
	case UDP_ECHO_RECV_START:
	    return "UDP_ECHO_RECV_START";
	case UDP_ECHO_RECV_END:
	    return "UDP_ECHO_RECV_END";
	case DNS_START:
	    return "DNS_START";
	case DNS_END:
	    return "DNS_END";
	

	case EVENT_READ_FROM_TUN: 

	    return "EVENT_READ_FROM_TUN";

	case EVENT_DATA_TUN_TO_OC_I: 

	    return "EVENT_DATA_TUN_TO_OC_I";

	case EVENT_DATA_OC_I_TO_OC_D: 

	    return "EVENT_DATA_OC_I_TO_OC_D";

	case EVENT_DATA_BEFORE_OC_D_TO_NET: 

	    return "EVENT_DATA_BEFORE_OC_D_TO_NET";

	case EVENT_DATA_AFTER_OC_D_TO_NET: 

	    return "EVENT_DATA_AFTER_OC_D_TO_NET";


	case EVENT_DATA_NET_TO_OC_D: 

	    return "EVENT_DATA_NET_TO_OC_D";

	case EVENT_DATA_OC_D_TO_OC_I: 

	    return "EVENT_DATA_OC_D_TO_OC_I";

	case EVENT_DATA_OC_I_TO_TUN: 

	    return "EVENT_DATA_OC_I_TO_TUN";



	case EVENT_DNS_REQUEST_FROM_TUN:
	    return "EVENT_DNS_REQUEST_FROM_TUN";

	case EVENT_DNS_REPLY_BEFORE_WRITE_TO_TUN:
	    return "EVENT_DNS_REPLY_BEFORE_WRITE_TO_TUN";

	case EVENT_DNS_REPLY_AFTER_WRITE_TO_TUN:
	    return "EVENT_DNS_REPLY_AFTER_WRITE_TO_TUN";

	case EVENT_DNS_OC_I_TO_OC_D:
	    	return "EVENT_DNS_OC_I_TO_OC_D";
	
	case EVENT_DNS_OC_D_TO_OC_I:
		return "EVENT_DNS_OC_D_TO_OC_I";

	
    case EVENT_PATH_BROKEN:
            return "EVENT_PATH_BROKEN";

    case EVENT_PATH_SETUP_COMPLETED:
            return "EVENT_PATH_SETUP_COMPLETED";

	    
        default:
            return "UNKNOWN EVENT";
        break;
    }

}
void
DumpEventInfo(int signal)
{
    int i;
    printf("Dumping timestamps of checkpoints\n");
    for (i = 0; i < event_count; i++) {
	    fprintf(stderr, "%Ld\t%s\n", timestamp[i].ts, getEventName (timestamp[i].type));
// 	    fprintf(stderr, "%s %d %Ld\n", EventName(timestamp[i].type), 
// 		        timestamp[i].seq, timestamp[i].ts);
	    fflush(stderr);
    }

    abort();
}
