/**
 *  \file  checkpoint.h
 *  \brief This file defines the various checkpoint events
 * and declares the functions to add/dump checkpoints.
 */

#ifndef _CHECKPOINT_H
#define _CHECKPOINT_H

#include <netdb.h>
#include <sys/time.h>
#ifdef __APPLE__
#include <stdint.h>
#endif

#define MAX_NUM_EVENTS 5000
#define UMILLION 1000000ULL

//events
#define EVENT_READ_FROM_TUN 100
#define EVENT_DATA_TUN_TO_OC_I 101
#define EVENT_DATA_OC_I_TO_OC_D 102
#define EVENT_DATA_BEFORE_OC_D_TO_NET 103
#define EVENT_DATA_AFTER_OC_D_TO_NET 104

#define EVENT_DATA_NET_TO_OC_D 105
#define EVENT_DATA_OC_D_TO_OC_I 106
#define EVENT_DATA_OC_I_TO_TUN 107

#define EVENT_DNS_REQUEST_FROM_TUN 200
#define EVENT_DNS_REPLY_BEFORE_WRITE_TO_TUN	201
#define EVENT_DNS_REPLY_AFTER_WRITE_TO_TUN	202
#define EVENT_DNS_OC_I_TO_OC_D	203
#define EVENT_DNS_OC_D_TO_OC_I 204

#define EVENT_PATH_BROKEN 301
#define EVENT_PATH_SETUP_COMPLETED 302

#define UDP_RECV		1
#define UDP_SEND_START		2
#define UDP_SEND_END		3
#define TCP_RECV_START		4
#define TCP_RECV_END		5
#define TCP_SEND_START		6
#define TCP_SEND_END		7
#define UDP_ECHO_RECV_START	8
#define UDP_ECHO_RECV_END	9
#define UDP_ECHO_SEND_START	10
#define UDP_ECHO_SEND_END	11
#define DNS_START		12
#define DNS_END			13





//
// TimeStamp structure
// 
typedef struct TimeStamp {
    int type;
    int seq;
    uint64_t ts;
    char event[200];
} TimeStamp;

void DumpEventInfo(int signal);

void addEvent(char *payload);
void addCheckpoint (int type, int seq);

#endif
