/** \file common.h
  * \brief This file contains definitions which are used through out
  * the project.
  */

#ifndef CONSTANTS_H
#define CONSTANTS_H 1

#include <stdio.h>
#include <string>

#ifndef _WIN32
	#include <netinet/in.h>
	#include <netdb.h>
#endif

#include "debug.h"

#ifndef _WIN32
//#	define BENCHMARK 1
#endif

#if defined (BENCHMARK)
extern "C" {
    #include "checkpoint.h"
}
#endif

#include "fwint.h"
#if defined(_WIN32)
extern "C" {
	#include "netwrap.h"
}
typedef int32_t suseconds_t;
#define strcasecmp _stricmp
#define inet_aton(str, addr) inet_pton(AF_INET, str, addr)
#endif


#if defined( _WIN32)
//#if defined( __CYGWIN__ ) || defined( __APPLE__ )
typedef long suseconds_t;
#endif

/** Maximum number of bytes required to store an IP address of the form
 * A.B.C.D
 */
#define MAX_IP_ADDR_LEN 40

/** Maximum length of URL regular expressions that can be specified 
 * in the LegacyURLRules section of the proxy configuration file.
 */
#define MAX_LEGACY_URL_RULE_LEN 500

/** Maximum number of bytes required for the buffer in which commands to
 * be executed via system call are constructed.
 */
#define CMD_BUF_LEN 200

/** This is the maximum length of the buffer to be used to print any
 * stuff into.
 */
#define MAX_PRINT_BUF 500

/** IPAddress as a C++ string class so that it can be used as key for
 * hash table entries.
 */
typedef std::string IPAddress;

/** IP Address in network byte order 
 *  All ip addresses in the code may not be using this data type.  
 *  Gradually convert to achieve consistency.
 */
typedef uint32_t ipaddr_n_t;

/** Port number in network byte order 
 *  All port numbers in the code may not be using this data type.  
 *  Gradually convert to achieve consistency.
 */
typedef uint16_t portnum_n_t;

typedef uint8_t protocol_n_t;

#define PROTOCOL_UDP 17
#define PROTOCOL_TCP 6
/**
  * A structure to encapsulate a protocl and port number.
  */
typedef struct {
    portnum_n_t port;

    /** PROTOCOL_UDP or PROTOCOL_TCP */
    protocol_n_t protocol;
} port_protocol_n_t;


/** URL is defined as a C++ string class so that it can be used as a key
 * in hash tables.
 */
typedef std::string URL;

/** The maximum length allowed for a URL */
#define MAX_URL_LEN 200

/** The maximum length of the ID (can be i3 id, or some other overlay specific id or even
  * an IP address) of the server proxy to be used for a particular URL.
  */
#define MAX_SERVER_PROXY_ID_LEN 200

/** The maximum length of a received packet */
#define MAX_PACKET_SIZE 4096

/** The maximum length of the buffer used to receive a packet */
#define MAX_PACKET_BUF_LEN 5000

/** The maximum length for a file name */
#define MAX_FILENAME_LEN 200

/** The maximum length of a parameter value in a configuration file.*/
#define MAX_PARAM_LEN 250

/** The maximum length of a command line argument.  */
#define MAX_CMD_LINE_ARG_LEN 300

/** The maximum length of a journal entry */
#define MAX_JOURNAL_RECORD_LEN 5000

/** The maximum length of the DNSname + preferences string */
#define MAX_PREF_STRING_LEN     300

/* IP Header -- RFC 791 */
typedef struct IPHdr
{
    uint8_t vihl;   // Version and IHL
    uint8_t tos;    // Type Of Service
    uint16_t    tlen;   // Total Length
    uint16_t    id; // Identification
    uint16_t    flagoff;// Flags and Fragment Offset
    uint8_t ttl;    // Time To Live
    uint8_t protocol;   // Protocol
    uint16_t    checksum;   // Checksum
    struct in_addr src;     // Internet Address - Source
    struct in_addr dst;     // Internet Address - Destination
} IPHdr;

/**
  * One second in terms of microseconds.
  */
#define SECOND_USEC 1000000

/** The number of bits in an IPv4 address */
#define NUM_BITS_IN_IP_ADDRESS  32

/** NULL_HANDLE for OCIHandle and OCDHandle variables */
#define NULL_HANDLE 0

/**
 * This Handle is returned by an OCD in response to a setup request
 * from the OCI when the OCD wants to signal to the OCI that this connection
 * should be dropped.  
 * Introduced to support the drop policy of RSPTunnelOCD.
 */
#define DROP_OCD_HANDLE 1

#ifdef _WIN32
#define MAX_REGISTRY_STR_VALUE_LEN 1000
#endif

#endif
