/**
 * \file CompressionState.h
 * \brief State information associated with compressing 
 * the data of a particular connection.
*/

#ifndef COMPRESSION_STATE_H
#define COMPRESSION_STATE_H

/** We have requested the other end to enable compression */
#define COMPRESSION_STATE_NEGOTIATION_IN_PROGRESS   1

/** Both ends have enabled compression */
#define COMPRESSION_STATE_OK 2

/** Compression could not be negotiated */
#define COMPRESSION_STATE_FAIL 3

#include "CompressionPacketBlocks.h"

class VirtualNode;
class CompressionBlackBox;

/**
  *  This class is provides functions for compressing and decompressing data.
  */
class CompressionState {

    /** The compression state is associated with this VN */
    VirtualNode *vn;

    /** The compression level to be used. 1 = best speed to 9 = best compression. 0 = no compression*/
    int compressionLevel;
  
    /** What compression scheme to use?
      * For eg: COMPRESSION_SCHEME_ZLIB
      */
    int compressionScheme;
   
    /** This blackbox is used to compress/uncompress data */
    CompressionBlackBox *blackBox;
    
    void handleCompressionRequest (CompressionPacketBlock *blk);
    void handleCompressionReplyOK (CompressionPacketBlock *blk);
    void handleCompressionReplyFail (CompressionPacketBlock *blk);

public:

    /** What is the state of compression negotiation of this manager?
      * Has the other end responded to our request for compression?
      * COMPRESSION_STATE_NEGOTIATION_IN_PROGRESS, COMPRESSION_STATE_OK etc.
      */
    int state;
    
    CompressionState (VirtualNode *vn, int level);
    CompressionState (VirtualNode *vn);
    ~CompressionState();

    void compress (const char *plainData, unsigned long plainDataLen, char *compressedData, unsigned long * compressedDataLen);
    void decompress (const char *compressedData, unsigned long compressedDataLen, char* plainData, unsigned long * plainDataLen);
    void initNewConnection();

    void receivedBlock (CompressionPacketBlock * blk);

    
};

#endif
