#ifndef BLACK_LIST_H
#define BLACK_LIST_H

#include <list>
#include <map>
#include <string>
using namespace std;

// Comment out the following line when compiling LSIP.
// TEST_MODE only when compiling testBlackBox
//#define TEST_MODE

#ifndef TEST_MODE

#ifdef _WIN32
    extern "C" {
#       include "netwrap.h"
    }
#else
#   include <netdb.h>
#endif

class Proxy;
#endif //TEST_MODE

#define BLACKLIST_DEFAULT_RELOAD_CHECK_PERIOD   120 // 2 minutes

/**
  * Checks if a particular URL is blacklisted or not.
  * The blacklisting decision is based on a list of URLs and suffix patterns that are forbidden.
  */
class BlackList {


    /** Key = suffixes of length equal to that of the smallest pattern/URL in the forbidden list file.
      * Value = list of patterns with the specified suffixes.
      */
    map <string, list<string>* >* forbiddenMap;
   
    /** This value is used to determine what suffix of a test string should be used as a key into forbiddenMap.
      */
    int minPatternLen;
   
    /** The name of the file storing the URLs/URL patterns that are blacklisted. */
    string forbiddenListFilename;
   
#ifndef TEST_MODE
    /** Access to add timer event */
    Proxy* mainLoop;
    
    /** This file contains a single line which says "yes" or "no".  
      * If "yes", the forbiddenListFile is reloaded during the periodic check.
      */
    string reloadCheckFilename;

    /** How often (in seconds) should we check if the forbiddenListFile should be reloaded. */
    uint16_t reloadCheckPeriod;
    
    void doPeriodicReload ();
#endif //TEST_MODE

    bool loadMap();

public:
#ifndef TEST_MODE
    BlackList (const char *forbiddenListFilename, const char* reloadCheckFile, Proxy* proxy, uint16_t reloadCheckPeriod);
#else
    BlackList (const char *forbiddenListFilename);
#endif

    ~BlackList();

    void dumpForbiddenList();
    bool isBlackListed (const char* url);
    
};
#endif //BLACK_LIST
