/**
 *  \file LSIPv4_OCD.h  
 *  \brief Declaration of the Legacy Server IP module.
 */

#ifndef REMOTE_IP_LAYER_H
#define REMOTE_IP_LAYER_H

#include "../common.h"
#include "../ocd/OCD.h"

#include <map>

class LSIPv4_OCDState;
class Proxy;
class OCI;
class Tunnel;
class IPPacket;
class BlackList;

class MyNAT;

class LSIPv4_OCD : public OCD {

    /** Used only if proxy->mySoftwareNATFlag is false */
	MyNAT* myNAT;


    Proxy *proxy;

    Tunnel *tunnel;

    /** Blacklist of URLs to be blocked */
    BlackList* blackList;
    
    /** The hash table of Virtual nodes keyed by their fake IP address */
    map<IPAddress, LSIPv4_OCDState*> map_VN_FakeIP;

    void addVN_FakeIP(LSIPv4_OCDState *vn);
    void removeVN_FakeIP(LSIPv4_OCDState *vn);
    LSIPv4_OCDState* lookupVN_FakeIP(IPAddress);


    LSIPv4_OCD (string type, string id, string desc);
    LSIPv4_OCD (OCD* ocd);

    virtual void init (Proxy *, OCI *, bool recoverFromCrash);
    
    char* toString(char *tempStr);

    void recoverFromCrash();

#ifdef __CYGWIN__
    const char* getWinRegistryKey() { return "REMOTE_IP_LAYER";}
#endif
    int getStatus() { return 1;}

    void makeDNSRequest (LSIPv4_OCDState *vn);


    // Used only if we are using the Linux software NAT.
    void tunnelCapturedPacket(IPPacket *captIPPkt);
    // Used only if we are using our own custom software NAT
    void tunnelCapturedPacket(IPPacket *captIPPkt, LSIPv4_OCDState *ocdState);
    
/* OCD API - Start */
    void send ( OCDHandle handle, char* data, int dataLen);
    OCDHandle setup (
                const string endPointName, 
                int endPointName,
                char *optionBlocks, 
                int optionLen,
                OCIHandle ociHandle,
                OCI* oci );
/* OCD API - End */

    friend class LSIPv4_OCDState;
    friend class Proxy;

    friend class MyNAT;
};

#endif
