/**
 *  \file LSIPv4_OCDState.cpp
 *  \brief This file defines the state information associated with a connection 
 *  passing though the LSIPv4_OCD.
 */
#include <stddef.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "LSIPv4_OCDState.h"
#include "LSIPv4_OCD.h"
#include "../Proxy.h"

/**
  */
LSIPv4_OCDState::LSIPv4_OCDState (OCIHandle myHandle, OCI* oci, LSIPv4_OCD* lsip) 
            : OCDStateInfo (myHandle, oci, lsip) {


    resolvedFlag = false;
}



/**
  * This function is called when the reply to a DNS request made
  * by this Virtual node arrives.
  * @param ip The IP address corresponding to the request.  Error if 0.
  */
void LSIPv4_OCDState::receivedLegacyDNSReply(uint32_t ip) {
    
    if (ip != 0) {
      
        {
            struct in_addr tmpIPAddr;
            tmpIPAddr.s_addr = ip;

            PRINT_DEBUG ( 
                    DEBUG_LEVEL_MINIMAL,
                    "LSIPv4 OCD state with handle %lu received DNS reply: domain=%s IP = %s.\n",
                    myHandle,
					endPointName,
                    inet_ntoa (tmpIPAddr)
                );
        }

	    targetIP_h = ip;
        resolvedFlag = true;
        notifySetupSuccess();
    
    } else {
        //Unable to resolve the requested URL
        PRINT_DEBUG (
                DEBUG_LEVEL_MINIMAL,
                "LSIPv4: Unable to resolve %s.\n",
                endPointName
            );
        notifySetupFailure();
    }

}

LSIPv4_OCDState::~LSIPv4_OCDState () {
   
   if (ocd != NULL) {
        ((LSIPv4_OCD *) ocd)->removeVN_FakeIP (this);
        ((LSIPv4_OCD *)ocd)->proxy->clearDNSRequest (this);
   }

}
