/**
 *  \file LSIPv4_OCDState.h
 *  \brief This file declares the LSIPv4 connection state information.
 */

#ifndef I3_LSIP_V4_OCD_STATE_H
#define I3_LSIP_V4_OCD_STATE_H

#include <netdb.h>
#include <sys/time.h>

#include "../common.h"
#include "../ocd/OCDStateInfo.h"
#include "../oci/OCIHandle.h"

class LSIPv4_OCD;

class LSIPv4_OCDState : public OCDStateInfo {

    time_t time;

    uint16_t dnsTransactionId;

    /** The IP address (in host byte order) of the requestedURL (this field will contain the address
      * once it is resolved.)
      */
    uint32_t targetIP_h;

    /** This flag is set to true once the DNS name has been successfully resolved. */
    bool resolvedFlag;

    /** This is used to demultiplex packets coming off the wire from the remote IP end point */
    IPAddress* fakeIP;
    
    /** Handle to the OCI state associated with this OCDState.
      * We do not share OCD tunnels between multiple connections 
      * for ease of demultiplexing.
      */
    OCIHandle ociHandle;

    LSIPv4_OCDState (OCIHandle myHandle, OCI* oci, LSIPv4_OCD* );
    ~LSIPv4_OCDState();
    
    virtual void receivedLegacyDNSReply(uint32_t resolvedIP);

    friend class LSIPv4_OCD;

    friend class MyNAT;

};

#endif
