#ifndef MY_NAT_H
#define MY_NAT_H 1

#include "../common.h"


#include "IPPacket.h"
#include <bitset>
#include <map>

using namespace std;

class LSIPv4_OCDState;
class MyNATPort;

/** The maximum number of ports that we are allowed to use for the NAT.
 */
#define MAX_NUM_PORTS 20000

class Proxy;
class LSIPv4_OCD;
class LSIPv4_OCDState;

/** This defines a full cone software NAT
 */
class MyNAT {

    /** Pointer to the LSIP served by this NAT. */
    LSIPv4_OCD* lsip;

    /** A pointer to the proxy of which the NAT is a part. */
    Proxy* proxy;
   
	/** flow --> NATPort data structure which stores the port, socket as well as
     *  OCDState info associated with this flow
     */
    map<uint32_t, MyNATPort*> map_NATPort_flowHash;
	
    /** Hash table of NATPort objects keyed by the socketFD associated with them.
      * This hash table is consulted to retrieve the state associated with a packet
      * that is received over a particular socket.
      */
	map<int, MyNATPort*> map_NATPort_sockFD;

	
	/** The first port in the range allocated to us - host byte order.*/
    uint16_t natPortRangeStart;
	/** The last port in the range allocated to us - host byte order. */
	uint16_t natPortRangeEnd;

    /** The number of ports allocated to us.*/
    unsigned long natPortCount;

    /** The bit set to keep track of which ports are currently in use. */
    bitset<MAX_NUM_PORTS> freePortSet;

    /** The index of the next free port in the free ports bit set. */
    unsigned long nextFreePort;
    
	MyNAT (Proxy* proxy, LSIPv4_OCD* lsip, uint16_t natPortRangeStart, uint16_t natPortRangeEnd, int natPortCount);

	bool getFreePort (uint16_t *freePort);
    void releasePort (uint16_t relPort);

	int grabPorts();

    int initPort (uint16_t port, int protocol);
    void destroyPort (MyNATPort *);

	MyNATPort* lookupNATState (IPPacket *pkt, LSIPv4_OCDState* ocdState);

    void receiveRawPacket (int sockFD);
    bool sendPacket (IPPacket* ipPkt, LSIPv4_OCDState* ocdState);
    
	friend class LSIPv4_OCD;
    friend class MyNATPort;
};


#endif
