#include "MyNATPort.h"
#include "MyNAT.h"
#include "LSIPv4_OCDState.h"
#include "TimerEvent.h"
#include "Proxy.h"

/**
  * @param portNum new port num in host byte order
  * @parama originalPortNum The original port number of packets associated with this state (in host byte order).
  * @param protocol IPPROTO_UDP or IPPROTO_TCP in host byte order.
  * @param ocdState The OCDState of packets associated with this NAT state.
  * @param sockFD The socket which is bound to this port.
  */
MyNATPort::MyNATPort (uint16_t portNum, uint16_t originalPortNum, uint8_t protocol, 
                    ipaddr_n_t remoteAddr, int sockFD, uint32_t pktFlowHash, LSIPv4_OCDState* ocdState,
                    MyNAT* myNAT) {

    this->portNum = portNum;
    this->protocol = protocol;
	this->remoteAddr = remoteAddr;
    this->sockFD = sockFD;
    this->ocdState = ocdState;
    this->originalPortNum = originalPortNum;
    this->pktFlowHash = pktFlowHash;
    this->myNAT = myNAT;

    this->activeFlag = true;
    this->activeCheckTimer = NULL;

    scheduleActiveCheck();
}

/**
  * This function schedules the activeCheck() function to be called
  * after the designated amount of time.
  */
void MyNATPort::scheduleActiveCheck() {
   
    // First disable any existing active Check timer before we add a new one.
    if (activeCheckTimer != NULL) {
        activeCheckTimer->setDisabled();
        activeCheckTimer = NULL;
    }
    
    TObjectFunctor<MyNATPort> *tmpFunctor = 
                new TObjectFunctor<MyNATPort> (this, &MyNATPort::activeCheck);
    activeCheckTimer = TimerEvent::create(10, 0, (TFunctor *) tmpFunctor, true);
    myNAT->proxy->addTimerEvent(activeCheckTimer);
}

/** Checks if this NAT state is active.
  * If inactive, kills it.
  */
void MyNATPort::activeCheck () {
    if (!activeFlag) {
        myNAT->destroyPort (this);
        delete (this);
    } else {
        //If someone doesn't set this flag back to true before the next check,
        //this NAT state will be killed.
        activeFlag = false;
        scheduleActiveCheck();
    }
}

/** Destructor 
  */
MyNATPort::~MyNATPort () {
    if (activeCheckTimer != NULL) {
        activeCheckTimer->setDisabled();
    }
}
