#ifndef MY_NAT_PORT_H
#define MY_NAT_PORT_H

#include "common.h"

class LSIPv4_OCDState;
class TimerEvent;
class MyNAT;

/** This class encapsulates the various details of a port used in our NAT.
  */
class MyNATPort {

    /** The (source) port number in host byte order. 
     *  This is the new source port allocated by the NAT.
     *  The original port of packets associated with this NAT port is
     *  stored in originalPortNum.
     */
    uint16_t portNum;

    /** The original port associated with this NATted portNum.
      * (in host byte order)
      */
    uint16_t originalPortNum;

    /** IPPROTO_TCP or IPPROTO_UDP, in host byte order */
    uint8_t protocol;
    
    /** The socket which is bound to portNum. */
    int sockFD;

    /** This flag is used to garbage collect inactive ports.
      * This flag is set to true whenever there is data exchange on the port.
      * It is periodically reset.  If a periodic reset process finds this flag to
      * be off, this MyNATPort object is discarded.
      */
    bool activeFlag;

	/** Only packets from this remote address are considered valid.
	 * This makes the NAT restricted cone.  Not useful for a generic NAT service/bittorrent.
	 */
	ipaddr_n_t remoteAddr;

    /** The flow for which this NAT state was created. */
    uint32_t pktFlowHash;

    /** The OCD state associated with this flow */
    LSIPv4_OCDState* ocdState;

    /** Just a pointer to the NAT */
    MyNAT* myNAT;
    
    TimerEvent* activeCheckTimer;
    void scheduleActiveCheck();
    void activeCheck();

public:
    MyNATPort (uint16_t portNum, uint16_t originalPortNum, uint8_t protocol, ipaddr_n_t remoteAddr, int sockFD, uint32_t pktFlowHash, LSIPv4_OCDState* ocdState, MyNAT* myNAT);
    ~MyNATPort ();

    friend class MyNAT;
};

#endif
