/**
 *  \file BroMiddleBox.h
 *  \brief Declaration of the BroMiddleBox shim layer.
 */

#ifndef BRO_MIDDLE_BOX_H
#define BRO_MIDDLE_BOX_H

#include <map>
using namespace std;

using namespace std;
#include "MiddleBox.h"

class OCIState;
class FakeAddressManager;
class BroState;

/** The name of the FIFO to be used for communicating with the Bro process. */
#define BRO_FIFO_NAME "/tmp/bro_fifo"

/** TODO Ask Jayanth*/
static const char hexheader[24] = {0xD4,0xC3,0xB2,0xA1,0x02,0x0,0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xFF,0xFF,0x0,0x0,0x71,0x0,0x0,0x0
};

/** Dummy link headers */
static const char linkheader[16] = {0x0,0x04,0x02,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x8,0x0};


/**
  * This middlebox is a stub which interfaces to the 
  * Bro intrusion detection system.
  */
class BroMiddleBox : public MiddleBox {

    /** OCI handle to BroState mappings. */
    map<OCIHandle, BroState*> broStateMap;
    
    /** This is the file descriptor to communicate with the
      * actual Bro process.
      */
    int outFD;

    
    /**
      * This fake address manager is used for Bro specific packet rewriting.  Nothing to do with the proxy fake address manager.
      */
    FakeAddressManager* fakeAddrManager;

    BroState* lookupBroState (OCIHandle ociHandle);
    void addBroState (OCIHandle ociHandle, BroState *);

    bool openFIFO();

public:

    BroMiddleBox (char *configFileName);
    virtual ~BroMiddleBox();

    bool receive (OCIState *ociState, const char* buf, uint16_t* bufLen);

};
#endif
