/**
 *  \file MiddleBox.h  
 *  \brief All middle boxes which need to interface to the proxy
 *  must expose this simple interface.
 */

#ifndef MIDDLE_BOX_H
#define MIDDLE_BOX_H
#include "common.h"
#include "../oci/OCIHandle.h"

class OCIState;

/** 
  * This class merely acts as an interface for middleboxes.
  */
class MiddleBox {

public:
    
    /**
      * This function is called whenever a packet is received
      * at the middlebox.  The contents of the packet are in buf
      * and its length in buf.  The middlebox can alter the packet contents
      * and length by modifying these parameters.  
      * This function should return only after all middlebox processing on 
      * buf is completed.  The packet is sent to the next hop as as this function
      * returns.
      * @param ociState The OCI State corresponding to the path on which the packet
      * was receieved.  This information may be useful for the middlebox in keeping track of
      * state information.
      *
      * @return The packet is forwarded to the next hop only if the middlebox returns true.
      */
    virtual bool receive (OCIState *ociState, const char * buf, uint16_t* bufLen)=0;
};

#endif
