#ifndef OCD_REGISTER_H
#define OCD_REGISTER_H

#include "OCALA_version.h"
#include "OCD.h"


#ifndef _WIN32

	bool registerOCD (string ocdType, makeOCDFull_t* fnPtr, int majorVersion, int minorVersion);

	extern "C" {    
		/** This function returns an object constructed using the constructor that takes no arguments.
		*/
		OCD* makeOCD() { return NULL;}

		/** 
		* This function returns an object constructed using the OCD (string type, ...) constructor.
		*/
		OCD* makeOCDFull (  string type, string id, string description, 
                    string configFileName, 
                    string urlSuffixes, bool handleLegacyURLs
                ) { 
            return new MY_OCD_CLASS_NAME (
                                type, id, 
                                description,configFileName, 
                                urlSuffixes, handleLegacyURLs
                            );
 
		}

	    class DummyOCDInit {
        public:
            DummyOCDInit() {
                //register the makeFullOCD() with the proxy

                string ocdType = MY_OCD_TYPE_STR;
                if (!registerOCD(ocdType, makeOCDFull, OCALA_MAJOR_VERSION, OCALA_MINOR_VERSION)) {
                    PRINT_INFO (
                                    INFO_LEVEL_WARNING,
                                    "Unable to register OCD with type %s.\n",
                                    ocdType.c_str()
                                );
                }
            }
		};
		//Create one instance of the dummy class
		static DummyOCDInit dummyOCDInit;

	} // exter "C"
	
#endif

#ifdef _WIN32
#include "OCDExport.h"

	extern "C" {    
		/** This function returns an object constructed using the constructor that takes no arguments.
		*/
		OCD* makeOCD() { return NULL;}

		/** 
		* This function returns an object constructed using the OCD (string type, ...) constructor.
		*/
		OCD_WINDOWS_API OCD*  makeOCDFull (  string type, string id, string description, 
                    string configFileName, 
                    string urlSuffixes, bool handleLegacyURLs
                ) { 
					return new MY_OCD_CLASS_NAME (
                                type, id, 
                                description,configFileName, 
                                urlSuffixes, handleLegacyURLs
                            );
 
		}
	}
#endif

extern "C" {

	/** Returns the major version number of the OCALA proxy against
	 * which this OCD module was compiled.
	 */
	int getOCALAMajorVersion() {return OCALA_MAJOR_VERSION;}
    
	/** Returns the minor version number of the OCALA proxy against
	* which this OCD module was compiled.
      */
    int getOCALAMinorVersion() {return OCALA_MINOR_VERSION;}

}

#endif 
