/**
 *  \file OCDStateInfo.h 
 *  \brief This file declares the OCDStateInfo class.
 */
#ifndef OCD_STATE_INFO_H
#define OCD_STATE_INFO_H

#include <list>

#include "OCD.h"
#include "../common.h"

#define OCD_STATUS_NOT_READY 0
#define OCD_INFO_STATUS_READY 1
#define OCD_INFO_STATUS_FAILED 3

#include "../DNSReplyCallback.h"

class OCD;
class OCI;

#define OCD_STATE_STATUS_NOT_READY 0
#define OCD_STATE_STATUS_READY 1
#define OCD_STATE_STATUS_FAILED 3

class OCDStateInfo : public JournalRecord, public DNSReplyCallback {

public:
    OCDHandle get_myHandle() {return myHandle;}
    string get_endPointName() { return string (endPointName); }
    bool hasSetupCompleted() { return setupCompletedFlag;}

protected:

    OCI *oci;
   

    int status;

    /** This flag is set to true if the OCD level setup has been completed.
      */
    bool setupCompletedFlag;

    /** The list of OCIStates currently using this OCDState.
      * This info is needed so that we can discard this OCDState
      * when no OCIState uses it.
      */
    list<OCIHandle> ociRefList;
    
    char *setupOptionBlocks;
    int setupOptionLen;
    
    /** The handle referring to this state */
    OCDHandle myHandle;

    
    /** The name of the remote (but immediate) end point associated with this state.
      */
    char endPointName [MAX_URL_LEN];


    /** List of handles of parent Layer States which are currently waiting for the setupSucceeded or 
      * setupFailed callback from this state.
      */
    list<OCIHandle> currWaitingParentHandleList;
    
    /** Pointer to the parent OCD of this OCDState */
    OCD* ocd;
    
    OCALA_LIB_EXPORT_IMPORT OCDStateInfo(OCDHandle myHandle, OCI* oci, OCD* ocd);
    OCALA_LIB_EXPORT_IMPORT OCDStateInfo (OCDHandle myHandle, OCD* ocd);
    OCALA_LIB_EXPORT_IMPORT virtual ~OCDStateInfo();
    
    OCALA_LIB_EXPORT_IMPORT void addParentHandle (OCIHandle handle);
    OCALA_LIB_EXPORT_IMPORT void delWaitingParentHandle (OCIHandle handle);

    OCALA_LIB_EXPORT_IMPORT void notifySetupFailure();
    OCALA_LIB_EXPORT_IMPORT void notifySetupSuccess();

    /** This function is relevant to OCDs like RemoteIPLayer and RON
      * which make DNS queries associated with their OCDStates.
      * @param resolvedIP the resolved IP address in host byte format.
      */
    OCALA_LIB_EXPORT_IMPORT virtual void receivedLegacyDNSReply(uint32_t resolvedIP);

    OCALA_LIB_EXPORT_IMPORT virtual void getJournalRecord (char *buf);

    OCALA_LIB_EXPORT_IMPORT void addOCIRefList (OCIHandle ociHandle);
    OCALA_LIB_EXPORT_IMPORT void delOCIRefList (OCIHandle ociHandle);
    OCALA_LIB_EXPORT_IMPORT int getOCIRefCount ();

    OCALA_LIB_EXPORT_IMPORT int getWaitingParentCount();
    
private:


	friend class OCI;
    friend class OCD;
    friend class OCIState;
    friend class Proxy;

};

#endif
