/**
 *  \file OCI.h
 *  \brief This file defines the Overlay Convergence Independent layer.
 */

#ifndef OCI_H
#define OCI_H 1

#include <map>
#include <list>
#include <queue>

#include "../OCALA_Export.h"
#include "../common.h"
#include "../IPPacket.h"

#include "../TimerEvent.h"

#include "../JournalSource.h"
#include "../ocd/OCDHandle.h"
#include "OCIPacketBlock.h"
#include "OCIHandle.h"

class Proxy;
class OCIState;
class JournalManager;
class JournalRecord;
class MiddleBox;
class OCD;

/** Indicates that the left OCD should be used for sending packets. */
#define OCI_LEFT    100

/** Indicates that the right OCD should be used for sending packets. */
#define OCI_RIGHT   200

/** Denotes an OCI level name which is to be resolved to an OCD-specific id. */
#define OCI_NAME    300

/** Denotes an OCD-specified identifier */
#define OCD_ID      400


using namespace std;


/**
  * This class encapsulates the OCI.
  */
class OCI : public JournalSource {
public:
    // Callbacks from OCD - Start
    OCALA_LIB_EXPORT_IMPORT void receive (char *pkt, OCD* ocd, OCDHandle ocdHandle);
    OCALA_LIB_EXPORT_IMPORT void setupFailed (OCIHandle handle);
    OCALA_LIB_EXPORT_IMPORT void setupSucceeded (OCIHandle handle);
    // Callbacks from OCD - End

    virtual ~OCI();

private:

    /** A reference to the proxy to which this OCI belongs. */
    Proxy* proxy;

    /** This hash table stores handle->stateinfo mappings */
    map<OCIHandle, OCIState *> handleMap;
 
    /** List of all active OCIStates */
    list<OCIState *> ociStateList;

    /** This object manages the periodic flushing of OCIState records to disk */
    JournalManager* journalManager;

    /** The actual middlebox, if this OCI is functioning as a middlebox. */
    MiddleBox* middleBox;
    
    void putStateInfo (OCIState* state);
    void delStateInfo (OCIState* state);
    OCIHandle getNewOCIHandle();

//public:

    OCI (Proxy *);

    OCIState* setUpNewConnection (
                    const URL *url,
                    IPPacket* pkt,
                    char *setupOptionBlocks,
                    int setupOptionLen,
					bool* dropConnectionFlag
                );

    void send ( VirtualNode*, const char* data, int dataLen);
    void send ( OCIState* , const char* data, int dataLen, int direction);

  
    void recoverRecord (char *recStr);

    OCDHandle setupPath (OCIState *);


    OCIState *getStateInfo (OCIHandle handle);
    OCALA_LIB_EXPORT_IMPORT int getStatus (OCIHandle handle);

    void destroyOCIState (OCIState *);

    void addTimerEvent (TimerEvent *);

    void getJournalRecordList(list<JournalRecord *> *);

    OCALA_LIB_EXPORT_IMPORT void initJournalManager (bool crashRecoveryFlag);

    void stateCleanUp();
    void addStateCleanUpTimer();
   
    friend class OCDStateInfo;
    friend class OCIState;
    friend class CryptoState;
    friend class OCIKeepAliveState;
    friend class JournalManager;
    friend class LegacyAppLayer;
    friend class Proxy;
    friend int main (int argc, char* argv[]);
	friend int init_generic_proxy(int argc, char* argv[]);
	friend int resumeGenericProxy();
    friend class LSIPv4_OCD; // HACK - Dilip
};
#endif

