/**
 *  \file EncryptDecryptBlackBox.h
 *  \brief The declaration of the EncryptDecryptBlackbox.
*/
#ifndef ENCRYPT_DECRYPT_BLACK_BOX
#define ENCRYPT_DECRYPT_BLACK_BOX

#include <openssl/evp.h>
#include "common.h"
#include "securityCommon.h"

/** The encryption/decryption algo to be used */
#define ENC_ALGO EVP_des_ede3_cbc()


/**
  * This class is to be used for symmetric key
  * encryption and decryption.
  */
class EncryptDecryptBlackBox {

    /** The cipher context used for all encryption done with this black box. */
    EVP_CIPHER_CTX encryptCtx;

    /** The cipher context used for all decryptions done with this black box */
    EVP_CIPHER_CTX decryptCtx;
   
    unsigned char key [EVP_MAX_KEY_LENGTH];
    unsigned char iv [EVP_MAX_IV_LENGTH];
    
public:
    EncryptDecryptBlackBox(unsigned char key[EVP_MAX_KEY_LENGTH], unsigned char iv[EVP_MAX_IV_LENGTH]);

    ~EncryptDecryptBlackBox();

    void encrypt ( 
            unsigned char* dataToEncrypt, uint16_t dataToEncryptLen,
            unsigned char* encryptedData, uint16_t *encryptedDataLen
        );
             

    void decrypt (
            unsigned char* dataToDecrypt, uint16_t dataToDecryptLen,
            unsigned char* decryptedData, uint16_t *decryptedDataLen
        );

    static void encrypt ( 
            unsigned char* dataToEncrypt, uint16_t dataToEncryptLen,
            unsigned char* encryptedData, uint16_t *encryptedDataLen,
            SymKey *k
        );
             

    static void decrypt (
            unsigned char* dataToDecrypt, uint16_t dataToDecryptLen,
            unsigned char* decryptedData, uint16_t *decryptedDataLen,
            SymKey *k
        );


};


#endif
