/**
 *  \file SecurityManager.h
 *  \brief The declaration of the SecurityManager class.
*/

#ifndef SECURITY_MANAGER_H
#define SECURITY_MANAGER_H

#include <map>
#include <string>

#include "securityCommon.h"
#include "cryptoFunctions.h"
#include "certificate.h"

#include "OCALA_Export.h"

using namespace std;

/** No security */
#define SECURITY_LEVEL_0 0

/** Only authentication, no encryption */
#define SECURITY_LEVEL_1 1

/** Authentication and encryption */
#define SECURITY_LEVEL_2 2


/**
  * This class handles all the authentication, encryption, decryption etc
  */
class SecurityManager {

private:
    
    /** The public key of the CA to be used. */
    EVP_PKEY* caPubKey;

    map<string, RSA*> privKeyTable;

    //TODO get rid of the duplicate private key tables
    map<string, EVP_PKEY*> privEVPKeyTable;
    
    map<string, Certificate*> certificateTable;
    void addCertificate (Certificate *cert);
    void loadCertificate (const char *certFileName, const char* privKeyFileName); 
    
public:

    /** Is authentication required at the current security level? */
    bool authenticationRequiredFlag;
    
    /** Is encryption required at the current security level? */
    bool encryptionRequiredFlag;
    
    
    /**
      * What is the current security level?
      * SECURITY_LEVEL_0, SECURITY_LEVEL_1 ...
      */
    int securityLevel;        

    SecurityManager(int securityLevel, char *certFileStr, char *caCertFile);
    Certificate* getCertificate(const char *name);
    RSA* getPrivateKey (const char *name);
    EVP_PKEY* getPrivateEVPKey (char *name);


    OCALA_LIB_EXPORT_IMPORT void loadCertificate (const char *certStr);
    OCALA_LIB_EXPORT_IMPORT void printCertificateList();

    bool checkCertificate(const char *name, Certificate *cert);

    bool allowMiddleBox (char *middleBoxName);
};


#endif
