/**
 *  \file cryptoFunctions.h
 *  \brief General purpose cryptographic function declarations.
 */
#ifndef CRYPTO_FUNCTIONS_H
#define CRYPTO_FUNCTIONS_H

#include "common.h"
#include "cryptoHeaders.h"
#include "OCALA_Export.h"


OCALA_LIB_EXPORT_IMPORT void cryptoInit();

OCALA_LIB_EXPORT_IMPORT EVP_PKEY* initPublicKey(char *certfile);
OCALA_LIB_EXPORT_IMPORT EVP_PKEY* initPrivateKey(char *pkeyFileName);
OCALA_LIB_EXPORT_IMPORT void freeKey(EVP_PKEY* pkey);

OCALA_LIB_EXPORT_IMPORT void sign (char *data, int dataLen, unsigned char *sigBuf, uint16_t *sigLen, EVP_PKEY *privateKey);
OCALA_LIB_EXPORT_IMPORT int verifySignature(char* data, int dataLen, unsigned char* sigBuf, unsigned int sigLen, EVP_PKEY* publicKey);


OCALA_LIB_EXPORT_IMPORT RSA* generateKeyPair();
OCALA_LIB_EXPORT_IMPORT RSA* getPublicKeyFromPair(RSA *keyPair);
OCALA_LIB_EXPORT_IMPORT RSA* rsa_dup(RSA* key);

OCALA_LIB_EXPORT_IMPORT RSA* loadPrivateKeyFromBuffer(unsigned char *buf, long bufLen);
OCALA_LIB_EXPORT_IMPORT RSA* loadPublicKeyFromBuffer(unsigned char *buf, long bufLen);
OCALA_LIB_EXPORT_IMPORT EVP_PKEY* loadPrivateKeyFromDisk (const char *fileName);
OCALA_LIB_EXPORT_IMPORT EVP_PKEY* loadPublicKeyFromDisk (const char *fileName);

OCALA_LIB_EXPORT_IMPORT int savePrivateKeyToBuffer(RSA* keypair, unsigned char *buf, long bufLen) ;
OCALA_LIB_EXPORT_IMPORT int savePublicKeyToBuffer(RSA* keypair, unsigned char *buf, long bufLen) ;

OCALA_LIB_EXPORT_IMPORT void generateRandomSymKey (unsigned char *outKey, unsigned char* outIV);

#endif
