/** 
 *  \file nonce.c
 *  \brief Functions to generate and compare nonces.
 */

#include "nonce.h"

#include <stdio.h>
#include <openssl/bn.h>


/** Generate a random nonce of length NONCE_BIT_LEN and place it in nonceBuf
  */
void generateNonce (unsigned char *nonceBuf) {
    
    BIGNUM *mbn = BN_new();
	int numbytes;

    BN_rand(mbn, NONCE_BIT_LEN,-1,0);
    numbytes = BN_bn2bin(mbn,nonceBuf);
    
    //pad any remaining bytes with 0 
    for(;numbytes < NONCE_BIT_LEN/8; numbytes++) {
        nonceBuf[numbytes] = 0;
    }
    BN_clear_free(mbn);
}

/** Compare the given two nonces.
  * Returns 1 if they are equal, else returns 0
  */
int isNonceEqual (unsigned char* nonce1, unsigned char* nonce2)
{
    int i;
    for( i=0; i < NONCE_BIT_LEN/8;i++)
        if ( nonce1[i] != nonce2[i])
            return 0;
    return 1;
}
