/**
 *  \file securityCommon.h
 *  \brief Declarations of various parameters used in various security related modules.
*/

#ifndef SECURITY_COMMON_H
#define SECURITY_COMMON_H

#include <openssl/evp.h>

/** This certificate contains the public key of the CA.
 * This can be self signed or signed by some commercial CA.
 */
#define CA_CERT_FILE "CA_cert.pem"
#define CA_PRIVATE_KEY_FILE "CA_private_key.pem"

/** The maximum length of information messages
  * exchanged between the client and the server.
  * Most messages are from server to client.
  */
#define MAX_INFO_MSG_LEN 1000

/** The maximum size of data that can be signed in one go */
#define MAX_SIGN_BUF_SIZE 4096

/** Maximum length of the requested name */
#define CERT_MAX_NAME_LEN 150

/** Maximum length of the Real name of the person requesting the name */
#define CERT_MAX_REAL_NAME_LEN 100


/** Maximum length of an email address associated with the certificate */
#define CERT_MAX_EMAIL_LEN 100

/** The length of the email nonce (password) */
#define CERT_EMAIL_NONCE_LEN 50

/** The length of the revoke password */
#define CERT_REVOKE_PASSWORD_LEN 50


/** Maximum length of the signature */
#define CERT_MAX_SIGN_LEN 128

/** Maximum length of the public key associated with a name */
#define CERT_MAX_PUBLIC_KEY_LEN 150 //maybe it is only 140

#define CERT_MAX_PRIVATE_KEY_LEN 4096

/** The total size of the data to be signed */
#define MAX_DATA_TO_SIGN_SIZE (CERT_MAX_NAME_LEN + CERT_MAX_REAL_NAME_LEN + CERT_MAX_EMAIL_LEN + CERT_MAX_PUBLIC_KEY_LEN)

/** What is the size of the key? 512, 1024, 2048? */
#define CERT_RSA_KEY_SIZE 1024


#define MAX_CIPHER_BLOCK_LENGTH EVP_MAX_IV_LENGTH

/** The maximum number of names that can be contained in a certificate request. */
#define MAX_NAMES_IN_CERT_REQ   6

/** 
  * This structure is simply used to group together a symmetric key
  * and its IV.
  */
typedef struct {
    
    unsigned char key [EVP_MAX_KEY_LENGTH];
    unsigned char iv [EVP_MAX_KEY_LENGTH];

} SymKey;

#endif
