/**
 *  \file mailFunctions.c
 *  \brief This file contains functions used for sending email notifications.
 */
 
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <pthread.h>

#include "mailFunctions.h"
#include "../../debug.h"


/** The mutex used for mail operations */
pthread_mutex_t mailMutex;    

/**
  * This function must be called before using
  * any other mail functions.
  */
void initMail() {

    pthread_mutex_init(&mailMutex, NULL);
}

void lockMail() {
    pthread_mutex_lock (&mailMutex);
}

void unlockMail() {
    pthread_mutex_unlock (&mailMutex);
}

/** Returns 1 if the send mail operation succeeded.
  * Else returns 0.
  * This function assumes a correctly configured standalone mail program.
  */
int sendMail(char *emailAddr, char *subjectLine, char *body){

    lockMail();
   
    // write the body of the mail to a temporary file
    FILE *fp = fopen(TMP_EMAIL_FILE_NAME, "w");
    int retVal = fputs (body, fp);
    if (retVal == 0) {
        PRINT_INFO (
                    INFO_LEVEL_WARNING,
                    "Could not completely save body of the email to a file.\n"
                );
        fclose(fp);
        unlockMail();
        return 0;
    }
    fclose(fp);
    
    char mailCmd[1000];
    sprintf (   mailCmd, 
                "%s -s \"%s\" %s <%s",
                MAIL_PROGRAM_NAME,
                subjectLine,
                emailAddr,
                TMP_EMAIL_FILE_NAME
            );
    retVal = system (mailCmd);
   
    printf ("\nmailcmd=%s\n", mailCmd);

    if (retVal == -1) {
        retVal = 0;
    } else {
        retVal = 1;
    }

    unlockMail();
    return retVal;
}
