/**
 *  \file nameRegPolicy.c
 *  \brief This file contains functions which check if a particular name
 *  satisfies a name allocation policy.
*/

#include <string.h>
#include <ctype.h>

#include "../lib/certificate.h"
#include "../lib/securityCommon.h"

//TODO utilize the isSuffix function ../utils/utilFunctions.cpp
/**
  * Returns 1 if suffix is a suffix of masterStr.
  * Else returns 0.
  */
int isSuffix ( char* masterStr, char* suffix) {

    int masterStrLen = strlen(masterStr);
    int suffixLen = strlen(suffix);

    if (suffixLen > masterStrLen) {
        return 0;
    }

    int i = masterStrLen - 1;
    int j = suffixLen - 1;
    
    while (i > 0 && j > 0) {
    
        if (tolower(masterStr[i]) != tolower(suffix[j])) {
            return 0;
        }
        i--;
        j--;
    }

    return 1;
}


/**
  * Checks if the requested name matches the email.
  * Returns NULL if match.
  * Else puts an error message into msg and returns the same.
  */
char *doNameEmailMatchCheck (char *name, char* emailAddr, char *msg) {

    char tmpEmailStr [CERT_MAX_NAME_LEN + 10];
    
    //Make sure that the request name NAME is of the form NAME.emailAddr.
    //For eg: laptop.dilip@cs.berkeley.edu
    sprintf (tmpEmailStr, ".%s", emailAddr);
    
    if (isSuffix(name, tmpEmailStr)) {
        //Success!  Policy okay
        return NULL;
    }

    sprintf (msg, "Name Allocation Policy Error:  The requested name should be of the form "
            " name.emailAddr.");
    return msg;
            
}
