/**
 *  \file mainLoop.cpp
 *  \brief Program used to test the select() loop.
*/

#include "../common.h"
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <queue>
#include "TimerEvent.h"
#include <assert.h>
#include <error.h>
#include <string.h>
#include <iostream>
#include <map>

#include "../FDCallback.h"
#include "../TFunctor.h"
#include "TestClass.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

int openSocket(int portNum) {
    int sockfd;
    struct sockaddr_in my_addr;    // my address information
    struct sockaddr_in their_addr; // connector's address information
    int addr_len, numbytes;

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        perror("socket");
        exit(1);
    }

    my_addr.sin_family = AF_INET;         // host byte order
    my_addr.sin_port = htons(portNum);     // short, network byte order
    inet_aton("192.168.0.4", &my_addr.sin_addr); // automatically fill with my IP
    memset(&(my_addr.sin_zero), '\0', 8); // zero the rest of the struct

	PRINT_DEBUG(50, "Open scoket: %s\n", inet_ntoa(my_addr.sin_addr));
    if (bind(sockfd, (struct sockaddr *)&my_addr,
		sizeof(struct sockaddr)) == -1) {
        perror("bind");
        exit(1);
    }
	
	return sockfd;

}

int fn1(char *buf) {
	PRINT_DEBUG(50, "Function 1 \n\n\n");
}
int fn2(char *buf) {
	PRINT_DEBUG(50, "Function 2 \n\n\n");
}
int fn3(char *buf) {
	PRINT_DEBUG(50, "Function 3 \n\n\n");
}
int fn4(char *buf) {
	PRINT_DEBUG(50, "Function 4 \n\n\n");
}
int fn5(char *buf) {
	PRINT_DEBUG(50, "Function 5 \n\n\n");
}

void coreSelect();

priority_queue<TimerEvent, vector<TimerEvent>, TimerEventComparator> timerEventQueue;
map<int, FDCallback*> fdCallbackTable;

int main() {


	TimerEvent e1(1,0, "e1");
	TimerEvent e2(1,0, "e2");
	TimerEvent e3(10,10, "e3");
	TimerEvent e4(2,50, "e4");
	TimerEvent e5(2,40, "e5");
	timerEventQueue.push(e1);
	timerEventQueue.push(e2);
	timerEventQueue.push(e3);
	timerEventQueue.push(e4);
	timerEventQueue.push(e5);

	FDCallback* cbk = new FDCallback(openSocket(6001), fn1);
	fdCallbackTable.insert(pair<int, FDCallback *>(cbk->fd, cbk));
	
	cbk = new FDCallback(openSocket(6002), fn2);
	fdCallbackTable.insert(pair<int, FDCallback *>(cbk->fd, cbk));

	TestClassA objA;
	TestClassB objB;
	TObjectFunctor<TestClassA> fnA (&objA, &TestClassA::printA);
	TObjectFunctor<TestClassB> fnB (&objB, &TestClassB::printB);

	cbk = new FDCallback(openSocket(6003), &fnA);
	fdCallbackTable.insert(pair<int, FDCallback *>(cbk->fd, cbk));
	
	cbk = new FDCallback(openSocket(6004), &fnB);
	fdCallbackTable.insert(pair<int, FDCallback *>(cbk->fd, cbk));
	
	cbk = new FDCallback(openSocket(6005), fn5);
	fdCallbackTable.insert(pair<int, FDCallback *>(cbk->fd, cbk));
	
	coreSelect();
}

FDCallback* lookupCallback (int fd) {
	
	map<int,FDCallback*>::iterator itr = fdCallbackTable.find(fd);
	
	if (itr != fdCallbackTable.end()) {
		return (FDCallback*)itr->second;
		
	} else {
			
		return NULL;
	}
}


void coreSelect() {
	while (1) {
	
		char tmpStr[MAX_PRINT_BUF];
		fd_set rfds;
		int retval;
		
		// Set the read set.
		FD_ZERO(&rfds);
		
		int highestFD = -1;
		
		map<int, FDCallback*>::iterator itr = fdCallbackTable.begin();
		while (itr != fdCallbackTable.end()) {
					
			FDCallback *cbk = (FDCallback *) itr->second;
			PRINT_DEBUG(50, "Adding fd %d\n", cbk->fd);
			FD_SET(cbk->fd, &rfds);
			if (cbk->fd > highestFD) {
				highestFD = cbk->fd;
			}
			
			itr++;
		}
		highestFD++;
	
		if (!timerEventQueue.empty()) {
				
			TimerEvent nextEvent = timerEventQueue.top();
			struct timeval* nextEventTimeVal = nextEvent.getTimeVal();
		
			//current time of day
			struct timeval currentTime;
			gettimeofday(&currentTime, NULL);
		
			// subtract current time from the next event time to get the 
			// interval to wait for.
			struct timeval waitInterval;
			waitInterval.tv_sec = nextEventTimeVal->tv_sec - currentTime.tv_sec;
			suseconds_t usecDiff = nextEventTimeVal->tv_usec - currentTime.tv_usec;
			if (usecDiff < 0) {
				usecDiff += 1000000;
				waitInterval.tv_sec -= 1;
			}
			waitInterval.tv_usec = usecDiff;
		
			if (waitInterval.tv_sec < 0) {
				// This means that we have passed the deadline for this timer event.
				// This is bad!  But we don't crash - instead we continue and execute the action
				// associated with this event. Hopefully, it wasn't more than a few seconds late!

				PRINT_INFO(INFO_LEVEL_VERBOSE, "Late timer event: %s.\n", nextEvent.toString(tmpStr));
				//TODO trigger nextEvent
				timerEventQueue.pop();
				continue;
			}	
		
		
			retval = select(highestFD, &rfds, NULL, NULL, &waitInterval);
			/* Don’t rely on the value of tv now! */
		
		} else {
			//The timer queue is empty.
			//Just wait indefinitely for activity on the file descriptors.
			//This is because any new TimerEvents can be added only then. Remember
			//we are single threaded.
			retval = select(highestFD, &rfds, NULL, NULL, NULL);
		}
		
		if (retval == -1) {
			PRINT_INFO(INFO_LEVEL_FATAL_ERROR, "Error in main SELECT statement!\n%s\n", strerror(errno));
			perror("select()");
			
		} else if (retval) {
			
			//Some activity on the file descriptors
		    PRINT_DEBUG(50, "SELECT: activity on file descriptors.\n");
		
			// iterate through all file descriptors and check for event.
			map<int, FDCallback*>::iterator itr = fdCallbackTable.begin();
			while (itr != fdCallbackTable.end()) {
					
				char* recvBuf = new char[MAX_PKT_LEN];
				//TODO Remember to use delete not free
			
				FDCallback *cbk = (FDCallback *) itr->second;
				if (FD_ISSET(cbk->fd, &rfds)) {
					int numBytes = read(cbk->fd, recvBuf, MAX_PKT_LEN);
					PRINT_DEBUG(50, "Read %d  bytes\n", numBytes);
					cbk->call(recvBuf);
				}
				
				itr++;
			
				/* FD_ISSET(0, &rfds) will be true. */
			}
		} else {
			//Timeout occurred
			TimerEvent currEvent = timerEventQueue.top();
			PRINT_DEBUG(50, "Timeout occurred: %s\n\n", currEvent.toString(tmpStr));
			timerEventQueue.pop();
			//TODO action associated with timer event
		}
	}
}
