/**
 *  \file HopInfo.cpp
 *  \brief This file defines the various functions associated with a HopInfo.
 *  A HopInfo object represents a node in a multi-hop path.
*/
#include "HopInfo.h"
#include "../ocd/OCD.h"

HopInfo::HopInfo () {
    
    this->ocd = NULL;
    this->avoidAuthFlag = false;
}

HopInfo::HopInfo (const char *ocdId, const char *endPointName) {

    this->ocdId = ocdId;
    this->hopEndPointName = endPointName;
    this->avoidAuthFlag = false;

}

/**
  * Creates a copy of the specified hop 
  */
HopInfo::HopInfo ( HopInfo *hop) {

    this->ocdId = hop->ocdId;
    this->hopEndPointName = hop->hopEndPointName;
    this->ocd = hop->ocd;
    this->avoidAuthFlag = hop->avoidAuthFlag;

}
/**
  * Returns true if this hop is equal to the specified hop x.
  */
bool HopInfo::equals (HopInfo *x) {

    if ( 
            (this->hopEndPointName == x->hopEndPointName) &&
            (this->ocdId == x->ocdId)
        ) {

        return true;
    }

    printf ("\n\nHop Info MISMATCH %s,%s - %s %s\n\n", 
            this->hopEndPointName.c_str(),
            this->ocdId.c_str(),
            x->hopEndPointName.c_str(),
            x->ocdId.c_str()
        );
            
    return false;
}
