/**
 *  \file HopInfo.h
 *  \brief Declaration of a hop in a path.
*/

#ifndef HOP_INFO_H
#define HOP_INFO_H

#include <string>

using namespace std;

#include "../common.h"

class OCD;

/**
  * This encapsulates a single hop of a PathInfo.
  */
class HopInfo {


    /** The OCD Id to be used for this hop */
    string ocdId;

    /** This points to the actual OCD associated
      * with this hop.  It is a pointer to the overlay network
      * with id overlayId.
      */
    OCD *ocd;
    
    /** The name of the end point of this hop.
      * This resolves to some overlay specific id.
      */
    string hopEndPointName;

    /**
      * This flag is set to true to indicate that this hop need not be authenticated
      * even the rest of the path is security enabled.
      */
    bool avoidAuthFlag;
    

public:

    HopInfo();
    HopInfo (HopInfo *);
    HopInfo (const char *ocdId, const char *endPointName);

    bool equals (HopInfo *);

friend class PathInfo;
friend class PathInfoConfigFile;
friend class URLResolver;
friend class OCI;
friend class OCIState;
friend class CryptoState;

};

#endif
