/**
 *  \file PathInfoConfigFile.h
 *  \brief This file defines the various tags used in the outbound.xml 
 * configuration file as well as declares the parser class.
*/

#ifndef PATH_INFO_CONFIG_FILE_H
#define PATH_INFO_CONFIG_FILE_H

#include <list>

using namespace std;

class PathInfo;

#define PATH_INFO_ROOT_ELEMENT  "OutboundPathConfig"
#define PATH_INFO_TAG_PATH_INFO "PathInfo"
#define PATH_INFO_TAG_SECURITY  "Security"
#define PATH_INFO_TAG_COMPRESSION   "Compression"
#define PATH_INFO_TAG_HOP       "Hop"
#define PATH_INFO_TAG_FAKE_NEGOTIATION  "FakeAddressNegotiation"
#define PATH_INFO_TAG_MATCH     "Match"

#define PATH_INFO_ATTRIB_MATCH_URL_PATTERN      "urlPattern"
#define PATH_INFO_ATTRIB_MATCH_DST_PORT     "dstPort"
#define PATH_INFO_ATTRIB_MATCH_DST_IPADDR     "dstIPAddr"
#define PATH_INFO_ATTRIB_MATCH_SRC_PORT     "srcPort"
#define PATH_INFO_ATTRIB_MATCH_SRC_IPADDR     "srcIPAddr"
#define PATH_INFO_ATTRIB_MATCH_PROTOCOL         "protocol"

#define PATH_INFO_ATTRIB_SECURITY_PROTOCOL      "protocol"
#define PATH_INFO_ATTRIB_SECURITY_MODE          "mode"
#define PATH_INFO_ATTRIB_COMPRESSION_ALGO       "algo"
#define PATH_INFO_ATTRIB_COMPRESSION_LEVEL      "level"
#define PATH_INFO_ATTRIB_FAKE_NEGOTIATION_PREF  "preference"
#define PATH_INFO_ATTRIB_HOP_OVERLAY_ID         "overlayId"
#define PATH_INFO_ATTRIB_HOP_ENDPOINT_NAME      "hopEndPointName" 
#define PATH_INFO_ATTRIB_HOP_AUTHENTICATE       "authenticate"


#define SECURITY_PROTOCOL_CUSTOM_SSL_TXT    "customSSL"
#define SECURITY_PROTOCOL_SSL_TXT           "SSL"

#define COMPRESSION_ALGO_ZLIB_TXT           "zlib"

/**
  * This class is used to parse the path info configuration file.
  */
class PathInfoConfigFile {

public:

    static void loadPathInfo ( const char *pathInfoFileName, list<PathInfo *> *pathInfoList);

};

#endif
