// ChangeTunMTU.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <Winreg.h>
#include <string.h>

#include <iostream>
#include <tchar.h>

extern "C" {
#	include "debug.h"
}
/** The maximum length of string values in our registry entries */
#define MAX_REGISTRY_STR_VALUE_LEN 1000

char tmpBuf [10000];

LPCSTR GetErrorMsgStr (LPCSTR lpMsgBuf) {
	FormatMessage(
		      FORMAT_MESSAGE_FROM_SYSTEM |
		      FORMAT_MESSAGE_IGNORE_INSERTS,
		      NULL, GetLastError(), 0, (LPSTR) & lpMsgBuf, 0,
		      NULL);
	return lpMsgBuf;
}


int _tmain(int argc, _TCHAR* argv[])
{

	if (argc < 2) {
		printf ("\nUsage: %s <MTU>\n\n",
				argv[0]
				);
		return -1;
	}

	int targetMTU = atoi (argv[1]);
	if (targetMTU < 1200 || targetMTU > 1500) {
		printf ("The value of MTU specified (%d) is invalid. "
				"Please specify an MTU between 1200 and 1500.\n",
				targetMTU
				);
		return -1;
	}
	char targetMTUStr [100];
	strcpy (targetMTUStr, argv[1]);
	printf ("The target MTU for the tun device is %d.\n", targetMTU);

	LONG res;
	DWORD valType;
	char stringVal [MAX_REGISTRY_STR_VALUE_LEN];
	DWORD stringValSize = MAX_REGISTRY_STR_VALUE_LEN;

	// Open the System Key
	HKEY keySystem;
	res = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
							 "System",
							 0, /* Reserved must be 0 */
							 KEY_READ,
							 &keySystem
						);

	if (res != ERROR_SUCCESS) {
		printf (
						"Unable to open HKEY_LOCAL_MACHINE\\System\n"
						);
		return FALSE;
	}

	// Open the CurrentControlSet
	HKEY keyCurrentControlSet;	
	res = RegOpenKeyEx(		keySystem,
							"CurrentControlSet",
							 0, /* Reserved must be 0 */
							 KEY_READ,
							 &keyCurrentControlSet
						);


	if (res != ERROR_SUCCESS) {
		printf (
						"Unable to open HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\n"
						);
		RegCloseKey (keySystem);
		return FALSE;
	}


	// Open the Control
	HKEY keyControl;	
	res = RegOpenKeyEx(		keyCurrentControlSet,
							"Control",
							 0, /* Reserved must be 0 */
							 KEY_READ,
							 &keyControl
						);


	if (res != ERROR_SUCCESS) {
		printf (
						"Unable to open HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\n"
						);
		RegCloseKey (keyCurrentControlSet);
		RegCloseKey (keySystem);
		return FALSE;
	}

	// Open the Class
	HKEY keyClass;	
	res = RegOpenKeyEx(		keyControl,
							"Class",
							 0, /* Reserved must be 0 */
							 KEY_READ,
							 &keyClass
						);


	if (res != ERROR_SUCCESS) {
		printf (
						"Unable to open HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\n"
						);
		RegCloseKey (keyControl);
		RegCloseKey (keyCurrentControlSet);
		RegCloseKey (keySystem);	
		
		return FALSE;
	}


	// Open the Network adapters
	HKEY keyNetworkAdapters;	
	res = RegOpenKeyEx(		keyClass,
							"{4D36E972-E325-11CE-BFC1-08002bE10318}",
							 0, /* Reserved must be 0 */
							 KEY_READ,
							 &keyNetworkAdapters
						);


	if (res != ERROR_SUCCESS) {
		printf (
						"Unable to open HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\n"
						);
		RegCloseKey (keyClass);
		RegCloseKey (keyControl);
		RegCloseKey (keyCurrentControlSet);
		RegCloseKey (keySystem);	
		
		return FALSE;
	}

	
	// Go through each one of the Network Adapters looking for the tun adapter.
	int currAdapterIndex = 0;
	char currAdapterId[MAX_REGISTRY_STR_VALUE_LEN];
	DWORD currAdapterIdLen = MAX_REGISTRY_STR_VALUE_LEN;
	while (TRUE) {
		// Note: The name of the sub key is the ocd's id.
		currAdapterIdLen = MAX_REGISTRY_STR_VALUE_LEN;
		res = RegEnumKeyEx(
							keyNetworkAdapters,
							currAdapterIndex,
							currAdapterId,
							&currAdapterIdLen,
							NULL,
							NULL,
							NULL,
							NULL
					);

		if (res != ERROR_SUCCESS && res != ERROR_NO_MORE_ITEMS) {
			printf ( "Error while enumerating network adapters: %s.\n",
						GetErrorMsgStr (tmpBuf)
					);
			break;
		} else if (res == ERROR_NO_MORE_ITEMS) {
				break;
		}
		currAdapterIndex++;

		
		// Open the key associated with the current adapter
		HKEY keyCurrAdapter;	
		res = RegOpenKeyEx( keyNetworkAdapters,
							 currAdapterId,
							 0, /* Reserved must be 0 */
							 KEY_ALL_ACCESS,
							 &keyCurrAdapter
						);

		if (res != ERROR_SUCCESS) {
			printf ("Error while opening network adapter %s: %s\n",
					currAdapterId,
					GetErrorMsgStr (tmpBuf)
				);
			continue;
		}


		// Get the MatchingDeviceId
		stringValSize = MAX_REGISTRY_STR_VALUE_LEN;
		res = RegQueryValueEx(
						keyCurrAdapter,
						"MatchingDeviceId",
						NULL,
						&valType,
						(BYTE*) stringVal,
						&stringValSize
					);

		if (res != ERROR_SUCCESS) {
			printf ("Error while reading the MatchingDeviceId of adapter %s: %s\n",
				currAdapterId,
				GetErrorMsgStr (tmpBuf)
			);
			RegCloseKey (keyCurrAdapter);
			continue;
		}

		// TODO Look for OCALA's own TUN device id and not
		// OpenVPNs tun device
		if (stricmp (stringVal, "tap0801") == 0) {
			printf ("Adapter %s is a tun device. Changing its MTU to %d\n",
					currAdapterId, targetMTU
				);
				res = RegSetValueEx(
							keyCurrAdapter,
							"MTU",
							0,
							REG_SZ,
							(BYTE*) targetMTUStr,
							strlen (targetMTUStr) + 1
						);
				if (res != ERROR_SUCCESS) {
					printf ("Unable to set the MTU of adapter %s to %s: %s\n",
							currAdapterId, targetMTUStr,
							GetErrorMsgStr (tmpBuf)
							);

				}
		}

		RegCloseKey (keyCurrAdapter);
  	} // while (TRUE)
		







	RegCloseKey (keyNetworkAdapters);
	RegCloseKey (keyClass);
	RegCloseKey (keyControl);
	RegCloseKey (keyCurrentControlSet);
	RegCloseKey (keySystem);	

}

