/**
 * \file This file contains some general utility functions.
 */

#include <afxwin.h>
#include <string>
using namespace std;

/**
 * Display either the specified error message or
 * if no error message is specified (i.e. szError is NULL),
 * display an error message corresponding to the last error 
 * that occurred.
 */
void ShowErrorMessage(LPCSTR szError) {
    
	LPVOID          lpMsgBuf = NULL;

	if (szError) {
		// Error Message was specified.
		MessageBox(NULL, szError, "Error", MB_OK | MB_ICONEXCLAMATION);
	
	}else {
		// Display the message corresponding to the error
		// that occurred last.
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
		      FORMAT_MESSAGE_FROM_SYSTEM |
		      FORMAT_MESSAGE_IGNORE_INSERTS,
		      NULL, GetLastError(), 0, (LPSTR) & lpMsgBuf, 0,
		      NULL);
		MessageBox(NULL, (LPCSTR) lpMsgBuf, "Error",
					   MB_OK | MB_ICONEXCLAMATION);
		LocalFree(lpMsgBuf);
    }
}

/**
  * This function just strips the whitespace in the specified string.
  * @param str The string to stripped of whitespace.
  */
void stripWS(char* str) {
    char tstr[1000];
  
    int lindex = 0;
    while ( str[lindex] != 0 && isspace(str[lindex]))
        lindex++;

    int rindex = strlen(str)-1;
    while ( rindex >= 0 && isspace(str[rindex]) )
        rindex--;

    if ( lindex == strlen(str)) {
        strcpy(str,"");
        return;
    }

    str[rindex+1] = 0;
    strcpy(tstr,str+lindex);
    strcpy(str,tstr);
}

/**
  * This function converts the given character array string to lower
  * case and places the result in the destionation argument.
  * @param src The character array string to be converted to lower case.
  * @param dst The dstination array to hold the converted string.
  */
void toLowerCase(const char *src, char *dst) {

    int i, len;
    len = strlen(src);
    for(i=0; i < len; i++) {
        dst[i] = tolower(src[i]);
    }
    dst[i] = '\0';
}

/**
  * This function converts the given character array string to lower
  * case and returns the resultant string.
  * @param src The character array string to be converted to lower case.
  */
string toLowerCase(string src) {

    int i, len;
	const char *tmpSrc = src.c_str();

	len = strlen(tmpSrc);
	char *dst = new char[len + 1];
	
    for(i=0; i < len; i++) {
        dst[i] = tolower(tmpSrc[i]);
    }
    dst[i] = '\0';

	string retStr = dst;

	delete dst;

	return retStr;

}
