#include "OCDStatus.h"
#include "GenFunctions.h"

#include "StringTokenizer.h"

#include "debug.h"
#include "resource.h"

/**
 * Constructor
 * @param iconList Name of icon files, separated by ','
 * @param currentIcon The name of one of the icons in the iconList which 
 * should be currently displayed.
 */
OCDStatus::OCDStatus (
							string id, 
							string type, 
							string description, 
							string statusMsg, 
							DWORD statusCode,
							string iconList,
							string currentIconName) {

	this->id = id;
	this->type = type;
	this->description = description;
	this->statusCode = statusCode;
	this->statusMsg = statusMsg;
	this->iconList = iconList;
	this->currentIconName = currentIconName;

	string tmpToolTip = getToolTipText();
/*
	loadIcons();

	HICON currIcon = getCurrentIcon();
	

	// Init/Create the system tray icon
	// Create the tray icon
	if (!m_TrayIcon.Create(
                        NULL,                            // Let icon deal with its own messages
                        WM_ICON_NOTIFY,                  // Icon notify message to use
                        _T(tmpToolTip.c_str()),  // tooltip
                        currIcon,
                        IDR_POPUP_MENU,                  // ID of tray icon
                        FALSE,
                        NULL,							// balloon tip
                        NULL,							// balloon title
                        NIIF_WARNING,                    // balloon icon
                        20 ))                            // balloon timeout
    {
		m_iconInitFlag = false;
		ShowErrorMessage (NULL);
		return; 
    }

	m_TrayIcon.popupMenuFlag = false;
	// System tray icon was successfully initialized.
	m_iconInitFlag = true;
*/
}

/** Destructor.
 */
OCDStatus::~OCDStatus() {
/*
	// Remove the icon of this OCD if it had been initialized.
	if (m_iconInitFlag) {
		m_TrayIcon.RemoveIcon();	
	}
	*/
}

void OCDStatus::update (string description, string statusMsg, DWORD statusCode, string currIconList, string currIconName, bool redrawIconFlag) {
	this->currentIconName = currIconName;
	this->description = description;
	this->statusMsg = statusMsg;
	this->statusCode = statusCode;
/*
	if (currIconList != string("") && currIconList != this->iconList) {
		// Load Icons again
		loadIcons();
	}

	if (redrawIconFlag) {
		updateIcon();
	}
	*/
}

/**
 * Update the system tray icon with the current ocd status.
 */
void OCDStatus::updateIcon () {
	/*
	m_TrayIcon.SetTooltipText (getToolTipText().c_str());

	HICON tmpIcon = getCurrentIcon ();
	if (tmpIcon != NULL) {
		m_TrayIcon.SetIcon (tmpIcon);
	}
	*/
}

string OCDStatus::getToolTipText() {

	return id + string (" - ") + description + string ("\n") + statusMsg;
}

/** Loads all the icons present associated with this OCD.
 * If an icon file is not found, the default OCD icon is loaded.
 */
void OCDStatus::loadIcons() {
	
	// first clean up any existing loaded icons
	map<string, HICON>::iterator itr = map_Icon_name.begin();
	while (itr != map_Icon_name.end()) {
		DestroyIcon (itr->second);
		itr++;
	}
	map_Icon_name.clear();


	StringTokenizer st (iconList, string(","));
	while (st.hasMoreTokens()) {
	
		char tmpStr[1000];
		strncpy (tmpStr, st.nextToken().c_str(), 999);
		stripWS (tmpStr);
        string currName = tmpStr;
		

		string iconPath = string ("icons\\") + type + string("\\") + currName;

		// Load the icon to use for this OCD.
		// The icon is formed
		HICON hIcon = (HICON) ::LoadImage ( 0,	// Load icon from a file
								iconPath.c_str(),
								IMAGE_ICON,
								0, // actual X size
								0, // actual Y size
								LR_LOADFROMFILE
							);

		if (hIcon == NULL) {
			// The icon corresponding to the OCD could not be found.
			// So load the default OCD icon.
			iconPath = "icons\\default\\ocd_main.ico";

			// Load the icon to use for this OCD.
			// The icon is formed
			hIcon = (HICON) ::LoadImage ( 0,	// Load icon from a file
								iconPath.c_str(),
								IMAGE_ICON,
								0, // actual X size
								0, // actual Y size
								LR_LOADFROMFILE
							);
			if (hIcon == NULL) {
				// Even the default icon failed to load.
				PRINT_DEBUG1 (DEBUG_LEVEL_WARNING,
								"The icon file %s could not be found.\n",
								iconPath.c_str()
							);

				ShowErrorMessage (NULL);
				m_iconInitFlag = true;
			}
		}

		// put the mapping
		map_Icon_name.insert (pair<string, HICON>(currName, hIcon));

	}
}

/** Returns a handle to the current icon.
 */
HICON OCDStatus::getCurrentIcon () {
	map<string, HICON>::iterator itr = map_Icon_name.find (currentIconName);
	if (itr != map_Icon_name.end()) {
		return itr->second;
	} else {
		return NULL;
	}
}

/**
 * Display a balloon with the specified message near the icon of
 * this OCD.
 */
void OCDStatus::showBalloon (string msg) {
	/*
	m_TrayIcon.ShowBalloon ( msg.c_str(), id.c_str());
	*/
}