#!/bin/sh
#
# /etc/init.d/mysystem
# Subsystem file for the OCALA proxy
# Author: Dilip Antony Joseph
#
# chkconfig: 2345 95 05	
# description: OCALA proxy daemon
#
# processname: OCALA_proxy
# config: /etc/sysconfig/OCALA.conf

# source function library
. /etc/rc.d/init.d/functions

# pull in sysconfig settings
#[ -f /etc/sysconfig/OCALA.conf ] && . /etc/sysconfig/OCALA.conf	

OCALA_INSTALL_DIR=/usr/local/OCALA

RETVAL=0
prog="OCALA_proxy"

start() {	
	echo -n $"Starting $prog:"

    cd $OCALA_INSTALL_DIR
    daemon ./OCALA_proxy -c -d

	RETVAL=$?
	[ "$RETVAL" = 0 ] && touch /var/lock/subsys/$prog
	echo
}

stop() {	
	echo -n $"Stopping $prog:"
	
    killproc $prog -TERM
	RETVAL=$?
	[ "$RETVAL" = 0 ] && rm -f /var/lock/subsys/$prog
	echo
}

reload() {	
	echo -n $"Reloading $prog:"
	killproc $prog -HUP
	RETVAL=$?
	echo
}

case "$1" in	
	start)
		start
		;;
	stop)
		stop
		;;
	restart)
		stop
		start
		;;
	reload)
		reload
		;;
	condrestart)
		if [ -f /var/lock/subsys/$prog ] ; then
			stop
			# avoid race
			sleep 3
			start
		fi
		;;
	status)
		status $prog
		RETVAL=$?
		;;
	*)	
		echo $"Usage: $0 {start|stop|restart|reload|condrestart|status}"
		RETVAL=1
esac
exit $RETVAL
