/**
  * \file RSPServerFinder.h
  * \brief This class is used to find the IP address of the current closest Remote Server Proxy.
  */

#include <string>
#include <list>
#include <map>
using namespace std;

#if defined(_WIN32)
#   include "fwint.h"
#else
#   include <netdb.h>
#endif

class RSPTunnelOCD;

/** Default interval (in seconds) at which the different rspDomainLists are resolved.
  */
#define RSP_SERVER_FINDER_DEFAULT_RESOLVE_PERIOD  10

/**
  * This class is used to find the IP address of the current closest Remote Server Proxy.
  */
class RSPServerFinder {
public:
    RSPServerFinder (list<string> rspDomainList, RSPTunnelOCD* ocd, int32_t resolvePeriod = RSP_SERVER_FINDER_DEFAULT_RESOLVE_PERIOD);
    bool lookupDomain (string domainName, uint32_t* retIP);
    void addMapping (string domainName, uint32_t ip);
	int findNumValidMappings ();
#ifdef _WIN32
	string getMappingStatusStr();
#endif


private:

    /** Pointer to the RSPTunnelOCD */
    RSPTunnelOCD* ocd;
    
    /** List of known RSP domains.
      * When you resolve an RSP domain, you get the IP address of the closest node in that domain.
      * For example, ocalaRSP.nyuld.net, ocalaRSP-secure.nyuld.net are two different RSPs
      * made possible by OASIS.
      */
    list<string> rspDomainList;        

    /**
      * How often should we resolve the RSP domains?
      * (in seconds)
      */
    int32_t resolvePeriod;

    /** Domain Name to IP address (host byte order) mappings. */
    map<string, uint32_t> map_domain_ip;

    void resolveAllDomains ();
    void doPeriodicResolve ();


};
