/**
 *  \file RSPTunnelOCDStateInfo.cpp
 *  \brief The definition of functions and constructors of the RSPTunnelOCDStateInfo.
*/

#ifndef _WIN32

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#else 
#	include "netwrap.h"
#endif

#include "OCD_include.h"

#include "RSPTunnelOCDStateInfo.h"
#include "RSPTunnelOCD.h"
#include "RSPTunnelOCDContext.h"
#include "RSPTunnelOCD_debug.h"
#include "TFunctor.h"

RSPTunnelOCDStateInfo::RSPTunnelOCDStateInfo(
			OCDHandle myHandle, OCI* oci, RSPTunnelOCD *overlay,
			string endPointName
			/* OCD specific parameters */
		)
            : OCDStateInfo (myHandle, oci, overlay) {

    // OCD specific initialization
	strcpy (this->endPointName, endPointName.c_str());
	this->fakeSetupSuccessTimer = NULL;
} 

/**
  * This function exists only in order for notifySetupSuccess to be called asynchronously
  * by refreshState
  */
void RSPTunnelOCDStateInfo::fakeSetupSuccess() {
	if (this->fakeSetupSuccessTimer != NULL) {
		this->fakeSetupSuccessTimer = NULL;
	}
    PRINT_DEBUG0 (DEBUG_LEVEL_VERBOSE, "RSPTunnelOCD___NOTIFY_SETUP_SUCCESS_STATE\n");
    notifySetupSuccess();
}

RSPTunnelOCDStateInfo::~RSPTunnelOCDStateInfo () {

	if (this->fakeSetupSuccessTimer != NULL) {
		this->fakeSetupSuccessTimer->setDisabled();
		this->fakeSetupSuccessTimer = NULL;
	}

	RSPTunnelOCD* rspTunnelOCD = (RSPTunnelOCD*) this->ocd;
	rspTunnelOCD->destroyOCDState (this);

}	

/**
 * Set up a timer that will be invoked after 1 microsecond.
 * The timer's sole function is to make setup notification success asynchronous
 * when it can actually be synchronous.
 */
void RSPTunnelOCDStateInfo::setFakeSetupTimer () {

	// Disable any existing fake setup timer
	if (this->fakeSetupSuccessTimer != NULL) {
		this->fakeSetupSuccessTimer->setDisabled();
		this->fakeSetupSuccessTimer = NULL;
	}

	TObjectFunctor<RSPTunnelOCDStateInfo> *tmpFakeSetupSuccessFunctor  = 
                                     new TObjectFunctor<RSPTunnelOCDStateInfo> (this, &RSPTunnelOCDStateInfo::fakeSetupSuccess);
           
	this->fakeSetupSuccessTimer = TimerEvent::create (0, 1, (TFunctor *) tmpFakeSetupSuccessFunctor, true /* automatically free cbk when TimerEvent is destroyed*/);
    this->ocd->addTimerEvent(this->fakeSetupSuccessTimer);


}
