# This script creates a new proxy-config.xml using the command line arguments.
# It first backsup any existing proxy-config.xml

use strict;

my $numArgs = $#ARGV + 1;
if ($numArgs < 2) {
    
    die "Usage: perl create-proxy-config.pl <path to proxy-config.xml> <path to tun device>\n"; 
}

my $targetFile = shift (@ARGV);
my $tunDevPath = shift (@ARGV);

# Backup existing target file
if (-f $targetFile) {
    print "Backing up $targetFile to $targetFile.backup.\n";
    `cp $targetFile $targetFile.backup`;
}

# Open a new target file for writing
print "Creating $targetFile ...\n";
open OUTFD, ">$targetFile" || die "Unable to open $targetFile for writing.\n";


print OUTFD <<END
<?xml version="1.0"?>

<ProxyConfigFile 
	stdDNSRequestPort="53"
	legacyDNSRequestPort="55667"
	fakeAddrRangeStart="1.0.0.2"
	fakeAddrRangeMask="255.255.255.0"
	myNamesConfigFile="config_files/myNames.xml"
	outboundConnectionConfigFile="config_files/outbound.xml"
 >


<OC-D-List>
	<OC-D
		type="I3"
		id="pli3"
		description="I3 Overlay Network running on PlanetLab"
		configFile="config_files/i3-cfg.xml"
		libFileName_windows="i3OCD_windows.dll"
		libFileName_linux="./libI3OCD.so"
		libFileName_apple="./libI3OCD.dylib"
		active="yes"
		defaultUrlSuffix=".pl.i3"
		I3_EnableShortcut="yes"
		I3_AddressBook="config_files/i3_addr_book.txt"
		I3_EnableFastConnectionStartup="no"
		I3_LocalPortNum="58748"
	/>

    <OC-D
    	type="sampleOCD"
		id="sampleOCD"
		description="A Sample OCD"
		configFile="config_files/sampleOCD.xml"
		active="yes"

		libFileName_windows="libSampleOCD.dll"
        	libFileName_linux="./libSampleOCD.so" 
        	libFileName_apple="./libSampleOCD.dylib" 

     />
     <OC-D
    		type="RSPTunnelOCD"
		id="rspTunnelOCD"
		description="A simple tunnel for the remote server proxy"
		configFile="config_files/RSPTunnelOCD.xml"
		active="yes"
        	libFileName_linux="./libRSPTunnelOCD.so" 
        	libFileName_apple="./libRSPTunnelOCD.dylib" 
		libFileName_windows="RSPTunnelOCD.dll"
     />


</OC-D-List>


<Tunnel>
        <DevicePath>$tunDevPath</DevicePath>
        <DeviceName>tun0</DeviceName>
        <MTU>1200</MTU>
        <IPAddress>10.0.0.1</IPAddress>
        <RoutingTable>tuntab</RoutingTable>

        <ifconfigCmd>/sbin/ifconfig</ifconfigCmd>
        <iptablesCmd>/sbin/iptables</iptablesCmd>
        <ipCmd>/sbin/ip</ipCmd>
        <modprobeCmd>/sbin/modprobe</modprobeCmd>
	
	<AppleCapturePort>13346</AppleCapturePort>
    </Tunnel>

    <Security
            SecurityLevel="1"
            CA_CertificateFile="config_files/certificates/CA_cert.pem"
        />

</ProxyConfigFile>
END
;

close (OUTFD);
