#!/bin/bash
# This script is used to fill in the name entered by the user 
# into the position in the specified myNames.xml marked by $FILL_IN_TAG

FILL_IN_TAG="MY_NAME_I3_FILL_IN"

if [ $# -lt 1 ]
then
    echo
    echo "Usage: ./fillInMyName.sh <myNames.xml config file path>"
    echo
    exit
fi

CONFIG_FILE=$1

# Check if the specified configuration file exists.
if [ ! -f $CONFIG_FILE ]
then
    echo 
    echo "Specified file $CONFIG_FILE does not exist.  Please specify a valid file."
    echo
    exit
fi

# Get the name from the user.
# Propose a name based on the host name of this machine
TMP_HOST_NAME=`hostname`
FILL_IN_NAME=`echo "$TMP_HOST_NAME" | awk -F '.' ' {print $1 ".pli3"} '`

TMP_CHECK=0
while [ $TMP_CHECK -eq 0 ]
do
    echo "Please enter a name for this machine (name should end in .pli3): [$FILL_IN_NAME]"
    read ans
    
    if [ ! $ans ]
    then
        TMP_CHECK=1
    else
        # Check if the name entered by the user ends in .pli3
        echo "$ans" | grep "\.pli3$" 2>&1>/dev/null
        if [ $? -eq 0 ]
        then
            FILL_IN_NAME=$ans
            TMP_CHECK=1
        fi
    fi
done

# Backup existing myNames.xml file
if [ -f $CONFIG_FILE ]
then
    cp $CONFIG_FILE $CONFIG_FILE.backup
fi

# Create the new myNames.xml file
perl createMyNames.pl $CONFIG_FILE $FILL_IN_NAME
