#include "HipOCDStateInfo.h"
#include "HipOCD.h"

#include "OCD_include.h"

HipOCDStateInfo::HipOCDStateInfo(
				 OCDHandle myHandle, OCI* oci, HipOCD *hipOCD
				 /* OCD specific parameters */
		)
  : OCDStateInfo (myHandle, oci, hipOCD) {
  
  // OCD specific initialization
  this->hipOCD = hipOCD;
  this->status = OCD_STATE_STATUS_NOT_READY;

  peer_addr.s_addr = INADDR_ANY;
 } 


void HipOCDStateInfo::receivedLegacyDNSReply(uint32_t ip) {
  char cmd[MAX_PRINT_BUF];
  char peer_addr_a[MAX_PRINT_BUF];
  //char proxy_addr_a[MAX_PRINT_BUF];
  
  if (ip == 0) {
    notifySetupFailure();
    return;
  }

  peer_addr.s_addr  = htonl(ip);

  strcpy(peer_addr_a, inet_ntoa(peer_addr));  
  //strcpy(proxy_addr_a, inet_ntoa(hipOCD->proxy_addr.sin_addr));  

  // add newly created state to hash table keyed by end point name.
  hipOCD-> putEndPoint  (peer_addr_a, this);

  // set the name of the end point of the tunnel.
  strcpy (endPointName, peer_addr_a);

  PRINT_DEBUG (
	       DEBUG_LEVEL_SUPER,
	       "HipOCD state with handle %d received DNS reply: IP = %s.\n",
	       myHandle,
	       inet_ntoa (peer_addr)
	       );
  
  // Do the OCD specific tunnel setup here.
  snprintf(cmd, sizeof(cmd),
	   "echo \"spdadd %s %s any -P out ipsec hip/transport//require;\" >/tmp/hipocd ", 		     
	   inet_ntoa(hipOCD->proxy->get_my_addr_nf()),
	   peer_addr_a);
  
  system(cmd);
  system(SET_KEY);
  printf("%s\n", cmd);

  snprintf(cmd, sizeof(cmd),
	   "ping -c 1 %s", 		     
	   peer_addr_a);  
  system(cmd);

  notifySetupSuccess(); 
}

