/**
  * \file i3Debug.h
  * \brief This file defines the constants used as debugging levels
  * in the i3 specific code.
  */

/**
  * This info level is used for displaying fatal error messages.
  * Always displayed.
  */
#define I3_INFO_LEVEL_FATAL_ERROR 0

/**
  * This info level is used for messages which are very essential.
  * Messages at this level will always be displayed.
  */
#define I3_INFO_LEVEL_MINIMAL  50

/**
  * This info level is used for warning messages.
  */
#define I3_INFO_LEVEL_WARNING  60

/**
  * The messages at this level are not critical.  So they may not be
  * displayed (depends on CURRENT_INFO_LEVEL).
  */
#define I3_INFO_LEVEL_VERBOSE  70

/**
  * This is used for error messages which are probably caused due to bugs in the 
  * code.  i.e. they are not INFO_LEVEL_WARNINGs
  */
#define I3_DEBUG_LEVEL_WARNING   20


/**
  * This is used for messages describing fatal errors.
  */
#define I3_DEBUG_LEVEL_FATAL   0

/**
  * This is used for messages that are not fatal, but quite important
  * or high level.
  */
#define I3_DEBUG_LEVEL_MINIMAL     30

/**
  * This is used for messages that are not very important.
  */
#define I3_DEBUG_LEVEL_VERBOSE     90


