#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>   /* basic system data types */
#include <sys/socket.h>  /* basic socket definitions */
#include <arpa/inet.h>
#include <sys/time.h>    /* timeval{} for select() */
#include <errno.h>    
#include <sys/utsname.h>
#include <time.h>                /* timespec{} for pselect() */
#include <netinet/in.h>  /* sockaddr_in{} and other Internet defns */
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include "i3.h"
#include "i3_fun.h"
#include "i3_api.h"
#include "i3_server.h"

#if NEWS_INSTRUMENT

#include "i3_server_info.h"
#include "i3_news.h"
#include "i3_news_instrument.h"

#endif

#include "chord_api.h"
#include "../utils/utils.h"

#define TRIVIAL_LOCAL_ADDR	"127.0.0.1"

extern unsigned short srv_port;


int is_id_local(ID *id,srv_context* ctx)
{
    chordID key;
  
    memmove(key.x, id, CHORD_ID_BITS/8 * sizeof(char));
    return chord_is_local(&key);

}


struct in_addr get_my_addr()
{
  struct hostent *hptr;
  struct utsname myname;
  char str[INET6_ADDRSTRLEN];
  struct sockaddr_in servaddr;

  uint32_t addr;
  addr = get_local_addr();
  servaddr.sin_addr.s_addr = ntohl(addr);
  return servaddr.sin_addr;
}
  

i3_option *create_i3_option_req_cache()
{
    i3_option	*opt;

    opt = alloc_i3_option();
    init_i3_option(opt, I3_OPT_REQUEST_FOR_CACHE, NULL);
    return opt;
}

i3_option *create_i3_option_sender(srv_context *ctx)
{
    i3_option	*opt;
    i3_addr	*addr;

    addr = alloc_i3_addr();
    init_i3_addr_ipv4(addr, ctx->local_ip_addr, ntohs(ctx->local.sin_port));
    
    opt = alloc_i3_option();
    init_i3_option(opt, I3_OPT_SENDER, (void *)addr);
    return opt;
}


/***************************************************************************
 *
 * Purpose: Pop and replace hdr->stack
 *
 * Return: true if number of ids does not grow beyond I3_MAX_STACK_LEN
 *
 **************************************************************************/
i3_header *replace_stack(i3_header *hdr, i3_addr *to)
{
    unsigned short	to_stack_len, hdr_stack_len;
    
    i3_header 		*new_hdr = alloc_i3_header();
    i3_stack		*new_stack = alloc_i3_stack();
    i3_option_list	*new_option_list;

    new_option_list = duplicate_i3_option_list(hdr->option_list);
    
    /* check if new stack is permissible */
    new_stack->len = hdr->stack->len + to->t.stack->len - 1;
    if (new_stack->len > I3_MAX_STACK_LEN) {
	printf("cannot have more than %d triggers in a stack!\n",
		I3_MAX_STACK_LEN);
	return NULL;
    }

    /* pack new_stack */
    new_stack->ids = (ID *)malloc(sizeof(ID) * new_stack->len);

    to_stack_len = (to->t.stack->len) * sizeof(ID);
    memcpy((char *)new_stack->ids, (char *)to->t.stack->ids, to_stack_len);
    
    hdr_stack_len = (hdr->stack->len - 1) * sizeof(ID);
    memcpy((char *)new_stack->ids + to_stack_len,
	    (char *)&hdr->stack->ids[1], hdr_stack_len);
    
    /* setup new header */
    init_i3_header(new_hdr, hdr->flags, new_stack, new_option_list);

    return new_hdr;
}


/* NAT ADDITION */
/* rewrite ip address */
void nat_translate_sender_ip_option(char* option,i3_addr* to_addr)
{
  unsigned short len;
  // translate all ips to to_addr
  assert (*(option+1) == I3_ADDR_TYPE_IPv4);
  pack_i3_addr(option+1,to_addr,&len);
  //assert(len == sizeof(unsigned short) + sizeof(unsigned int) + 1);
}

/* NAT ADDITION */
/* check options in packet, and decide whether is natted, 
 * whether trigger is inserted, removed 
 */
void checkoptions(char *pkt, char* natted, char* insert,
		  char* remove, char* data, i3_addr* real_addr)
{
  unsigned short lenp, leno;
  char *p;
  i3_option* option;

  p = get_hdr_options(pkt);
  lenp = ntohs(*(unsigned short *)p); /* get option list's length */
  lenp -= sizeof(unsigned short); /* skip option list's length field */
  p += sizeof(unsigned short);

  if ( (get_flags(pkt) & I3_DATA) != 0 )
    *data = 1;

  while (lenp)
  {

    option = unpack_i3_option(p, &leno);
    
    if  (option->type == I3_OPT_SENDER && 
	 option->entry.ret_addr->type == I3_ADDR_TYPE_IPv4 && 
	 !addr_equal(option->entry.ret_addr, real_addr))
      *natted = 1;
    
    if ( option->type == I3_OPT_TRIGGER_REMOVE )
      *remove = 1;
    
    if ( option->type == I3_OPT_TRIGGER_INSERT )
      *insert = 1;
    
    p += leno;
    lenp -= leno;
    free_i3_option(option);
  }
  
}




