#!/bin/bash
# This script is intended only to be run on Mac OS X.
# This script is used targetted at first time users
# compiling the proxy from the source distribution.
# It hopes to make compiling the proxy, all the bundled OCDs
# and configuring the proxy a one step and interactive
# process from the commandline.
# AUTHOR: Dilip Antony Joseph

# ASSUMPTION: This script is run from the OCALA_Suite directory.
# Check this assumption
if [ ! -d OCALA    -o      ! -d i3 -o    ! -d sampleOCD 	-o ! -d RSPTunnelOCD ]
then
    echo
    echo "Unable to find some of the following required directories: OCALA, i3, sampleOCD, RSPTunnelOCD."
    echo "Please make sure that you are running this script from within the OCALA_Suite directory."
    echo
    exit
fi

# ASSUMPTION: You are using the quick install on a fresh src distribution.
# That is you haven't compiled anything using quickStart or manually before.
# Check this assumption.
if [ -f OCALA/generic_proxy/OCALA_proxy    -o     -f i3/i3OCD/libI3OCD.dylib    -o    -f sampleOCD/libSampleOCD.dylib 	-o	-f RSPTunnelOCD/libRSPTunnelOCD.dylib ]
then
    echo
    echo "Found compiled versions of OCALA_proxy, libI3OCD.dylib or libSampleOCD.dylib."
    echo "This means that you have probably used quickStart or manual compilation on this directory before."
    ans="invalid"
    while [ $ans != "y"  -a  $ans != "Y"  -a   $ans != "N"   -a  $ans != "n" ]
    do
        echo "Do you wish to continue? (yYnN)"
        read ans

        if [ ! $ans ]
        then
            ans="invalid"
        fi
    done

    if [ $ans = "n"  -o  $ans = "N" ]
    then
        echo
        echo "Cancelling quickStart on user request."
        echo
        exit
    fi
fi 


# Delete any of the final output files. 
# Just to make sure we get only the latest ones.
if [ -f OCALA/generic_proxy/OCALA_proxy ]
then
	rm -rf OCALA/generic_proxy/OCALA_proxy
fi

if [ -f i3/i3OCD/libI3OCD.dylib ]
then
	rm -rf i3/i3OCD/libI3OCD.dylib
fi

if [ -f sampleOCD/libSampleOCD.dylib ]
then
	rm -rf sampleOCD/libSampleOCD.dylib
fi

if [ -f RSPTunnelOCD/libRSPTunnelOCD.dylib ]
then
	rm -rf RSPTunnelOCD/libRSPTunnelOCD.dylib
fi


# Clear the screen
clear

# Print the welcome information

echo -e "\033[34m" # Set blue text color
echo "Welcome to OCALA Quick Start"
echo -e "\033[0m" # Restore normal text color

# Get the current directory
CURRDIR=`pwd`
# delete any previous log file
if [ -f $CURRDIR/quickStart.log ]
then
    rm $CURRDIR/quickStart.log
fi

echo "All compile time messages are logged in quickStart.log"
echo

# Compiling OCALA
echo "Compiling OCALA proxy (This may take a few minutes)..."
cd OCALA/generic_proxy
./make.apple >> $CURRDIR/quickStart.log 2>&1
cd ../..
# Check if the OCALA_proxy binary has been created
if [ ! -f OCALA/generic_proxy/OCALA_proxy ]
then
    echo
    echo "Unable to find OCALA_proxy binary in OCALA/generic_proxy/OCALA_proxy."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile the OCALA as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# OCALA_proxy has been successfully compiled
echo "OCALA_proxy successfully compiled."
echo


# Compiling i3
echo "Compiling i3 (This may take a few minutes) ..."
cd i3
./make.apple >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the i3 OCD has been created
if [ ! -f i3/i3OCD/libI3OCD.dylib ]
then
    echo
    echo "Unable to find libI3OCD.dylib in i3/i3OCD."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile i3 as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# i3 has been successfully compiled
echo "i3 has been successfully compiled."
echo

# Compiling sampleOCD
echo "Compiling sampleOCD (This may take a few minutes) ..."
cd sampleOCD
./make.apple >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the sample OCD has been created
if [ ! -f sampleOCD/libSampleOCD.dylib ]
then
    echo
    echo "Unable to find libSampleOCD.dylib in sampleOCD."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile sampleOCD as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# sampleOCD has been successfully compiled
echo "sampleOCD has been successfully compiled."
echo

# Compiling RSPTunnelOCD
echo "Compiling RSPTunnelOCD (This may take a few minutes) ..."
cd RSPTunnelOCD
./make.apple >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the RSPTunnelOCD has been created
if [ ! -f RSPTunnelOCD/libRSPTunnelOCD.dylib ]
then
    echo
    echo "Unable to find libRSPTunnelOCD.dylib in RSPTunnelOCD."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile RSPTunnelOCD as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# RSPTunnelOCD has been successfully compiled
echo "RSPTunnelOCD has been successfully compiled."
echo



echo
echo "All required files have been successfully created."

./quickStart.makeProxyConfig.sh


# Copy the i3, SampleOCD, RSPTunnelOCD files to the generic_proxy directory
echo "Copying i3, SampleOCD, RSPTunnelOCD libraries to OCALA/generic_proxy directory..."
cp i3/i3OCD/libI3OCD.dylib OCALA/generic_proxy
cp sampleOCD/libSampleOCD.dylib OCALA/generic_proxy
cp RSPTunnelOCD/libRSPTunnelOCD.dylib OCALA/generic_proxy

# For the remaining operations, we might need superuser privileges.
# Check for that.
#test $UID != 0 && echo "You need root permissions to continue installing OCALA. Aborting." && exit -1

# Create the proxy-config.xml configuration file
perl ./create-proxy-config.pl OCALA/generic_proxy/config_files/proxy-config.xml  /dev/tun0 

echo
echo
echo

# Launch ProxyConfig GUI or cmd line config utility
which java 2>/dev/null 1>/dev/null
if [ $? -eq 0 ] && [ -f OCALA/generic_proxy/ProxyConfig.jar ]
then
   	echo "Launching Java ProxyConfig GUI ..."
    echo
    cd OCALA/generic_proxy
    java -jar ProxyConfig.jar
    cd -
else
    echo "java or ProxyConfig.jar not found.  Launching limited command line configuration..."
    echo

	ans="invalid"
	while [ $ans != "y"  -a  $ans != "Y"  -a   $ans != "N"   -a  $ans != "n" ]
	do
    	echo "Do you want to set an i3 name for this machine? (yYnN)"
    	read ans
		if [ ! $ans ]
		then
			ans="invalid"
		fi
	done

	if [ $ans = "y"  -o  $ans = "Y" ]
	then
    	./fillInMyName.sh OCALA/generic_proxy/config_files/myNames.xml
	fi
fi
 



# Check if the tun kernel extension is installed.
# If not, direct the user to install it.
if [ ! -d /Library/Extensions/tun.kext ]
then 
    echo "tun kernel extension not installed. Make sure you run installTun.apple.sh before running the proxy."
fi

echo
echo 
echo "You can now run OCALA by typing ./OCALA_proxy -c afer cd-ing to OCALA/generic_proxy.  You need superuser permissions to run OCALA."
echo "For more information, please see the docs and the wiki at http://ocala.cs.berkeley.edu"
echo
echo


