#!/bin/bash
# This script is used targetted at first time users
# compiling the proxy from the source distribution.
# It hopes to make compiling the proxy, all the bundled OCDs
# and configuring the proxy a one step and interactive
# process from the commandline.
# AUTHOR: Dilip Antony Joseph

# Check if  mac
echo $OSTYPE | grep darwin
if [ $? -eq 0 ]
then
	echo "Detected $OSTYPE.  quickStart.sh is to be run only on linux systems."
	echo "Launching quickStart.apple.sh ..."
	echo
	./quickStart.apple.sh
	exit 0
fi

# ASSUMPTION: This script is run from the OCALA_Suite directory.
# Check this assumption
if [ ! -d OCALA    -o      ! -d i3 -o    ! -d sampleOCD -o  ! -d RSPTunnelOCD ]
then
    echo
    echo "Unable to find some of the following required directories: OCALA, i3, sampleOCD, RSPTunnelOCD."
    echo "Please make sure that you are running this script from within the OCALA_Suite directory."
    echo
    exit
fi

# ASSUMPTION: You are using the quick install on a fresh src distribution.
# That is you haven't compiled anything using quickStart or manually before.
# Check this assumption.
if [ -f OCALA/generic_proxy/OCALA_proxy    -o     -f i3/i3OCD/libI3OCD.so    -o    -f sampleOCD/libSampleOCD.so     -o  -f RSPTunnelOCD/libRSPTunnelOCD.so ]
then
    echo
    echo "Found compiled versions of OCALA_proxy, libI3OCD.so, libRSPTunnelOCD.so or libSampleOCD.so."
    echo "This means that you have probably used quickStart or manual compilation on this directory before."
    ans="invalid"
    while [ $ans != "y"  -a  $ans != "Y"  -a   $ans != "N"   -a  $ans != "n" ]
    do
        echo "Do you wish to continue? (yYnN)"
        read ans

        if [ ! $ans ]
        then
            ans="invalid"
        fi
    done

    if [ $ans = "n"  -o  $ans = "N" ]
    then
        echo
        echo "Cancelling quickStart on user request."
        echo
        exit
    fi
fi 


# Delete any of the final output files. 
# Just to make sure we get only the latest ones.
if [ -f OCALA/generic_proxy/OCALA_proxy ]
then
	rm -rf OCALA/generic_proxy/OCALA_proxy
fi

if [ -f i3/i3OCD/libI3OCD.so ]
then
	rm -rf i3/i3OCD/libI3OCD.so
fi

if [ -f sampleOCD/libSampleOCD.so ]
then
	rm -rf sampleOCD/libSampleOCD.so
fi

if [ -f RSPTunnelOCD/libRSPTunnelOCD.so ]
then
	rm -rf RSPTunnelOCD/libRSPTunnelOCD.so
fi


# Clear the screen
clear

# Print the welcome information

echo -e "\033[34m" # Set blue text color
echo "Welcome to OCALA Quick Start"
echo -e "\033[0m" # Restore normal text color

# Get the current directory
CURRDIR=`pwd`

# delete any previous log file
if [ -f $CURRDIR/quickStart.log ]
then
    rm $CURRDIR/quickStart.log
fi

echo "All compile time messages are logged in quickStart.log"
echo

# Compiling OCALA
echo "Compiling OCALA proxy (This may take a few minutes)..."
# Go to the OCALA directory and do a ./compile
cd OCALA
../compile >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the OCALA_proxy binary has been created
if [ ! -f OCALA/generic_proxy/OCALA_proxy ]
then
    echo
    echo "Unable to find OCALA_proxy binary in OCALA/generic_proxy/OCALA_proxy."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile the OCALA as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# OCALA_proxy has been successfully compiled
echo "OCALA_proxy successfully compiled."
echo


# Compiling i3
echo "Compiling i3 (This may take a few minutes) ..."
cd i3
../compile >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the i3 OCD has been created
if [ ! -f i3/i3OCD/libI3OCD.so ]
then
    echo
    echo "Unable to find libI3OCD.so in i3/i3OCD."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile i3 as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# i3 has been successfully compiled
echo "i3 has been successfully compiled."
echo

# Compiling sampleOCD
echo "Compiling sampleOCD (This may take a few minutes) ..."
cd sampleOCD
../compile >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the sample OCD has been created
if [ ! -f sampleOCD/libSampleOCD.so ]
then
    echo
    echo "Unable to find libSampleOCD.so in sampleOCD."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile sampleOCD as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# sampleOCD has been successfully compiled
echo "sampleOCD has been successfully compiled."
echo

# Compiling RSPTunnelOCD
echo "Compiling RSPTunnelOCD (This may take a few minutes) ..."
cd RSPTunnelOCD
../compile >> $CURRDIR/quickStart.log 2>&1
cd ..
# Check if the sample OCD has been created
if [ ! -f RSPTunnelOCD/libRSPTunnelOCD.so ]
then
    echo
    echo "Unable to find libRSPTunnelOCD.so in RSPTunnelOCD."
    echo "There must have been some error during compilation (details in quickStart.log). "
    echo "Please manually compile RSPTunnelOCD as described in README.txt."
    echo "Good Luck!"
    echo
    exit
fi
# RSPTunnelOCD has been successfully compiled
echo "RSPTunnelOCD has been successfully compiled."
echo


echo
echo "All required OCALA files have been successfully created."

echo 


./quickStart.makeProxyConfig.sh




# Copy the i3, RSPTunnelOCD and SampleOCD files to the generic_proxy directory
echo "Copying i3, RSPTunnelOCD and SampleOCD libraries to OCALA/generic_proxy directory..."
cp i3/i3OCD/libI3OCD.so OCALA/generic_proxy
cp sampleOCD/libSampleOCD.so OCALA/generic_proxy
cp RSPTunnelOCD/libRSPTunnelOCD.so OCALA/generic_proxy


# Create the proxy-config.xml configuration file
perl ./create-proxy-config.pl OCALA/generic_proxy/config_files/proxy-config.xml  /dev/net/tun


# Configure the tun device
echo
echo "Configuring tun device ..."
OCALA/configure_tun.pl
CONFIGURE_TUN_RETVAL=$?

INSTALL_OCALA_RETVAL=0

echo
echo

echo "What would you like to do now?"
echo -e "\t1. Configure and install OCALA as a Linux service (RedHat/Fedora only)."
echo -e "\t2. Configure and run OCALA manually."
echo -e "\t3. Exit."
echo
ans="invalid"
while [ $ans != "1"  -a  $ans != "2"  -a  $ans != "3" ]
do
    echo "Please enter your choice [1-3]:"
    read ans

    if [ ! $ans ]
    then
        ans="invalid"
    fi
done

case $ans in
    "1") ./installOCALA.sh
        INSTALL_OCALA_RETVAL=$?
            ;;
#"2") ./fillInMyName.sh OCALA/generic_proxy/config_files/myNames.xml
    "2") which java 2>/dev/null 1>/dev/null
        if [ $? -eq 0 ] && [ -f OCALA/generic_proxy/ProxyConfig.jar ]
        then
            echo "Launching Java ProxyConfig GUI ..."
            echo
            cd OCALA/generic_proxy
            java -jar ProxyConfig.jar
            cd ../..
        else
            echo "java or ProxyConfig.jar not found.  Launching limited command line configuration..."
            echo
            ./fillInMyName.sh OCALA/generic_proxy/config_files/myNames.xml
        fi
            
        echo
        echo "You can now manually run OCALA by going to OCALA/generic_proxy and typing ./OCALA_proxy -c"
		echo "You can re-configure OCALA any time by running the configureOCALA.sh script from within the OCALA/generic_proxy directory."
	    echo "NOTE: you need superuser privileges to run OCALA_Proxy"
        echo
            ;;
    "3") echo "Thank you for using OCALA Quick start."
        echo
            ;;
esac

echo
echo

if [ $INSTALL_OCALA_RETVAL -ne 0 ]
then
    echo "WARNING: Unable to install OCALA.  See the error messages above."
    echo

elif [ $CONFIGURE_TUN_RETVAL -eq 2 ]
then
    # We had been unable to install the tun device due to lack of super user privileges.
    # Warn the user.
    echo "WARNING: tun device not installed due to lack of superuser privileges.  Please run OCALA/configure_tun.pl with superuser privileges."
    echo
fi

echo "For more information, please see the documentation in the docs directory and the wiki at http://ocala.cs.berkeley.edu"
echo
echo
