#ifndef _INTERFACE_H_
#define _INTERFACE_H_

#include <sys/types.h>
#include <netinet/in.h>

/*
 * Describe ipv4 interfaces.  We'll deal with ipv6 later.
 *
 * These routines provide platform-dependent code to gain the list
 * of interfaces and the IPv4 addresses associated with those
 * interfaces.
 *
 * These routines allocate memory, which must be freed with
 * free_intf_descs().
 *
 * get_intf_descs() returns NULL on failure.
 *
 * This library supports the debug.h debugging mechanisms.
 */

struct intf_desc {
    char *intf_name;
    struct in_addr intf_addr;
    struct intf_desc *next;
};

#ifdef __cplusplus
extern "C" {
#endif

struct intf_desc *get_intf_descs();
void free_intf_descs(struct intf_desc *ilist);

#ifdef __cplusplus
}
#endif
#endif /* _INTERFACE_H_ */
