#ifndef _NU_DEBUG_H_
#define _NU_DEBUG_H_

#define NU_DEBUG 1

#include <stdio.h>  /* for perror */
/* Things for the packet debugging.  Probably don't belong in here.. */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ctype.h>
#include <sys/param.h>
#include <sys/time.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include "hdr.h"  /* For myudphdr struct */


#ifdef NU_DEBUG
extern unsigned int nu_debug;
#define NU_DPRINTF(level, fmt, args...) \
        do { if (nu_debug & (level)) fprintf(stderr, fmt , ##args ); } while(0)
#define NU_DERRLOG(fh, fmt, args...) \
        if (fh) do { fprintf(fh, fmt , ##args ); fflush(fh); } while(0)
#define NU_DEBUG_PERROR(errmsg) \
        do { if (nu_debug & NU_DEBUG_ERRS) perror(errmsg); } while(0)
#else
#define NU_DPRINTF(args...)
#define NU_DEBUG_PERROR(args...)
#endif

/*
 * The format of this should be obvious.  Please add some explanatory
 * text if you add a debugging value.  This text will show up in
 * -d list
 */
#define NU_DEBUG_NONE      0x00	// DBTEXT:  No debugging
#define NU_DEBUG_ERRS      0x01	// DBTEXT:  Verbose error reporting
#define NU_DEBUG_INIT      0x02	// DBTEXT:  Debug initialization
#define NU_DEBUG_SOCKETS   0x04 // DBTEXT:  Debug socket operations
#define NU_DEBUG_PROCESSES 0x08	// DBTEXT:  Debug processes (fork/reap/etc)
#define NU_DEBUG_PACKETS   0x20	// DBTEXT:  Debug all packets

#define NU_DEBUG_ALL  0xffffffff

#ifdef __cplusplus
extern "C" {
#endif
int nu_set_debug(char *arg);  /* Returns 0 on success, -1 on failure */

/* Packet debugging utility functions */
void NU_UDPDUMP(myudphdr *udp);
void NU_IPDUMP(struct ip *ip);
#ifdef __cplusplus
}
#endif

#endif /* _NU_DEBUG_H_ */
