/*
 * Extract the list of IP addresses that are local to the machine.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <net/if.h>
#include <string.h>
#include "interface.h"

/* Things linux wants */
#include <sys/ioctl.h>

#include "nu_debug.h"

#define MAXINTERFACES 50

void
free_intf_descs(struct intf_desc *ilist)
{
    struct intf_desc *ni;

    while (ilist != NULL) {
	ni = ilist->next;
	if (ilist->intf_name) free(ilist->intf_name);
	free(ilist);
	ilist = ni;
    }
}


struct intf_desc *
get_intf_descs()
{
    int s = -1;
    int numifs = MAXINTERFACES; /* XXX... should be a better way */
    struct ifconf ifc;
    struct ifreq *ifr;
    int i;

    struct intf_desc *retlist = NULL, *tmp = NULL;

    ifc.ifc_buf = NULL;
    ifc.ifc_len = 0;

    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s == -1) {
	perror("interface.c:get_intf_descs could not get socket");
	goto err;
    }

    ifc.ifc_len = numifs * sizeof(struct ifreq);
    ifc.ifc_buf = malloc(ifc.ifc_len);

    if (!ifc.ifc_buf) {
	perror("interface.c:get_intf_descs int malloc failed");
	goto err;
    }

    if (ioctl(s, SIOCGIFCONF, (char *)&ifc) < 0) {
	perror("interface.c:get_intf_descs SIOCGIFCONF failed");
	goto err;
    }

    for (i = 0; i < ifc.ifc_len;) {
	struct sockaddr *sa;
	struct sockaddr_in *sin;
	
	ifr = (struct ifreq *) &(ifc.ifc_buf[i]);
	sa = (struct sockaddr *)&ifr->ifr_addr;
	if ((ifc.ifc_len - i) < sizeof(*ifr))
	    break;
#ifdef BSD4_4 /* BSD has strange ifrs where the size varies */
	if (sa->sa_len > sizeof(ifr->ifr_addr))
	    i += sizeof(ifr->ifr_name) + sa->sa_len;
	else
#endif
	    i += sizeof(*ifr);

	if (ifr->ifr_addr.sa_family != AF_INET) continue;

	sin = (struct sockaddr_in *)sa;

#if 0
	printf("Intf %s IP %s\n", ifr->ifr_name, inet_ntoa(sin->sin_addr));
#endif

	tmp = (struct intf_desc *)malloc(sizeof(*tmp));
	if (!tmp) {
	    NU_DEBUG_PERROR("interface.c:get_intf_descs malloc");
	    goto err;
	}
	tmp->next = retlist;
	retlist = tmp;
	tmp = NULL;
	retlist->intf_addr = sin->sin_addr;
	retlist->intf_name = strdup(ifr->ifr_name);
	if (!retlist->intf_name) {
	    NU_DEBUG_PERROR("interface_freebsd::get_intf_descs strdup");
	    goto err;
	}
    }
    close(s);
    return retlist;
    
 err:
    if (s != -1) close(s);
    if (ifc.ifc_buf) free(ifc.ifc_buf);
    if (retlist) free_intf_descs(retlist);
    return NULL;
}

#ifdef _TEST_INTERFACE_
int main()
{
    struct intf_desc *addrlist, *tmp;
    
    addrlist = get_intf_descs();

    if (!addrlist) {
	printf("interface_freebsd::main null address list\n");
    } else {
	tmp = addrlist;
	while (tmp != NULL) {
		printf("interface_freebsd::get_intf_descs  %-5.5s  :  %s\n",
		       tmp->intf_name, inet_ntoa(tmp->intf_addr));
	    
	    tmp = tmp->next;
	}
    }
    free_intf_descs(addrlist);
    exit(0);
}
#endif /* _TEST_INTERFACE_ */
