/*
 * Some convenient routines for handling sockets.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <netdb.h>

#include "nu_debug.h"
#include "config.h"
#ifdef DMALLOC
#include <dmalloc.h>
#endif

int bound_udp_socket(short port)
{
    int sock;
    struct sockaddr_in saddr;

    sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (sock == -1) {
	NU_DEBUG_PERROR("socket::bound_udp_socket socket call");
	return -1;
    }

    bzero(&saddr, sizeof(saddr));
#ifdef RON_HAS_STRUCT_SINLEN
    saddr.sin_len = sizeof(saddr);
#endif
    saddr.sin_port = htons(port);
    saddr.sin_addr.s_addr = htonl(INADDR_ANY);

    if (bind(sock, (struct sockaddr *)&saddr, sizeof(saddr)) == -1) {
	NU_DEBUG_PERROR("socket::bound_udp_socket bind call");
	return -1;
    }
    return sock;
}

int raw_socket() {
    int sock;
    int on = 1;

    sock = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
    if (sock == -1) {
	NU_DEBUG_PERROR("socket::raw_socket socket raw create failed");
	return -1;
    }

    if (setsockopt(sock, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on)) < 0) {
	NU_DEBUG_PERROR("socket::raw_socket setsockopt HDRINCL failed");
	return -1;
    }
    return sock;
}

/*
 * in_cksum --
 *	Checksum routine for Internet Protocol family headers (C Version)
 *      From FreeBSD's ping.c
 *  ... not sure if this really belongs in socket.c, but what the heck. :)
 */

u_short
netutil_in_cksum(u_short *addr, int len)
{
	register int nleft = len;
	register u_short *w = addr;
	register int sum = 0;
	u_short answer = 0;

	/*
	 * Our algorithm is simple, using a 32 bit accumulator (sum), we add
	 * sequential 16 bit words to it, and at the end, fold back all the
	 * carry bits from the top 16 bits into the lower 16 bits.
	 */
	while (nleft > 1)  {
		sum += *w++;
		nleft -= 2;
	}

	/* mop up an odd byte, if necessary */
	if (nleft == 1) {
		*(u_char *)(&answer) = *(u_char *)w ;
		sum += answer;
	}

	/* add back carry outs from top 16 bits to low 16 bits */
	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */
	return(answer);
}

/*
 * Laziness is good.  Take an in_addr and fill in a hostname
 */

void
iptoname(struct in_addr *inaddr, char *hostname, int namelen)
{
    struct hostent *he;

    he = gethostbyaddr((char *)inaddr, sizeof(*inaddr), AF_INET);
    if (!he) {
	strncpy(hostname, inet_ntoa(*inaddr), namelen);
    } else {
	strncpy(hostname, he->h_name, namelen);
    }
}

/*
 * Same idea.  Convert a hostname to an inaddr
 */

int
nametoip(char *hostname, struct in_addr *inaddr)
{
    struct hostent *hp;

    if (!inet_aton(hostname, inaddr)) {
	hp = gethostbyname(hostname);
	if (!hp) {
	    /* Both tries at resolving failed... */
	    return -1;
	}

	bcopy(hp->h_addr_list[0], inaddr, hp->h_length);
    }
    return 0;
}

