#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdarg.h>
#include <ctype.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/time.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include "rondebug.h"
#include "forw.h"
#include "ron.h"
#include "ronlib.h"
#include "broadcast_membership.h"

#include "flood_protocol.h"
#include "scheduler.h"

using namespace RON;

void usage()
{
    fprintf(stderr,
	    "[ip|test]-forw  [-Mvhe] [-d <debug>] [-p <port>] [-m <myip>]\n"
	    );
}

void help()
{
    usage();
    fprintf(stderr,
	    "       -h ......... help (this message)\n"
	    "       -d <debug> . see -d list for a list of debug flags\n"
	    "       -m <myip> .. set the local IP addr if we guess wrong\n"
	    "       -p <port> .. use <port> as the RON communication port\n"
	    );
}

void sigcaught(int sig)
{
    exit(0);
}

int main(int argc, char **argv)
{
    extern char *optarg;
    extern int optind;
    int ch;
    membership *memb;

    int forw_port = DEFAULT_RON_FORWARDER_PORT;
    char myhostname[MAXHOSTNAMELEN];
    int hostnameset = 0;

    Scheduler *sched;
    forw *myforw;
    unsigned long myaddr;
    struct in_addr myinaddr;

    while ((ch = getopt(argc, argv, "m:hp:d:")) != -1)
	switch (ch) {
	case 'm':
	    if (nametoip(optarg, &myinaddr)) {
		perror(optarg);
		exit(-1);
	    }
	    hostnameset = 1;
	    break;
	case 'p':
	    forw_port = atoi(optarg);
	    break;
	case 'h':
	    help();
	    exit(0);
	case 'd':
	    printf("Setting debug: %s\n", optarg);
	    if (set_debug(optarg)) {
		exit(-1);
	    }
	    break;
	default:
	    usage();
	    exit(-1);
	}
    argc -= optind;
    argv += optind;

    srandom(time(NULL)+getpid());
    signal(SIGINT, sigcaught);
    signal(SIGTERM, sigcaught);
    struct sigaction ignme;
    ignme.sa_handler = SIG_IGN;
    sigaction(SIGPIPE, &ignme, NULL);

    DPRINTF(DEBUG_INIT, "membership::main starting up\n");

    if (!hostnameset) {
	DPRINTF(DEBUG_INIT, "membership::main getting hostname\n");
	if (gethostname(myhostname, sizeof(myhostname)-1)) {
	    perror("gethostname");
	    exit(-1);
	}
	DPRINTF(DEBUG_INIT, "membership::main got hostname: %s\n",
		myhostname);
	if (nametoip(myhostname, &myinaddr)) {
	    perror(myhostname);
	    exit(-1);
	}
    }

    sched = new Scheduler();
    if (!sched) {
	perror("could not allocate Scheduler.  Bailing!\n");
	exit(-1);
    }
    
    myaddr = myinaddr.s_addr;
    DPRINTF(DEBUG_INIT, "membership::main got address %s\n",
	    inet_ntoa(myinaddr));

    flood_protocol *fp;
    fp = new flood_protocol();

    memb = new broadcast_membership("peerinit.txt", fp);

    if (!memb) {
	perror("could not create membership class");
	exit(-1);
    }

    myforw = new forw(sched, myinaddr, forw_port, NULL, memb);
    DPRINTF(DEBUG_INIT, "membership::membership %p forwarder %p\n",
	    memb, myforw);
    if (!myforw) {
	perror("could not initialize forwarder");
	exit(-1);
    }
    myforw->init(forw_port);
    fp->register_default(myforw);
    fp->configure(sched, myforw);
    memb->register_default(myforw);
    memb->configure(sched, myforw);
    
    while (1) {
	sched->main();
    }
    
    exit(0);
}
