/* -*-  Mode:C; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */

#ifndef _PACKET_H_
#define _PACKET_H_
#include <sys/types.h>
#include <sys/time.h>
#include "ron.h"
#include "node.h"

namespace RON {

class pkt_control {
public:
    int via_ron;
};

class pkt {
public:
	int size;
    char *contents;
    struct timeval rectime;
    struct sockaddr_in fromsock;

	// (SAR) Make packet, allocate n bytes for contents 
    pkt(unsigned n = maxpktsize);
    virtual ~pkt();

	// (SAR) Declared here, but nowhere else.  
	// Should generate a  compilation error if ever used.
	pkt const &operator=(pkt const &other); 
	
    struct ron_pkt_header *ron_hdr() {
	return reinterpret_cast<struct ron_pkt_header *>(contents);
    }
    char *data() {
	return contents + sizeof(struct ron_pkt_header);
    }
	// (SAR)
    virtual unsigned int datalen() {
	return size - sizeof(struct ron_pkt_header);
    }
    void set_datalen(unsigned int newlen) {
	size = newlen + sizeof(struct ron_pkt_header);
    }
    void set_dst(node dst) {
	ron_hdr()->dst_addr = dst.node_addr;
	ron_hdr()->dst_port = dst.node_port;
    }
    void set_src(node src) {
	ron_hdr()->src_addr = src.node_addr;
	ron_hdr()->src_port = src.node_port;
    }
    node get_src() {
	return node(ron_hdr()->src_addr, ron_hdr()->src_port);
    }
    node get_dst() {
	return node(ron_hdr()->dst_addr, ron_hdr()->dst_port);
    }

    /* Access methods to get at the RON header in the right
     * byte order */

    virtual int get_size()          { 
		return size; 
	}
    u_int16_t get_hlim()          { return (ron_hdr()->hlim); }
    u_int8_t  get_version()       { return (ron_hdr()->version); }
    u_int32_t get_type()          { return (ntohl(ron_hdr()->type)); }
    u_int32_t get_flowid()        { return (ntohl(ron_hdr()->flowid)); }
    u_int32_t get_seq()           { return ntohl(ron_hdr()->seq); }
    u_int16_t get_routing_flags() { return ntohs(ron_hdr()->routing_flags); }
    u_int16_t get_traffic_type()  { return ntohs(ron_hdr()->traffic_type); }
	struct in_addr get_src_addr() { return ron_hdr()->src_addr; }
	struct in_addr get_dst_addr() { return ron_hdr()->dst_addr; }

    
    void      set_hlim(u_int16_t n)     { ron_hdr()->hlim = n; }
    void      set_version(u_int8_t n)   { ron_hdr()->version = n; }
    void      set_type(u_int32_t n)      { ron_hdr()->type = htonl(n); }
    void      set_flowid(u_int32_t fid) { ron_hdr()->flowid = htonl(fid); }
    void      set_seq(u_int32_t seq)    { ron_hdr()->seq = htonl(seq); }
    void      set_routing_flags(u_int16_t n) {
	ron_hdr()->routing_flags = htons(n);
    }
    void      set_traffic_type(u_int16_t nflags) {
	ron_hdr()->traffic_type = htons(nflags);
    }


};

// (SAR)
void describe_pkt(pkt *p);


}



#endif /* _PACKET_H_ */
