/*
 * The passive agressive network monitor:
 * Passively sniff on traffic to determine what's active
 * use a failure of passive traffic to trigger aggressive probing
 * to route around failures.
 *
 * If we're cool, maybe we can do this via the pdb without changing
 * the interface in the routers...
 *
 * The passive monitor should be interposed on all outbound
 * _and_ inbound traffic at the same stage in the pipeline.
 */

/* 2003 Jan 14 - dga */

#ifndef _PASSIVE_MONITOR_H_
#define _PASSIVE_MONITOR_H_

#include "ron.h"
#include "plumber.h"
#include "forw.h"
#include "scheduler.h"
#include "flowid.h"
#include <list>

namespace RON {
class flow_info {
public:
    flow_info() : last_transmit(0), last_seq(0) {; }
    fixedpoint last_transmit;
    u_int32_t last_seq;
};

class received_flow_info {
public:
    u_int32_t high_seq;
    list<int> missing;
};

class passive_monitor : public Plumber {
public:
    passive_monitor();
    virtual ~passive_monitor();

    void configure(Scheduler *s, forw *f);
    int recv(pkt *mypkt, node *mynode, int via_ron);
    int transmit(pkt *mypkt, node *mynode, int via_ron);
    
private:
    forw *myforw;
    Scheduler *mysched;
    map<flowid, flow_info> flowcache;
    map<flowid, received_flow_info> received_flowcache;

    //void purge_flowcache(); /* Flush da old stuff */
    void sequence_transmit(pkt *mypkt);
    void sequence_receive(pkt *mypkt);
    }; /* class passive_monitor */
} /* namespace RON */

#endif /* _PASSIVE_MONITOR_H_ */
