#ifndef _PING_PROTOCOL_H_
#define _PING_PROTOCOL_H_

#include "ron.h"
#include "forw.h"
#include "errno.h"
#include "../pdb/pdb_client.h" /* XXX */

namespace RON {

#define DELIVER_DIRECT 0
#define DELIVER_RON    1

struct ron_ping_header {
    u_int32_t id1;
    u_int32_t id2;
    struct fixedpoint_wire proctime;
    u_int32_t rtable_version;
    u_int8_t  type;
};

#define TYPE_FIRST      0x0
#define TYPE_SECOND     0x1
#define TYPE_THIRD      0x2


class peer_contact {
public:
    int operator==(const peer_contact &p) { return (p.peer == peer); }
    node peer;
    fixedpoint last_ping;
    u_int32_t failures;		/* Number of failures in a row */
    fixedpoint lat_avg;		/* Used for setting timeout */
    fixedpoint lat_md;		/* Mean deviation of latency */
};

class ping_record {
public:
    peer_contact *con;
    fixedpoint time_sent;
    node dst;
    u_int64_t id;
    int marked_failed;		/* Have we timed it out already? */
    int type;			/* Did we originate it or not? */
    int operator ==(const ping_record &a) {
	return (id == a.id);
    }
    int operator !=(const ping_record &a) {
	return (id != a.id);
    }
};

#define PDB_Q 0.8


class ping_protocol : public Plumber, time_callable {
public:
    ping_protocol(pdb_client *npdbc) 
	{ 
	    set_timer_name("Ping Protocol");
	    pdbc = npdbc; 
	}

    virtual ~ping_protocol() { }
    
    virtual int recv(pkt *packet, node *mynode, int via_ron);
    virtual int timer_callback(void *data);

    void configure(Scheduler *s, forw *f);
    void maybedown(node *downnode);

    /* (SAR) Will need to be overwritten in NS emulation */
    virtual int gettimeofday(struct timeval *tv, struct timezone *tz)
	{
	    return ::gettimeofday(tv, tz);
	}

    /* (SAR) Will need to be overwritten in NS emulation */
    virtual long int random(void)
	{
	    return ::random();
	}
private:
    forw *myforw;
    Scheduler *sched;
    pdb_client *pdbc;
    static const unsigned int max_cache_size = 10000;
    list<node> peers;
    list<peer_contact> contacts;
    list<ping_record> record;
    map<u_int64_t, u_int32_t> cache1;
    map<u_int64_t, u_int32_t> cache2;
    int current_cache;
    void send_pings();
    int send_ping(node *mynode, u_int64_t id, u_int8_t type,
		  int via_ron, fixedpoint proctime);
    void reconcile_peers();
    void clean_pings();

    fixedpoint nowpoint;  /* Updated every recv or timer_callback */
};

}

#endif /* _PING_PROTOCOL_H_ */
