#include "policy_classifier.h"
#include "rondebug.h"

using namespace RON;

vbns_classifier::vbns_classifier()
{
    char inbuf[64];
    char s_net[64], s_mask[64];
    u_int32_t net, mask;

    FILE *in;
    vbns_nets.clear();
    in = fopen("vbns.txt", "r");
    if (!in) {
	DEBUG_PERROR("could not open vbns.txt");
	return;
    }
    while (fgets(inbuf, sizeof(inbuf)-1, in) != NULL) {
	char *comment;
	comment = strchr(inbuf, '#');   if (comment) *comment = 0;
	comment = strchr(inbuf, '\n');  if (comment) *comment = 0;
	comment = strchr(inbuf, '\r');  if (comment) *comment = 0;
	if (!strlen(inbuf)) { continue; }
	if (sscanf(inbuf, "net %s %s", s_net, s_mask) == 2) {
	    DPRINTF(DEBUG_POLICY,
		    "vbns_classifier:  net %s ... %s\n", s_net, s_mask);
	    net = inet_addr(s_net);
	    mask = inet_addr(s_mask);
	    net_pair np;
	    np.net = net & mask; /* pre-filter it, makes life easier */
	    np.mask = mask;
	    vbns_nets.push_back(np);
	} else {
	    DPRINTF(DEBUG_ERRS | DEBUG_POLICY,
		    "vbns_classifier bad line: %s\n", inbuf);
	}
    }
    fclose(in);
}

/* See if a link is permitted for NOVBNS traffic */
int
vbns_classifier::permitted(const node * const src, const node * const dst)
{
    if (is_vbns(src) && is_vbns(dst)) {
	return POLICY_DENIED;
    }
    
    return POLICY_PERMITTED;
}

int
vbns_classifier::pkt_match(char *packet_data)
{
    u_int32_t addr;
    struct ip *ip;

    ip = (struct ip *)packet_data;

    addr = ip->ip_src.s_addr;
    
    list<net_pair>::const_iterator i;

    for (i = vbns_nets.begin(); i != vbns_nets.end(); i++) {
	if ((addr & (*i).mask) == (*i).net) {
	    return PKT_NOMATCH;
	}
    }
    return PKT_MATCH;
}

int
vbns_classifier::is_vbns(const node *n)
{
    u_int32_t addr;
    addr = n->node_addr.s_addr;

    list<net_pair>::const_iterator i;
    for (i = vbns_nets.begin(); i != vbns_nets.end(); i++) {
	if ((*i).match(addr)) return 1;
    }

    return 0;
}
