#ifndef _POLICY_CLASSIFIER_H_
#define _POLICY_CLASSIFIER_H_

#include <ron.h>
#include <forw.h>

#define POLICY_DENIED 0
#define POLICY_PERMITTED 1

#define PKT_NOMATCH 0
#define PKT_MATCH 1

namespace RON {

/* The default policy classifier allows all packets on all links */
class policy_classifier {
public:
    virtual ~policy_classifier() { }
    virtual u_int16_t traffic_type() { return TRAFFIC_NORMAL; }
    virtual int pkt_match(char *packet_data) { return PKT_MATCH; }
    /* Is the link from src-dst permitted by this policy? */
    virtual int permitted(const node * const src, const node * const dst) {
	return POLICY_PERMITTED;
    }
    virtual string to_string() { return string("default policy"); }
};

class vbns_classifier : public policy_classifier {
public:
    u_int16_t traffic_type() { return TRAFFIC_NOVBNS; }
    vbns_classifier();
    virtual int pkt_match(char *packet_data);
    virtual int permitted(const node * const src, const node * const dst);
    virtual string to_string() { return string("vBNS policy"); }
private:
    class net_pair {
    public:
	u_int32_t net;
	u_int32_t mask;
	int match(u_int32_t addr) const { return ((addr & mask) == net); }
    };

    list<net_pair> vbns_nets;
    int is_vbns(const node *n); /* Is a node on the vBNS? */
};

}

#endif /* _POLICY_CLASSIFIER_H_ */
