#ifndef _PRINT_PROTOCOL_H_
#define _PRINT_PROTOCOL_H_
#include "forw.h"

namespace RON {

class print_protocol : public Plumber, public time_callable {
public:
    int recv(pkt *packet, node *mynode, int via_ron) {
	static int count = 0;
	if (packet->datalen() > 0) {
	    printf("print_protocol::recv data %*.*s\n",
		   (int)packet->datalen(), (int)packet->datalen(),
		   packet->data());
	    if (packet->datalen() < 3 || strncmp(packet->data(), "ACK", 3)) {
		pkt *mypkt = new pkt();
		node src = packet->get_src();
		sprintf(mypkt->data(), "ACK %d", packet->get_seq());
		mypkt->set_datalen(strlen(mypkt->data()) + 1);
		mypkt->set_dst(packet->get_src());
		mypkt->set_type(RON_PRINT);
		mypkt->ron_hdr()->src_addr = myforw->get_myaddr();
		mypkt->ron_hdr()->src_port = myforw->get_port();
		mypkt->set_flowid(2);
		mypkt->set_seq(count++); /* shrug */
		dispatch(mypkt, &src, 1);
		delete mypkt;
	    }
	} else {
	    printf("print_protocol::recv empty packet\n");
	}
	return 0;
    }
    void configure(Scheduler *s, forw *f) {
	sched = s;
	myforw = f;
	f->register_type(RON_PRINT, this);
    }
    node pick_random_node() {
	
	list<node> *peers = myforw->get_membership()->peers();
	list<node>::const_iterator peer;
	node dst;
	
	int hopidx = 0;
	int newidx = 0;
	int npeers = peers->size();
	
	if (!npeers) { return dst; }

	newidx = random() % npeers;
	
	hopidx = 0;
	for (peer = peers->begin();  peer != peers->end(); peer++) {
	    if (hopidx == newidx) {
		dst.node_addr = peer->node_addr;
		dst.node_port = peer->node_port;
		break;
	    }
	    hopidx++;
	}
	return dst;
    }

    int timer_callback(void *data) {
	static int count = 0;
	if (myforw->get_membership()->peers()->size() >= 1) {
	    node dst = pick_random_node();
	    pkt *mypkt = new pkt();
	    sprintf(mypkt->data(), "Moo Packet %d", count);
	    mypkt->set_datalen(strlen(mypkt->data()) + 1);
	    mypkt->set_dst(dst);
	    mypkt->set_type(RON_PRINT);
	    mypkt->ron_hdr()->src_addr = myforw->get_myaddr();
	    mypkt->ron_hdr()->src_port = myforw->get_port();
	    mypkt->set_flowid(1);
	    mypkt->set_seq(count++); /* shrug */
	    node *hop = myforw->nexthop(&dst, mypkt);
	    printf("print_protocol send %d to %s via %s : %s\n",
		   mypkt->get_type(),
		   mypkt->get_dst().to_string().c_str(),
		   hop == NULL ? "<null>" : hop->to_string().c_str(),
		   mypkt->data());
	    dispatch(mypkt, &dst, 1);
	    if (hop) delete hop;
	    delete mypkt;
	}
	if (active) sched->register_timer(5000000, this, NULL);
	return 0;
    }
    void set_active(int a) {
	if (a && !active) {
	    sched->register_timer(5000000, this, NULL);
	} else if (!a && active) {
	    /* XXX:  Should cancel the timer;  timer leak if on/off/on. */
	}
	active = a;
    }
    print_protocol() { active = 0; }
private:
    forw *myforw;
    Scheduler *sched;
    int active;
};

}

#endif /* _PRINT_PROTOCOL_H_ */
