/* -*-  Mode:C; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
#ifndef _SCHEDULER_H_
#define _SCHEDULER_H_

#include <map>
#include <list>
#include <vector>
#include <queue>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/param.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "rondebug.h"
#include "fixedpoint.h"

#include "ron_named.h"

namespace RON {

class socket_callable {
public:
    virtual int socket_callback(int sockno, void *data) = 0;
};

class time_callable {
 public:
	virtual ~time_callable() {}
    virtual int timer_callback(void *data) = 0;
	
	void set_timer_name(string n)
		{
			_timer_name = n;
		}
	string get_timer_name()
		{
			return _timer_name + " timer";
		}

 private:
	string _timer_name;
};

class time_callback_entry {
public:

    time_callback_entry() { exptime = 0;  callback = NULL;  data = NULL; }
    time_callback_entry(const time_callback_entry &a) {
	exptime = a.exptime;
	callback = a.callback;
	data = a.data;
    }
    
    fixedpoint exptime;
    time_callable *callback;
    void *data;
    bool operator<( const time_callback_entry &a ) const {
        //return exptime < a.exptime;
        return a.exptime < exptime;
	// XXX:  This is a cheesy temporary hack to make the priority
	// queue work.  For some reason, less<> and greater<> give
	// the same results.  STL problem, or Dave problem?
    }
    void set_time(struct timeval t) {
	exptime.ones = t.tv_sec;
	exptime.micros = t.tv_usec;
    }
    struct timeval get_time() {
	struct timeval t;
	t.tv_sec = exptime.ones;
	t.tv_usec = exptime.micros;
	return t;
    }
};

class Scheduler {
public:
    Scheduler();
	
	/* Now virtual, to allow to be specified by NSRonScheduler as well (SAR) */
    virtual int main();

    virtual int register_socket(int sockno, socket_callable *callback, void *data);
    /* usec is number of miliseconds in the future */
    virtual int register_timer(u_int32_t msec, time_callable *callback, void *data);

	virtual ~Scheduler() {}

private:
    map<int, socket_callable *> socket_callback_table;
    list<int> socket_list;
    priority_queue< time_callback_entry,
	            vector<time_callback_entry> > timer_callback_table;
};

}

#endif /* _SCHEDULER_H_ */
