/*
 * Utility thing so we can hash based on flow ID
 *   - A flow is defined as a unique combination of:
 *     - source address / port  (src_addr, src_port)
 *     - destination address / port (dst_addr, dst_port)
 *     - Flow ID (flowid)
 *     - traffic type (type)???
 */

#ifndef _FLOWID_H_
#define _FLOWID_H_

#include "ron.h"
#include "forw.h"
#include "packet.h"
#include "rondebug.h"

using namespace RON;

class flowid {
public:
    flowid() { }
    void set(pkt *p) {
	src = p->get_src();
	dst = p->get_dst();
	pkt_flowid = p->get_flowid();
	type = p->get_type();
    }
    flowid(pkt *p) { set(p); }

    const flowid& operator=(const flowid& right) {
	type = right.type;
	src = right.src;
	dst = right.dst;
	pkt_flowid = right.pkt_flowid;
	return right;
    }

    bool operator==(const flowid& right) const {
	return (pkt_flowid == right.pkt_flowid && 
		type == right.type &&
		src == right.src &&
		dst == right.dst);
    }

    /*
     * This could be made much faster by precomputing
     * a single 32 bit flowid and doing a hash comparison
     * of that.  I don't care for now.
     */
    bool operator<(const flowid& right) const {
	if (pkt_flowid != right.pkt_flowid)
	    return (pkt_flowid < right.pkt_flowid);
	if (type != right.type)
	    return (type < right.type);
	if (!(src == right.src))
	    return (src < right.src);
	if (! (dst == right.dst))
	    return (dst < right.dst);
	return 0;
    }

    string to_string() const {
	char mystring[128];
	bzero(mystring, sizeof(mystring));
	sprintf(mystring, "(%s,%s) %d %d",
		src.to_string().c_str(), dst.to_string().c_str(),
		type, pkt_flowid);
	string s1(mystring);
	return s1;
    }

private:
    u_int32_t type;
    node src;
    node dst;
    u_int32_t pkt_flowid;
};

#endif /* _FLOWID_H_ */
