/* See ip_conduit.h for documentation */

#include "IPPacket.h"
#include "common.h"
extern "C" {
	#include "benchmarks/checkpoint.h"
}



#include "../../ronOCD/ronDebug.h"

#include "ronsys.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdarg.h>
#include <ctype.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/time.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <string.h>
#include "rondebug.h"
#include "forw.h"
#include "ron.h"
#include "ronlib.h"
#include "membership.h"

#include "ip_conduit.h"

#include "../../ronOCD/RONOverlayNetwork.h"

#ifdef LINUX

#define uh_sport source
#define uh_dport dest
#define uh_ulen len
#define uh_sum check

#define th_sport source
#define th_dport dest

#endif

using namespace RON;

/*
 * Local delivery mechanism for encapsulated packets
 */

ip_conduit::ip_conduit(RONOverlayNetwork *ronOverlay)
{
    this->ronOverlay = ronOverlay;

    DPRINTF(DEBUG_INIT, "ip_conduit initializing\n");

    //insock = create_divert_socket(9696);
    if (insock == -1) {
	    DEBUG_PERROR("ip_conduit constructor:  divert socket");
    }

    outsock = raw_socket(); 
    if (outsock == -1) {
	    DEBUG_PERROR("ip_conduit_output constructor:  raw_socket");
    }
    classifiers[0] = new vbns_classifier;
}

/*
 * Classify the packet and assign to it some routing flags
 * This is a temporary hack for measurement purposes;
 * this should be a more general framework for letting
 * the user tag their data.
 */

u_int16_t
RON::pkt_classify(char *pkt_contents, size_t len)
{
    struct ip *ip;
    ip = (struct ip *)pkt_contents;

    if (len < sizeof(struct ip)) {
	return 0;
    }
    if (len < sizeof(struct udphdr)) {
	return 0;
    }
    if (ip->ip_p == IPPROTO_TCP) {
	return RON_ROUTE_BW;
    }
    if (ip->ip_p == IPPROTO_UDP) {
	struct udphdr *h;
	h = (struct udphdr *)(pkt_contents + sizeof(struct ip));
	if (ntohs(h->dest) == 5003) {
	    return RON_ROUTE_LOSS;
	}
	else if (ntohs(h->dest) == 5004) {
	    return RON_ROUTE_MESH_LAT;
	}
	else if (ntohs(h->dest) == 5005) {
	    return RON_ROUTE_MESH_LOSS;
	}
	return RON_ROUTE_LAT;
    }
    if (ip->ip_p == IPPROTO_ICMP) {
	return RON_ROUTE_MESH_LAT;
	return RON_ROUTE_LAT;
    }
    return 0;
}

/*
 * Given an IP packet, find the destination node for it.
 * Step 1:  Match against any node in the peers table that
 *          has the same IP address.
 * Step 2:  Statically configured routing on the localhost
 *          (e.g. 18.31.0.0/16 -> 18.31.0.114)
 * Step 3:  Dynamic routing from local peer configuration
 *          files.  Opens an interesting trust issue..
 */

/* Return 0 on success, -1 on failure */
int
ip_conduit::ip_find_dst(struct in_addr dstAddr , node *dst)
{
    list<node> *peers;
    list<node>::const_iterator peer;


    DPRINTF (DEBUG_PACKETS, "ip_conduit:ip_find_dst: looking for ip : %s\n", inet_ntoa (dstAddr));

    /* Yeah, this is ugly, and linear.  while(1) printf("N is small."); */
    peers = myforw->get_membership()->peers();
    for (peer = peers->begin(); peer != peers->end(); peer++) {
        DPRINTF (DEBUG_PACKETS, "ip_conduit:ip_find_dst: peer: %s\n", inet_ntoa ((*peer).node_addr));
	    if (dstAddr.s_addr == (*peer).node_addr.s_addr) {
	        *dst = *peer;
	        return 0;
	    }
    }
    return -1;
}

/*
 * Long term, we should examine the routing table and
 * determine the max MTU for whatever its destination
 * is, and do the fragmentation accordingly.
 *
 * Also make the MTU dependent in some way upon the eventual encapsulation
 * to which the packet would be subjected.  This is a tricky one,
 * since in theory this encapsulation could be dynamic.. but some
 * simple solution should suffice.
 */


int ip_conduit::socket_callback (int sockno, void *data) {

    DPRINTF (DEBUG_PACKETS, "\n\n\nERROR: Dummy socket_callback function invoked.\n\n");
}

/**
  * This function sends the specified data to the specified IP address over RON.
  */
int
ip_conduit::send(char *pktContents, int pktSize, struct in_addr realIPAddress )
{
    DPRINTF(DEBUG_PACKETS, "ip_conduit::receivedDataFromApp pktFromAppLen = %d!\n", pktSize);
    char *buf;
    struct ip *ip;
    pkt *mypkt;

    mypkt = new pkt();
    if (!mypkt) {
	    DEBUG_PERROR("ip_conduit::socket_callback could not allocate pkt");
	    return -1;
    }

    buf = mypkt->data();

    memcpy (buf, pktContents, pktSize);

    mypkt->set_datalen(pktSize);
    ip = (struct ip *)buf;

    //DILIP: I have to change the destination fake ip address in the captured packet to that
    // of the real address.

    node dst;
    
    int rc = ip_find_dst(realIPAddress, &dst);
    if (rc != 0) {
        PRINT_DEBUG (DEBUG_LEVEL_SUPER,
                "Unable to find exit point. TODO send over IP.\n"
                );
	    /* Could not find an exit point for this packet.  Therefore,
	    * send it directly over the Internet ASAP. */
	    //rc = sendto(insock, buf, mypkt->datalen(), 0,
		//        (struct sockaddr *)&fromaddr, fromlen);
	    if (mypkt) delete mypkt;
	    return rc;
    }
#if 0
    if (!(myforw->dstmap[dst].is_null())) {
	dst = myforw->dstmap[dst];
    }
#endif
    
    mypkt->set_dst(dst);
    mypkt->ron_hdr()->src_addr = myforw->get_myaddr();
    mypkt->ron_hdr()->src_port = myforw->get_port();
    mypkt->set_type(RON_DATA);

    if (ip->ip_p == IPPROTO_TCP) {
    	struct tcphdr * t = (struct tcphdr *)((char *)ip
					      + sizeof(struct ip));
	    u_int32_t fid;
	    fid = (t->source + t->dest +
	        ip->ip_src.s_addr + ip->ip_dst.s_addr);
	    mypkt->set_flowid(fid);
    }

    //mypkt->set_routing_flags(pkt_classify(mypkt->data(), mypkt->datalen()));
    	//DILIP SOSP HACK
	mypkt->set_routing_flags(ronOverlay->routingMetric);

    for (int i = 0; i < num_classifiers; i++) {
	    if (classifiers[i]->pkt_match(mypkt->data()) == PKT_MATCH) {
	        mypkt->set_traffic_type(classifiers[i]->traffic_type());
	        break;
	    }
    }
    /* Otherwise, it defaults to 0 */
    
#ifdef _OPT_IP_CONDUIT_

    node *nh = myforw->nexthop(&dst, mypkt);
    
    printf ("aaaaaaaaa nh=%p %d %d\n", nh, nh->node_addr.s_addr, dst.node_addr.s_addr);
    if (!nh || nh->node_addr.s_addr == dst.node_addr.s_addr) {
	
#ifdef BENCHMARK
	addCheckpoint (EVENT_DATA_BEFORE_OC_D_TO_NET, 1);
#endif
	
	    rc = sendto(insock, buf, mypkt->datalen(), 0,
		    (struct sockaddr *)&fromaddr, fromlen);

#ifdef BENCHMARK
	addCheckpoint (EVENT_DATA_AFTER_OC_D_TO_NET, 1);
#endif
	    if (nh) delete nh;
	    if (mypkt) delete mypkt;
	    return rc;
    }
#endif
    
    /*
     * Handle path MTU discovery.
     *
     * This is ugly, stupid, and horrible.  I'm very sorry if
     * you have to read this code, or worse yet, fix it.
     *
     * The MTU is assumed to be 1500.  That's also stupid, but
     * easily fixed once we have an interface into the performance
     * database.
     *
     */
   
    /*
    if ((ntohs(ip->ip_off) & IP_DF) && (ntohs(ip->ip_len) > 
				 1500 - (sizeof(struct ip) + sizeof(struct udphdr)
				  + sizeof(struct ron_pkt_header)))) {
	    DPRINTF(DEBUG_PACKETS, "ip_conduit:  Must fragment %d byte packet\n",
		        ntohs(ip->ip_len));
	    pkt outpkt;
	    char *icmpbuf = outpkt.data();
	    struct icmp *icmp = (struct icmp *)(icmpbuf + sizeof(struct ip));
	    struct ip *retip = (struct ip *)icmpbuf;
	
	    bzero(icmpbuf, sizeof(icmpbuf));
	    memcpy(icmpbuf, buf, sizeof(struct ip));
	    memcpy((char *)&icmp->icmp_ip, buf, sizeof(struct ip)+8);

	    retip->ip_len = htons(sizeof(struct ip) + sizeof(struct icmp) + 20); 
	    retip->ip_p = IPPROTO_ICMP;
	    retip->ip_sum = 0;
	    struct in_addr tmpaddr = retip->ip_dst;
	    retip->ip_dst = retip->ip_src;
	    retip->ip_src = tmpaddr;
	    retip->ip_off = htons(0);
	    outpkt.size = sizeof(struct ron_pkt_header) + sizeof(struct ip) + sizeof(struct icmp) + 20;

	    icmp->icmp_type = ICMP_UNREACH;
	    icmp->icmp_code = ICMP_UNREACH_NEEDFRAG;
	    icmp->icmp_cksum = 0;
	    icmp->icmp_nextmtu = htons((1500 - sizeof(struct ip) - sizeof(struct udphdr) - sizeof(struct ron_pkt_header)));
	    icmp->icmp_cksum = netutil_in_cksum((u_short *)icmp, sizeof(struct icmp) + 20);

	    recv(&outpkt, NULL, 0);  //This is the recv call on the ip_conduit.  It probably sends an ICMP message to the legacy application.
	    if (mypkt) delete mypkt;
	    return -1;
    }

    */


    if (mypkt->get_routing_flags() == RON_ROUTE_MESH_LAT) {
	    mypkt->set_routing_flags(RON_ROUTE_LAT);
	    mypkt->set_type(RON_MESH_DATA);
    }
    if (mypkt->get_routing_flags() == RON_ROUTE_MESH_LOSS) {
	    mypkt->set_routing_flags(RON_ROUTE_LOSS);
	    mypkt->set_type(RON_MESH_DATA);
    }

    	//DILIP SOSP HACK
	mypkt->set_routing_flags(ronOverlay->routingMetric);
	DPRINTF (DEBUG_PACKETS, "\n\n\n-----> Sending packet of routing packet flag  = %d\n\n", mypkt->get_routing_flags());


    dst.node_addr.s_addr = dst.node_addr.s_addr;
    
    rc = dispatch(mypkt, &dst, 1);

    delete mypkt;
    return rc;
}

int
ip_conduit::recv(pkt *mypkt, node *mynode, int via_ron)
{
    char *enpacket = NULL;
    struct sockaddr_in dst;
    struct ip *ip;
    int rc;

    DPRINTF(DEBUG_PACKETS, "ip_conduit::recv handed a packet\n");

    enpacket = mypkt->data();

    ronOverlay->handleRONPacket(enpacket, mypkt->datalen());

    ip = (struct ip *)enpacket;

    bzero(&dst, sizeof(dst));
#ifdef RON_HAS_STRUCT_SINLEN
    dst.sin_len = sizeof(dst);
#endif
    dst.sin_family = AF_INET;
    dst.sin_addr = ip->ip_dst;

    /*
     * This next bit is to counteract some BSD stupidity.
     * The kernel insists on fixing up the IP len and IP
     * offset fields, so it wants them in host byte order...
     */
    ip->ip_sum = 0;
    ip->ip_len = ntohs(ip->ip_len);
    ip->ip_off = ntohs(ip->ip_off);  /* Gag me with a spoon */
    ip->ip_ttl--;   /* Make sure we don't permit routing loops */

    //DILIP: We have to call writeToTunnel here.
    if ((rc = sendto(outsock, enpacket, mypkt->datalen(), 0,
		     (struct sockaddr *)&dst, sizeof(dst))) <= 0) {
	perror("ip_conduit::recv could not sendto");
    }
    DPRINTF(DEBUG_PACKETS, "ip_conduit::recv sendto rc %d\n",
	    rc);
    return rc;
}

void
ip_conduit::configure(Scheduler *s, forw *f)
{
    sched = s;
    myforw = f;

    
    myforw->register_type(RON_DATA, this, NULL);
    myforw->register_type(RON_MESH_DATA, this, NULL);
    sched->register_socket(insock, this, NULL);
}

