#ifndef _SNARF_H_
#define _SNARF_H_

#include "IPPacket.h"

#include "forw.h"
#include "policy_classifier.h"

class RONOverlayNetwork;

/*
 * An input / output pair for tunneling packets directly over
 * the RON.  Right now, the code is FreeBSD specific in two
 * ways:
 *   a)  It uses divert sockets to grab packets that pass through
 *       it.  (This is very FreeBSD specific).
 *   b)  When it outputs its packets via a raw socket, it ntohs()s
 *       some of the IP parameters.  This is BSD specific.
 *       The output routine is really quite general, however,
 *       and with two lines of changes will work for Linux as well.
 */

extern "C" int create_divert_socket(int divert_port);

namespace RON {

class ip_conduit : public Plumber, public socket_callable {
public:
    ip_conduit(RONOverlayNetwork *);
    virtual ~ip_conduit() {
	    close(insock);
	    close(outsock);
    }

    void configure(Scheduler *s, forw *f);
    int socket_callback(int sockno, void *data);
    int recv(pkt *mypkt, node *mynode, int via_ron);
    int ip_find_dst(struct in_addr dstAddr, node *dst);

    int send(char *pktContents, int pktSize, struct in_addr realIPAddress);
    forw *myforw;
private:
    RONOverlayNetwork *ronOverlay;
    Scheduler *sched;
    int insock;
    int outsock;
    static const int num_classifiers = 1;
    /* This is bad;  the classifiers should be a shared thing
     * with the forwardr, not local.  But this will get
     * them working now */
    policy_classifier *classifiers[num_classifiers];
};

// (SAR)
u_int16_t pkt_classify(char *pkt_contents, size_t len);

}


#endif /* _SNARF_H_ */
