/*
 * The passive agressive network monitor:
 */

#include "passive_monitor.h"
#include "flowid.h"

using namespace RON;


passive_monitor::passive_monitor()
{
    /* Nothin' doing */
}

passive_monitor::~passive_monitor()
{
    /* Yawn */
}

void
passive_monitor::configure(Scheduler *s, forw *f)
{
    printf("passive_monitor got configured\n");
    mysched = s;
    myforw = f;
    f->register_type(RON_MONITORED, this, NULL);
}

void
passive_monitor::sequence_transmit(pkt *mypkt)
{
    /* 1:  Have we seen this host before?  If not, add it to the
     * active set of remote hosts.
     *
     * 2:  Assign a sequence number to the packet and mark
     *     that sequence number as transmitted.
     */

    /* Find the flowid for the host */
    flowid fid(mypkt);
    map<flowid, flow_info>::const_iterator res;
    flow_info fi;

    res = flowcache.find(fid);
    if (res != flowcache.end()) {
	fi = flowcache[fid];
    } else {
	fi.last_seq = 0;
    }
    fi.last_seq++;
    struct timeval tv;
    
    gettimeofday(&tv, NULL);
    fi.last_transmit.set(tv.tv_sec, tv.tv_usec);
    
    flowcache[fid] = fi;
    DPRINTF(DEBUG_PASSIVE, "Next seqno for %s:  %d at %s\n",
           fid.to_string().c_str(), fi.last_seq,
	    fi.last_transmit.to_string().c_str());
    struct ron_monitor_header *mh =
	(struct ron_monitor_header *)mypkt->data();
    mh->seqno = htonl(fi.last_seq);
}

void
passive_monitor::sequence_receive(pkt *mypkt)
{
    struct ron_monitor_header *mh =
	(struct ron_monitor_header *)mypkt->data();

    flowid fid(mypkt);
    u_int32_t pseq = ntohl(mh->seqno);

    map<flowid, received_flow_info>::const_iterator res;
    received_flow_info fi;
    res = received_flowcache.find(fid);

    if (res != received_flowcache.end()) {
	fi = res->second;
    } else {
	// Initialize the flowcache
    }
    received_flowcache[fid] = fi;
    DPRINTF(DEBUG_PASSIVE, "Recording FID %s seqno %d\n",
	    fid.to_string().c_str(), pseq);
}

int
passive_monitor::transmit(pkt *mypkt, node *mynode, int via_ron)
{
    pkt *ret = new pkt();
    if (!ret) {
	DEBUG_PERROR("Could not allocate pkt");
	return NULL;
    }
    struct ron_monitor_header *fh;
    fh = (struct ron_monitor_header *)ret->data();

    *ret->ron_hdr() = *mypkt->ron_hdr();
    
    fh->new_type = mypkt->ron_hdr()->type;
    ret->set_type(RON_MONITORED);
    ret->set_datalen(mypkt->datalen() + sizeof(struct ron_monitor_header));

    memcpy(ret->data() + sizeof(struct ron_monitor_header),
	   mypkt->data(),
	   mypkt->datalen());

    /* Keep a record of the transmission ... */
    sequence_transmit(ret);
    
    int rc = dispatch(ret, mynode, via_ron);
    delete(ret);
    return rc;
}

int
passive_monitor::recv(pkt *mypkt, node *mynode, int via_ron)
{
    struct ron_monitor_header *mh;
    //    DPRINTF(DEBUG_PASSIVE, "passive_monitor::recv got packet type %d\n",
    //    mypkt->get_type());
    if (mypkt->get_type() != RON_MONITORED) {
	return transmit(mypkt, mynode, via_ron);
    }
    
    if (mypkt->datalen() < sizeof(struct ron_monitor_header)) {
	DPRINTF(DEBUG_PASSIVE|DEBUG_ERRS,
		"passive_monitor::recv runt packet (%d bytes)\n",
		mypkt->datalen());
	return -1;
    }

    mh = (struct ron_monitor_header *)mypkt->data();

    flowid fid(mypkt);
    u_int32_t pseq = ntohl(mh->seqno);
    // (These two are unnecessary except for debugging output)
    sequence_receive(mypkt);

    /*
     * Take the existing packet and send it to the local delivery
     * mechanism, with its new type header.  This is inefficient
     * and eventually will become annoying and need fixing.  For now,
     * it works.
     */

    pkt *newpkt;
    newpkt = new pkt();
    *newpkt->ron_hdr() = *mypkt->ron_hdr();
    memcpy(newpkt->data(),
	   mypkt->data() + sizeof(struct ron_monitor_header),
	   mypkt->datalen() - sizeof(struct ron_monitor_header));

    newpkt->ron_hdr()->type = mh->new_type;
    newpkt->set_datalen(mypkt->datalen() - sizeof(struct ron_monitor_header));

    DPRINTF(DEBUG_PASSIVE,
	    "passive_monitor::recv FID %s seq %d from %s type %d\n",
	    fid.to_string().c_str(), pseq,
	    mypkt->get_src().to_string().c_str(), newpkt->get_type());

    dispatch(newpkt, NULL, 1);

    delete newpkt;
    return 0;
}
