
/* -*-  Mode:C; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
 
#include <string>

#include "plumber.h"
#include "rondebug.h"
#include "packet.h"
#include "forw.h"
#include "scheduler.h"

using namespace RON;

 
/* Debug descriptions of RON packets */
char *_ron_pkt_descr[] = { "RON_DATA",
			   "RON_ROUTING",
			   "RON_PING",
			   "RON_FLOOD",
			   "RON_MEMBERSHIP",
			   "RON_MAYBEDOWN" ,
                           "RON_TEST" };
 
Plumber::Plumber()
{
    default_dst = NULL;
    type_dst_table.clear();

}

Plumber::~Plumber()
{
    type_dst_table.clear();
}

int
Plumber::recv(pkt *packet, node *mynode, int via_ron)
{
    return dispatch(packet, mynode, via_ron);
}

/* XXX - backwards compatibility, sort this out */
int
Plumber::register_type(u_int32_t packet_type, Plumber *dst, void *data)
{
    return register_type(packet_type, dst);
}

int
Plumber::register_type(u_int32_t packet_type, Plumber *dst)
{
    DPRINTF(DEBUG_DISPATCH,
	    "Plumber::register_type callback %p for packet type 0x%x\n",
	    dst, packet_type);
    type_dst_table[packet_type] = dst;
    return 0;
}

void
Plumber::register_default(Plumber *dst)
{
    default_dst = dst;
}

int
Plumber::dispatch(pkt *packet, node *mynode, int via_ron)
{
    int ptype = packet->get_type();
    map<u_int32_t, Plumber *>::const_iterator p = type_dst_table.find(ptype);

    if (p == type_dst_table.end()) {
	if (default_dst != NULL) {
	    return default_dst->recv(packet, mynode, via_ron);
	} else {
	    DPRINTF(DEBUG_ERRS|DEBUG_DISPATCH,
		    "Plumber::dispatch no default unknown packet type 0x%x\n",
		    ptype);
	    return -1;
	}
    }
    if (p->second == 0) {
	DPRINTF(DEBUG_ERRS|DEBUG_DISPATCH,
		"Plumber::dispatch bad callback entry for type 0x%x\n",
		ptype);
	return -1;
    }
    return p->second->recv(packet, mynode, via_ron);
}

void Plumber::describe()
{
	map<u_int32_t, Plumber *>::iterator iter;
	int len1 = 0, len2 = 0;

	for (iter = type_dst_table.begin();
		 iter != type_dst_table.end();
		 ++iter)
		{
			int index = iter->first,
				curLen1 = strlen(_ron_pkt_descr[index]), 
				curLen2 = iter->second->get_name().length();

			if (curLen1 > len1)
				len1 = curLen1;

			if (curLen2 > len2)
				len2 = curLen2;
		}

	for (int i = 0; i < (len1 + len2 + 7); i++)
		{
			DPRINTF(DEBUG_DISPATCH, "-");
		}
	DPRINTF(DEBUG_DISPATCH, "\n");

	DPRINTF(DEBUG_DISPATCH, " Routing table of <%s>\n", get_name().c_str());
	
	for (int i = 0; i < (len1 + len2 + 7); i++)
		{
			DPRINTF(DEBUG_DISPATCH, "-");
		}
	DPRINTF(DEBUG_DISPATCH, "\n");
	
	for (iter = type_dst_table.begin();
		 iter != type_dst_table.end();
		 ++iter)
		{
			DPRINTF(DEBUG_DISPATCH, "| %*s | %*s |\n",
				   len1,
				   _ron_pkt_descr[iter->first],
				   len2,
				   iter->second->get_name().c_str());				   
		}
	
	for (int i = 0; i < (len1 + len2 + 7); i++)
		{
			DPRINTF(DEBUG_DISPATCH, "-");
		}
	DPRINTF(DEBUG_DISPATCH, "\n");

	if (default_dst)
	    {
		// Now print default
		DPRINTF(DEBUG_DISPATCH, "Default destination: %s\n", default_dst->get_name().c_str());
		
		for (int i = 0; i < (len1 + len2 + 7); i++)
		    {
			DPRINTF(DEBUG_DISPATCH, "-");
		    }
		DPRINTF(DEBUG_DISPATCH, "\n");
	    }


}


//#define _TEST_PLUMBER_
#ifdef _TEST_PLUMBER_
unsigned int debug = 32767;

class testme : public Plumber
{
public:
    int recv(pkt *packet, pkt_control *control) {
	DPRINTF("Got packet: %s\n", packet->data());
	return 0;
    }
};

Packet::Packet() { contents = (char *)malloc(100); }
pkt::~pkt() { free(contents); }

int
main()
{
    Plumber p1, p2;
    testme t1;
    pkt p;
    p1.register_type(0, &p2);
    p2.register_type(0, &t1);
    DPRINTF("Plumber::main\n");

    p.set_type(0);
    sprintf(p.data(), "Hi");
    p1.recv(&p, NULL, 0);
}

#endif
