
#include "static_router.h"
#include "rondebug.h"

using namespace RON;


void
static_router::configure(Scheduler *s, forw *f)
{
    sched = s;
    myforw = f;
    f->register_type(RON_ROUTING, this, NULL);
    DPRINTF(DEBUG_INIT|DEBUG_ROUTING,
	    "static_router::configure with forw %p\n", f);
}

void
static_router::init()
{
    FILE *infile;
    char inbuf[256];
    char *fname;

    table.clear(); /* XXX:  Should check to make sure we can read... */

    fname = infilename;

    if (!fname) {
	fname = getenv("ROUTES");
	if (!fname) {
	    fname = "routes.txt";
	}
    }

    infile = fopen(fname, "r");
    if (!infile) {
	DEBUG_PERROR("static_router::init could not open input file");
	return;
    }

    while ((fgets(inbuf, sizeof(inbuf)-1, infile)) != NULL) {
	char *comment;
	char dst[256], nxt[256];

	comment = strchr(inbuf, '#');
	if (comment) *comment = '\0';
	/* XXX:  note that this parsing can overflow.  shrug. */

	comment = strchr(inbuf, '\n');
	if (comment) *comment = '\0';
	if ((sscanf(inbuf, "%s %s", dst, nxt)) == 2) {
	    table[inet_addr(dst)] = inet_addr(nxt);
	}
    }
    fclose(infile);
}

int
static_router::reload()
{
    init();
    return 0; /* XXX */
}

static_router::static_router()
{
    infilename = NULL;
    init();
}

static_router::static_router(char *filename)
{
    infilename = filename;
    init();
}

node *
static_router::lookup(node *dst, pkt *mypkt) {
    unsigned long dstaddr;
    dstaddr = dst->node_addr.s_addr;
    
    map<unsigned long, unsigned long>::const_iterator p = table.find(dstaddr);
    if (p == table.end()) {
	return NULL;
    }
    
    node *newnode = new node();
    if (!newnode) {
	DEBUG_PERROR("static_router::lookup could not allocate newnode");
	return NULL;
    }
    newnode->node_addr.s_addr = p->second;
    
    return newnode;
}

int
static_router::recv(pkt *mypkt, node *mynode, int via_ron)
{
    DPRINTF(DEBUG_ERRS,
	    "static_router::receive_packet got a packet;  unsupported\n");

    return 0;
}
