#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ron.h"

#include "ronlib.h"
#include "rondebug.h"

/* XXX: Stolen from flood_protocol.h */
struct flood_header {
    u_int32_t id1;
    u_int32_t id2;
    u_int32_t new_type;
    struct in_addr src_addr;
    u_int16_t src_port;
};


void usage() {
    fprintf(stderr,
	    "Usage:\n"
	    "test-udp [-t type] [-v vers] [-D ron_dst] <target>\n"
	    );
}


void help() {
    usage();
    fprintf(stderr,
	    " target ............. Machine to send the UDP packet to\n"
	    " -v <version> ....... Set the RON version (default 1)\n"
	    " -t <type>    ....... set the packet type (default 0)\n"
	    " -F <flood ID> ...... Append a flood header\n"
	    " -T <flood type> .... Set flood payload type\n"
	    " -D <dst> ........... Set header destination\n"
	    " -M <host> .......... Send a maybedown notification for M\n"
	    "\n"
	    " -S ................. send to stdout, not network\n"
	    "\n"
	    " -h ................. Help (this message)\n"
	    " -d <debug_flags> ... Enable debugging\n"
	    );
}

int main(int argc, char **argv)
{
    char buf[256];
    struct ron_pkt_header *ph;
    char *dst;

    extern char *optarg;
    extern int optind;
    int ch;

    int hostnameset = 0;
    char myhostname[MAXHOSTNAMELEN];
    unsigned long myaddr;
    struct in_addr myinaddr;
    int to_stdout = 0;

    struct sockaddr_in dstsock;
    struct flood_header *fh;
    struct ron_maybedown *may;

    int forw_port;

    int dst_set = 0;
    int do_flood = 0;
    int do_maybe = 0;

    int sock;
    int msglen, rc;
    char tmp[32];
    int indata;

    bzero(buf, sizeof(buf));

    ph = (struct ron_pkt_header *)buf;

    ph->version = 1;
    ph->hlim = DEFAULT_RON_HLIM;
    ph->src_port = htons(DEFAULT_RON_FORWARDER_PORT);
    ph->dst_port = htons(DEFAULT_RON_FORWARDER_PORT);

    fh = (struct flood_header *)((char *)buf + sizeof(struct ron_pkt_header));
    may = (struct ron_maybedown *)((char *)buf + sizeof(struct ron_pkt_header));
    msglen = sizeof(struct ron_pkt_header);

    while ((ch = getopt(argc, argv, "m:hp:d:D:v:t:SF:T:M:")) != -1)
	switch (ch) {
	case 'S':
	    to_stdout = 1;
	case 'M':
	    do_maybe = 1;
	    if (nametoip(optarg, &may->node_addr)) {
		perror(optarg);
		exit(-1);
	    }
	    ph->type = htonl(RON_MAYBEDOWN);
	    may->node_port = htons(DEFAULT_RON_FORWARDER_PORT); /* XXX */
	    msglen += sizeof(struct ron_maybedown);
	    break;
	case 'D':
	    if (nametoip(optarg, &ph->dst_addr)) {
		perror(optarg);
		exit(-1);
	    }
	    break;
	case 'F':
	    if (strlen(optarg) < 18 || optarg[0] != '0' || optarg[1] != 'x') {
		fprintf(stderr, "invalid flood ID\n");
		usage();
		exit(-1);
	    } else {
		do_flood = 1;
		optarg += 2;
		strncpy(tmp, optarg, 16);
		tmp[8] = '\0';
		optarg += 8;
		fh->id1 = htonl(strtol(tmp, NULL, 16));
		fh->id2 = htonl(strtol(optarg, NULL, 16));
		printf("id1:  %ld  id2:  %ld\n",
		       ntohl(fh->id1),
		       ntohl(fh->id2));
		msglen += sizeof(struct flood_header);
		fh->new_type = 0;
	    }
	    break;
	case 'T':
	    if (isdigit(optarg[0])) { 
		fh->new_type = htonl(atoi(optarg));
	    } else {
		usage();
		exit(-1);
	    }
	    break;
	case 'm':
	    hostnameset = 1;
	    if (nametoip(optarg, &myinaddr)) {
		perror(optarg);
		exit(-1);
	    }
	    myaddr = myinaddr.s_addr;
	    break;
	case 't':
	    if (!isdigit(optarg[0])) {
		fprintf(stderr, "invalid type %s\n", optarg);
		exit(-1);
	    }
	    ph->type = htonl(atoi(optarg));
	    break;
	case 'p':
	    forw_port = atoi(optarg);
	    break;
	case 'h':
	    help();
	    exit(0);
	case 'd':
	    if (set_debug(optarg)) {
		exit(-1);
	    }
	    break;
	default:
	    usage();
	    exit(-1);
	}
    argc -= optind;
    argv += optind;

    sock = bound_udp_socket(9999);
    if (sock == -1) {
	perror("bound_udp_socket");
	exit(-1);
    }

    if (!hostnameset) {
	if (gethostname(myhostname, sizeof(myhostname)-1)) {
	    perror("gethostname");
	    exit(-1);
	}
	if (nametoip(myhostname, &myinaddr)) {
	    perror(myhostname);
	    exit(-1);
	}
    }

    
    if (argc < 1) {
	dst = "127.0.0.1";
    } else {
	dst = argv[0];
    }

    if (!dst_set) {
	if (nametoip(dst, &(ph->dst_addr))) {
	    perror("could not inet_aton");
	}
    }
    printf("My IP:  %s\n", inet_ntoa(myinaddr));
    printf("Target: %s\n", inet_ntoa(ph->dst_addr));

    if (do_flood) {
	fh->src_addr = myinaddr;
	fh->src_port = ph->dst_port;
    }

#ifdef RON_HAS_STRUCT_SINLEN
    dstsock.sin_len = sizeof(struct sockaddr_in);
#endif
    dstsock.sin_family = AF_INET;
    dstsock.sin_port = ph->dst_port;
    if (nametoip(dst, &(dstsock.sin_addr))) {
	perror(dst);
	exit(-1);
    }

    /* Now start filling in the buffer with user-supplied data */
    

    indata = sizeof(buf) - msglen;

    while (indata > 0) {
	rc = read(STDIN_FILENO, (buf+msglen), indata);
	if (rc <= 0) {
	    indata = 0;
	} else {
	    indata -= rc;
	    msglen += rc;
	}
    }

    printf("Sending %d bytes\n", msglen);

    if (!to_stdout) {
	rc = sendto(sock, buf, msglen, 0,
		    (struct sockaddr *)&dstsock, sizeof(struct sockaddr_in));
    } else {
	rc = write(STDOUT_FILENO, buf, msglen);
    }

    printf("sendto: %d\n", rc);
    
    exit(0);
}
