#ifndef _RON_H_
#define _RON_H_

#include <sys/types.h>
#include <netinet/in.h>
#include <fixedpoint.h>
#ifdef DMALLOC
#include "dmalloc.h"
#endif

const int maxpktsize = 1600;
const int maxpayload = 1500;

#define DEFAULT_RON_FORWARDER_PORT 9432
#define DEFAULT_RON_HLIM   8

#define DEFAULT_RON_TTCP_PORT1 5001
#define DEFAULT_RON_TTCP_PORT2 5002

/* How often do we probe each other host? */
#define PING_INTERVAL 10.0
/* #define PING_INTERVAL 60.0 */ /* temporary test */
/* Defines, indirectly, how often we send routing updates.  should be
 * cleaned up. */
#define LATROUTER_CALLBACK_INTERVAL 1400000
/* #define LATROUTER_CALLBACK_INTERVAL 10000000 *//* temporary test */

#define RON_VERSION 1

/* Types of RON packets */
#define RON_DATA       0x00000000
#define RON_ROUTING    0x00000001
#define RON_PING       0x00000002
#define RON_FLOOD      0x00000003
#define RON_MEMBERSHIP 0x00000004
#define RON_MAYBEDOWN  0x00000005 /* Node may be down notification */
#define RON_TEST       0x00000006
#define RON_MESH       0x00000007
#define RON_MESH_DATA  0x00000008 /* Data headed for the mesh router */
#define RON_MESHTEST   0x00000009 /* Probes for playing with meshing */
#define RON_MONITORED  0x00000010 /* Traffic tagged by the monitor */
#define RON_PRINT      0x00000100 /* Testing traffic... just dump */

/* Metrics upon which to route */
#define RON_ROUTE_DEFAULT 0x0000
#define RON_ROUTE_LOSS    0x0001
#define RON_ROUTE_LAT     0x0002
#define RON_ROUTE_BW      0x0003
#define RON_ROUTE_MESH_LAT  0x0004
#define RON_ROUTE_MESH_LOSS 0x0005


#define TRAFFIC_NORMAL 0x0000
#define TRAFFIC_NOVBNS 0x0001

#define VERS_DIRECTMASK 0x80 /* Set the high bit */
struct ron_pkt_header {
    u_int8_t  version;
    u_int8_t  hlim;		/* Hop limit (# hops remaining) */
    u_int16_t routing_flags;	/* How do we route it?  Akin to IP TOS */
    struct in_addr src_addr;
    struct in_addr dst_addr;	/* IP address of destination RON node */
    u_int32_t type;		/* Data or control or something else? */
    u_int32_t seq;		/* Sequence number; used by QoS and mpath */
    u_int32_t flowid;		/* Sender-created flow ID. */
    u_int16_t traffic_type;	/* Policy routing traffic type */
    u_int16_t src_port;
    u_int16_t dst_port;		/* destination port at remote RON node */
};

struct ron_routing_header {
    struct in_addr src_addr;	  /* src for entire packet */
    u_int32_t      table_version; /* my current table version */
    u_int16_t      src_port;      /* port, net byte order, for entire packet */
    u_int8_t nroutes;		  /* Number of route entries in pkt */
};

struct ron_maybedown {
    struct in_addr node_addr;
    u_int32_t      node_port;
};


#define RON_METRIC_LATENCY 0
#define RON_METRIC_REACH   1
#define RON_METRIC_OUTAGE  2
/* Users define more */

/* Simple entries for the basic two-level RON forwarding */
struct ron_routing_entry {
    /*struct in_addr src_addr;*/   /* IP address of announcing RON node */
    struct in_addr dst_addr;   /* IP address of dest RON node */
    u_int16_t dst_port;            /* port of dest RON node */
    u_int8_t  metric;
    struct fixedpoint_wire val;
};

struct ron_flooding_header {
    u_int32_t id1;
    u_int32_t id2;
    u_int32_t new_type;
    struct in_addr src_addr;
    u_int16_t src_port;
};

struct ron_meshing_header {
    u_int32_t new_type;
};

struct ron_monitor_header {
    u_int32_t new_type;
    u_int32_t seqno; /* Per-destination, end-to-end sequence number */
};


#endif /* _RON_H_ */
