#ifndef _DEBUG_H_
#define _DEBUG_H_

#include <stdio.h>  /* for perror */
/* Things for the packet debugging.  Probably don't belong in here.. */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ctype.h>
#include <sys/param.h>
#include <sys/time.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include "nu_debug.h"

//#define DEBUG 

extern unsigned int debug;

#ifdef DEBUG
#define DPRINTF(level, fmt, args...) \
        do { if (debug & (level)) fprintf(stderr, fmt , ##args ); } while(0)
#define DEBUG_PERROR(errmsg) \
        do { if (debug & DEBUG_ERRS) perror(errmsg); } while(0)
#else
#define DPRINTF(args...)
#define DEBUG_PERROR(args...)
#endif

/*
 * The format of this should be obvious.  Please add some explanatory
 * text if you add a debugging value.  This text will show up in
 * -d list
 */
#define DEBUG_NONE      0x00	// DBTEXT:  No debugging
#define DEBUG_ERRS      0x01	// DBTEXT:  Verbose error reporting
#define DEBUG_INIT      0x02	// DBTEXT:  Debug initialization
#define DEBUG_SOCKETS   0x04    // DBTEXT:  Debug socket operations
#define DEBUG_PROCESSES 0x08	// DBTEXT:  Debug processes (fork/reap/etc)
#define DEBUG_TCP       0x10	// DBTEXT:  Debug TCP operations
#define DEBUG_PACKETS   0x20	// DBTEXT:  Debug all packets
#define DEBUG_ROUTING   0x40	// DBTEXT:  Debug routing
#define DEBUG_DISPATCH  0x80	// DBTEXT:  Debug event dispatch
#define DEBUG_MEMBERSHIP 0x100	// DBTEXT:  Debug membership
#define DEBUG_FLOOD      0x200	// DBTEXT:  Debug flooding
#define DEBUG_PING       0x400	// DBTEXT:  Debug network probing
#define DEBUG_POLICY     0x800	// DBTEXT:  Debug policy routing
#define DEBUG_MULTIPATH  0x1000 // DBTEXT:  Debug multipath extensions
#define DEBUG_ROUTING_MESSAGES 0x2000 // DBTEXT: Debug routing messages
#define DEBUG_MESH      0x4000 // DBTEXT: Debug mesh-based routing
#define DEBUG_REQUESTS  0x8000	// DBTEXT:  Debug requests
#define DEBUG_MESHTEST  0x10000 // DBTEXT:  Debug mesh test packets
#define DEBUG_PASSIVE   0x20000 // DBTEXT:  Debug passive monitoring
#define DEBUG_ROUTING_V 0x40000 // DBTEXT:  Verbose routing debugging

#define DEBUG_ALL  0xffffffff

#ifdef __cplusplus
extern "C" {
#endif
int set_debug(char *arg);  /* Returns 0 on success, -1 on failure */
#ifdef __cplusplus
}
#endif

#endif /* _DEBUG_H_ */
