#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <arpa/inet.h>
#include <errno.h>
#include <net/if.h>

#ifndef IPPROTO_DIVERT
#warning "Snarf_divert requires kernel divert socket support"
int
create_divert_socket(int divert_port)
{
    fprintf(stderr, "snarf_divert: no divert socket support\n");
    exit(-1);
}
#else
/*
 * This routine is vaguely FreeBSD specific.  It requires Divert
 * sockets.  A port to Linux can be found (as of Sep 2001) at
 * http://www.anr.mcnc.org/~divert/index.shtml
 */

#include "rondebug.h"

int
create_divert_socket(int divert_port)
{
    int sock;
    struct sockaddr_in divertsock;
    int rc;

    sock = socket(PF_INET, SOCK_RAW, IPPROTO_DIVERT);
    if (sock < 0) {
	perror("snarf_freebsd::create_divert_socket socket call");
	DEBUG_PERROR("snarf_freebsd::create_divert_socket socket call");
	return -1;
    }

    bzero(&divertsock, sizeof(divertsock));
    divertsock.sin_addr.s_addr = INADDR_ANY;
    divertsock.sin_port = htons(divert_port);
    divertsock.sin_len = sizeof(divertsock);
    divertsock.sin_family = AF_INET;

    DPRINTF(DEBUG_INIT, "Binding raw socket to divert port %d\n", divert_port);
    rc = bind(sock, (struct sockaddr *)&divertsock,
	      sizeof(divertsock));
    if (rc) {
	perror("snarf_freebsd::create_divert_socket bind");
	DEBUG_PERROR("snarf_freebsd::create_divert_socket bind");
	close(sock);
	return -1;
    }

    return sock;
}

#endif /* IPPROTO_DIVERT */
