/*
 * Emulated RON Performance Database
 *
 * Lightweight/don't need to run Berkeley DB3 to store data
 * Conforms to the pdb_client spec/extends from it; implements all summarizers
 *  
 * -----------  Stan Rost
 */

#ifndef _PDB_LOCAL_CLIENT_H_
#define _PDB_LOCAL_CLIENT_H_

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/uio.h>
#include <string.h>
#include <unistd.h>
#include <string>
#include <list>
#include <map>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <math.h>

// RON includes
#include "fixedpoint.h"
#include "pdb_wire.h"
#include "ronlib.h"
#include "rondebug.h"
#include "forw.h"
#include "ron.h"

#include "pdb.h"
#include "pdb_client.h"
#include "node.h"

// Maximum length of history of samples to keep track of...
#define MAX_HISTORY PDB_MAX_ENTRIES
namespace RON {

/* Handy utility widget for mapping nodes and probes to lists.. */
class nodetype {
public:
    node _n;
    probe_name_t _p;
    nodetype(node n, probe_name_t p) {
	_n = n;
	_p = p;
    }
    nodetype() {}
    const nodetype& operator=(const nodetype& right) {
	_n = right._n;
	_p = right._p;
	return right;
    }
    bool operator==(const nodetype& right) {
	if (right._n == _n)
	    return right._p == _p;
	return (right._n == _n);
    }
    bool operator<(const nodetype& right) const {
	if (right._n == _n)
	    return (_p < right._p);
	return (_n < right._n);
    }
};

class RonDbClass {
 public:
    RonDbClass() : winLen(0), winPos(0), error(false)
	{}

    bool isError()
	{
	    bool retVal = error;
	    error = false;
	    
	    return retVal;
	}

    // Summarizers
    fixedpoint getMax(fixedpoint lastN);
    fixedpoint getMin(fixedpoint lastN);
    fixedpoint getRecent();
    fixedpoint getFirst();
    fixedpoint getEwma(fixedpoint alpha);
    fixedpoint getAvg(fixedpoint param);

    // Register sample
    void sample(fixedpoint f);

 private:
    // Position of the count's sample starting from the last one
    // (s.t. fromLast(0) = position of the last sample retained in the window)
    inline int fromLast(int count)
	{
	    if (winLen < MAX_HISTORY)
		{
		    int pos = (winLen - count - 1);
		    assert(pos >= 0);

		    return pos;
		}
	    else
		{
		    int pos = winPos - count;
		    if (pos < 0)
			pos = MAX_HISTORY + pos;

		    return pos;
		}
	}

    // History window of samples
    fixedpoint window[MAX_HISTORY];
    int winLen, // The length of the current array of samples 
	winPos; // The position of the oldest entry in the array

    // Just a flag
    bool error;
};

class pdb_local_client : public RON::pdb_client {

 public:
    pdb_local_client();
    ~pdb_local_client();
    
    void init() {}
    void configure(Scheduler *s, forw *f) { };

    int query(node n, probe_name_t probetype, int summarizer,
	      fixedpoint param, fixedpoint *result);
    int insert(node n, probe_name_t probetype, fixedpoint value);

 private:
    map<nodetype, RonDbClass *> classes;
};

} /* Namespace RON */
#endif /* _PDB_LOCAL_CLIENT_H_ */
