/*
 * A simple UDP-based performance database.
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/uio.h>
#include <unistd.h>
#include <string>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <math.h>
#include <signal.h>
#include "fixedpoint.h"
#include "pdb_wire.h"
#include "ronlib.h"
#include "rondebug.h"
#include "pdb_client.h"

using namespace RON;

void usage()
{
    fprintf(stderr,
	    "pdb_q [-sgh] [-m <host>] classname datatype [value|summarizer] [param]\n"
	    );
}

void help()
{
    usage();
    fprintf(stderr,
	    "             -h ............ help (this message)\n"
	    "             -s ............ set value\n"
	    "             -g ............ get value (default)\n"
            "             -m <host> ..... remote host\n"
	    );
}

void
alarmhandler(int sig)
{
    exit(0);
}

class pdbc_callback : public pdb_callable {
public:
    void pdb_callback(int success, char *classname,
		      int summarizer, fixedpoint val) {
	printf("GOT:  Success %d class %s  val: %s\n",
	       success, classname, val.to_string().data());
    }
};

#define OP_GET 0
#define OP_SET 1

int
main(int argc, char **argv)
{
    int op = OP_GET;
    char *classname;
    char *datatype;
    char *inval;
    fixedpoint param;

    int nreqs = 1; /* Default to only 1 request */

    extern char *optarg;
    extern int optind;
    int ch;
    char *remhost = NULL;
    
    pdb_client *pdbc;
    pdbc_callback cb;


    while ((ch = getopt(argc, argv, "hsgm:")) != -1)
	switch (ch) {
	case 'm':
	    remhost = optarg;
	    break;
	case 's':
	    op = OP_SET;
	    break;
	case 'g':
	    op = OP_GET;
	    break;
	case 'h':
	    help();
	    exit(0);
	default:
	    usage();
	    exit(-1);
	}
    argc -= optind;
    argv += optind;

    if (argc < 3) {
	usage();
	exit(-1);
    }
    if (remhost) pdbc = new pdb_client(remhost);
    else pdbc = new pdb_client();
    pdbc->init();

    classname = argv[0];
    datatype = argv[1];
    inval = argv[2];
    if (argc > 3) {
	param.set(atof(argv[3]));
    }

    DPRINTF(DEBUG_REQUESTS, "pdb_querygen::main class %s type %s inval: %s\n",
	   classname, datatype, inval);

    signal(SIGALRM, alarmhandler);


    
    if (strcmp(datatype, "fixed")) {
	fprintf(stderr, "pdb_q::main we only support fixed right now\n");
	exit(-1);
    }

    if (op == OP_SET) {
	fixedpoint val;
	val.set(atof(inval));

	int rc = pdbc->update_fixed(classname, val);
	printf("rc: %d\n", rc);
    } else if (op == OP_GET) {
	int sumtype;
	if (!strcmp(inval, "first")) sumtype = SUMMARIZE_FIRST;
	else if (!strcmp(inval, "ewma")) sumtype = SUMMARIZE_EWMA;
	else if (!strcmp(inval, "recent")) sumtype = SUMMARIZE_RECENT;
	else if (!strcmp(inval, "avg")) sumtype = SUMMARIZE_AVG;
	else if (!strcmp(inval, "min")) sumtype = SUMMARIZE_MIN;
	else if (!strcmp(inval, "max")) sumtype = SUMMARIZE_MAX;
	else {
	    fprintf(stderr, "invalid summarizer type\n");
	    exit(-1);
	}
	pdbc->query_fixed(classname, sumtype, param, &cb);
	alarm(3);
	pdbc->socket_callback(pdbc->getsock(), NULL);
    }
    
    exit(0);

}
