#ifndef _PDB_RES_H_
#define _PDB_RES_H_

/*
 * pdb_res is a generic result type;  it performs the task of
 * getting specific typed data out of the database and
 * returning it in a type-safe and easy-to-use manner.
 *
 * For the moment, it provides only a few access methods:
 *
 * An initializer, which takes a Dbt cursor and a Db database pointer
 * cur(type *return_value) and
 * next(type *return_value)
 *
 * Both of which fill in *return_value with an item of the
 * specified type from the database cursor handed to us in
 * the initialization.
 *
 * If it proves necessary, this class will be extended to handle
 * prev(), head(), and tail() methods as well.
 */

#include <assert.h>
#include <iostream.h>
#include <db_cxx.h>
#include <sys/time.h>

#include "pdb.h"
#include "pdb_samples.h"

class pdb;

template<class T>
class pdb_res {
private:
    Dbc *scursor;		// The DB3 access cursor we draw from
    Db *dbp;			// The DB3 database we access
    int at_head_;		// Markers:  Where in the data are we?
    int at_tail_;
    
    int db_get(T *return_value, int db_method) {
	int ret;
	T item; /* The item we return */
	Dbt key, data;
	memset(&key, 0, sizeof(key));
	memset(&data, 0, sizeof(data));
	
	ret = scursor->get(&key, &data, db_method);
	if (ret != 0) {
	    if (ret != DB_NOTFOUND)
		dbp->err(ret, "scursor->get in result class");
	    return ret;
	}
	/* Here's the evil cast */
	/* XXX:  This is nasty.  REALLY nasty.  Fix this, Dave.  Please. */

	item = *(T *)&((struct sampletype *)(data.get_data()))->val;
	*return_value = item;
	return 0;
    }
    
public:
    pdb_res(char *classname, pdb *user_db);
    pdb_res(Dbc *user_search_cursor, Db *user_db);
    ~pdb_res();

    int succeeded();
    
    /* Generic result functions for iterating through the results */
    int at_tail() { return at_tail_; }
    int at_head() { return at_head_; }

    /* Type-specific functions:  they all return an item of our type */

    int count();

    int cur(T *return_value);
    int next(T *return_value);
    int head(T *return_value);
    int prev(T *return_value);
    int tail(T *return_value);
};

#endif /* _PDB_RES_H_ */
