#ifndef _PDB_WIRE_H_
#define _PDB_WIRE_H_

#include "ronlib.h"

/*
 * The wire format of PDB queries
 */

/*
 * Types of data we understand
 */

#define TYPE_INVALID      0x0000
/* Simple types */
#define TYPE_FIXED        0x0001
#define TYPE_TWOFIXED     0x0002  /* Two paired fixed-point values */
#define TYPE_STRING       0x0003

/* And ways we know of to summarize them */
#define SUMMARIZE_INVALID 0x0000
#define SUMMARIZE_EWMA    0x0001 /* EWMA'ing to an int is bad. */
#define SUMMARIZE_MAX     0x0002
#define SUMMARIZE_MIN     0x0003
#define SUMMARIZE_FIRST   0x0004
#define SUMMARIZE_RECENT  0x0005 /* Most recent entry */
#define SUMMARIZE_AVG     0x0006 /* Average of last N items */
#define SUMMARIZE_MEANDEV 0x0007 /* Mean deviation, a'la TCP */

#define PDB_INSERT 0		/* for querytype */
#define PDB_QUERY  1
#define PDB_ANSWER 2
#define PDB_NOANSWER 3

#define PDB_APPEND    0
#define PDB_OVERWRITE 1          /* for overwrite */

#define PDB_MAXSTRING 2		/* XXX */

struct pdb_update {
    u_int8_t querytype;      /* update, insert, or something else */
    u_int8_t classname[32];  /* ASCII class name, 32 char max */
    u_int32_t classtype;     /* One of the above types, network byte order */
    struct {
	u_int32_t sec;		/* All in network byte order */
	u_int32_t usec;
    } timestamp;
    u_int32_t queryid;       /* Returned to the queryant for demux */
    union {
	u_int8_t  summarize;     /* Summarization method for queries */
	u_int8_t  overwrite;     /* Overwrite flag for inserts */
    } method;
    struct fixedpoint_wire param; /* Param to pass to summarizer */
    union {
	struct fixedpoint_wire fixedval;
	struct {
	    u_int16_t datalen;   /* Length of data, network byte order */
	    char contents[PDB_MAXSTRING];
	} freedata;
    } data;
};

struct pdb_header {
    u_int32_t nsamples;  /* Currently must be set to 1.. */
};

#endif
