/**
 *  \file RONOCDStateInfo.h 
 *  \brief This file declares the RON specific state information object.
 */
#ifndef RON_VIRTUAL_NODE_H
#define RON_VIRTUAL_NODE 1

#include <netinet/in.h>
#include <arpa/inet.h>

#include "OCDStateInfo.h"

class RONOverlayNetwork;
class DNSPacket;

class RONOCDStateInfo:public OCDStateInfo {
private:

    /** The RONOverlay associated with this state */
    RONOverlayNetwork *overlay;

    /** The real IP address of the end host represented
      * by this virtual node. (in network byte order)
      * This is the address of the remote end point.
      */
    struct in_addr realIPAddress;

    /** This is the IP address of the local end point 
      * (in network byte format)
      */
    struct in_addr myIPAddr;
    
    /** The destination handle to be put into packets sent out on this connection */
    OCDHandle remoteHandle;
    

public:
    RONOCDStateInfo(OCDHandle myHandle, OCI* oci, RONOverlayNetwork* overlay);
    RONOCDStateInfo(OCDHandle myHandle, OCI* oci, RONOverlayNetwork* overlay, struct in_addr realIPAddress);

    ~RONOCDStateInfo();

    void receivedLegacyDNSReply(uint32_t ip);

    static long calculateFlowId (char *ip_hdr);

    friend class RONOverlayNetwork;
};

#endif
