/**
 *  \file RONPacket.h
 *  \brief  This file defines the RON packet formats.
 */

#ifndef RON_PACKET_H
#define RON_PACKET_H

#include "OCDHandle.h"

#define RON_BLOCK_SETUP_REQUEST 1
#define RON_BLOCK_SETUP_REPLY 2
#define RON_BLOCK_PROXY 3
#define RON_BLOCK_FAST_PROXY 4

typedef struct {

    /** This like a destination port.
      * It is used to uniquely identify the connection state at the other end .
      */
    OCDHandle destHandle;
    
    uint8_t blockType;

    /** Offset to the start of the next block from the start of this block.
      * A value of 0 implies that there is no nextBlock.
      */
    uint16_t nextBlock;

    uint8_t piggybackLen;
} RONBlock;

typedef struct {

    RONBlock header;
    OCDHandle srcHandle;
  
    /** The real source address of the end point sending the setup request. */
    struct in_addr srcAddr;

} RONSetupRequestBlock;

typedef struct {

    RONBlock header;
    OCDHandle srcHandle;

} RONSetupReplyBlock;

typedef struct {

    RONBlock header;

    char proxyBlock[0];
} RONProxyBlock;

typedef struct {

    RONBlock header;

    OCDHandle myHandle;
    char proxyBlock[0];
} RONFastProxyBlock;


#endif
