/**
 *  \file  SampleOCD.h
 *  \brief Sample OCD definition - to act as a reference for writing a new overlay.
 */
#ifndef SAMPLE_OCD_H
#define SAMPLE_OCD_H 1

#include "OCD_include.h"

#include <string>
#include <vector>
#include <list>
#include <map>

using namespace std;


class SampleOCDStateInfo;
class OCI;
class SampleOCDContext;

/**
  * This class encapsulates a sample OCD.
  * This OCD is meant to be a reference for incorporating a new overlay into OCALA.
  * It does not illustrate all the features of OCALA (for example, middleboxes).
  * For a more detailed example, please look at the implementation of the I3 OCD module.
  */
class SampleOCD : public OCD {
    
    /** All the OC-D specific processing is done by this context object. */
    SampleOCDContext* context;
    
public:
    SampleOCD (string type, string id, 
                    string desc, string fileName, 
                    string urlSuffixes, bool handleLegacyURLs
                );
    SampleOCD (OCD *);

    virtual void init(Proxy*, OCI*, bool recoverFromCrash );

    char* toString(char *tmpStr);
    void initMyNames ();


    void send ( OCDHandle handle, char* data, int dataLen);
    OCDHandle setup (
                const string endPointName, 
                int endPointNameType,
                char *optionBlocks, 
                int optionLen,
                OCIHandle ociHandle,
                OCI* oci );

    friend class SampleOCDContext;
    friend class SampleOCDStateInfo;
};

#endif
