/**
 *  \file  SampleOCDContext.h
 *  \brief This is where the bulk of the SampleOCDSpecific processing takes place.
 */
#ifndef SAMPLE_OCD_CONTEXT_H
#define SAMPLE_OCD_CONTEXT_H 1

#ifndef _WIN32
#	include <sys/types.h>
#	include <sys/socket.h>
#else
#	include "netwrap.h"
#endif

#include <errno.h>

#include <map>

using namespace std;

class SampleOCD;
class SampleOCDStateInfo;
class OCI;

/**
  * This class contains the SampleOCD specific protocol and processing.
  * The SampleOCD class just acts as an interface to the OC-I, the bulk of the 
  * work happens here.
  */
class SampleOCDContext {

    /** The sample OCD listens for data on this connection. */
    int mySockFD;
   
    /** The port on which the sampleOCD listens. 
     *  The port number is in host byte order.
     */
    uint16_t myPortNum;
   
    /** fromAddr --> SampleOCDState hash table.
      * The hash table is keyed by the IP address (network byte order).
      * There can be many OCDState entries with the same from IP address,
      * but with different port numbers.  We linearly go through all the
      * OCDStates with the same from IP while searching for a particular 
      * from port number.
     */
    multimap<uint32_t, SampleOCDStateInfo*> map_fromAddr2OCDState;


    /** Pointer to the OCD associated with this context. */
    SampleOCD* sampleOCD;
    
    
    SampleOCDContext ( SampleOCD* sampleOCD, string fileName );

    void receivedPacketFromOverlay();
    bool sendSetupReplyOK (SampleOCDStateInfo* ocdState);
    bool sendSetupPacket (SampleOCDStateInfo* ocdState);
    bool setupTunnel(SampleOCDStateInfo* ocdState);
    void continueTunnelSetup(SampleOCDStateInfo* ocdState);
    void processSetupPacket(const char* recvBuf, int recvLen, struct sockaddr_in fromAddr);
    void processDataPacket(char* recvBuf, int recvLen, struct sockaddr_in remoteAddress);
    void processSetupOKPacket(const char* recvBuf, int recvLen, struct sockaddr_in remoteAddress);
    void processSetupFailPacket(const char* recvBuf, int recvLen, struct sockaddr_in remoteAddress);

    void init();

    void addRemoteHost (SampleOCDStateInfo* currOCDState);
    SampleOCDStateInfo* lookupRemoteHost (struct in_addr remoteAddress, uint16_t remotePort);

    int sendPacket (SampleOCDStateInfo* ocdState, char* buf, int bufLen);
    bool sendDataPacket (SampleOCDStateInfo* ocdState, char* buf, int bufLen);

    friend class SampleOCD;
    friend class SampleOCDStateInfo;
};

#endif
