/**
 *  \file SampleOCDPacket.h
 *  \brief Defines the format of packets sent out and received by the SampleOCD.
 */

#ifndef SAMPLE_OCD_PACKET_H
#define SAMPLE_OCD_PACKET_H 1

#include "OCD_include.h"

// Define the Sample OCD packet types.

/** This packet type is used to set up a connection. */
#define SAMPLE_OCD_PKT_SETUP    0x01

/** This packet type is used to indicate successful setup */
#define SAMPLE_OCD_PKT_SETUP_OK 0x02

/** This packet type is used to indicate failed setup */
#define SAMPLE_OCD_PKT_SETUP_FAIL 0x03

/** This packet type is used to indicate data payload 
 *  i.e. the contents of the packet are to be handed off to the OC-I layer,
 *  It is not meant to be interpreted by the OC-D layer.
 */
#define SAMPLE_OCD_PKT_DATA     0x04

/**
  * Generic Sample OCD Packet Block 
  */
typedef struct {

    /* SAMPLE_OCD_PKT_SETUP, SAMPLE_OCD_PKT_SETUP_OK, SAMPLE_OCD_PKT_DATA etc. */
    uint8_t blockType;
    
} SampleOCDPacketHeader;

/** This represents a packet containing payload that is to be handed off
  * to the OC-I layer.
  */
typedef struct {

    SampleOCDPacketHeader header;

    /** This is just a way to refer to the payload.  
      * No memory is actually allocated. 
      * You should allocate enough memory for the payload
      * and then cast the allocated meore to a pointer of this structure type.
     */
    char payload[0];

} SampleOCDPacketData;


#endif //SAMPLE_OCD_PACKET
