/**
 *  \file SampleOCDStateInfo.cpp
 *  \brief The definition of functions and constructors of the SampleOCDStateInfo.
*/

#ifndef _WIN32

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#else 
#	include "netwrap.h"
#endif

#include "OCD_include.h"

#include "SampleOCDStateInfo.h"
#include "SampleOCD.h"
#include "SampleOCDContext.h"
#include "SampleOCD_debug.h"
#include "TFunctor.h"

SampleOCDStateInfo::SampleOCDStateInfo(
			OCDHandle myHandle, OCI* oci, SampleOCD *overlay,
			string endPointName
			/* OCD specific parameters */
		)
            : OCDStateInfo (myHandle, oci, overlay) {

    // OCD specific initialization
	strcpy (this->endPointName, endPointName.c_str());
} 

/**
  * This function is called when the DNS reply for the end point name of this OCD state
  * has been received.
  * @param ipAddress The IP address in host byte order. 0 if DNS resolution failed.
  */
void SampleOCDStateInfo::receivedLegacyDNSReply (uint32_t ipAddress) {

    if (ipAddress == 0) {
        // DNS resolution failed.
		SAMPLE_OCD_PRINT_DEBUG1 (DEBUG_LEVEL_WARNING,
                    "DNS resolution of %s failed.\n",
					this->endPointName
				);
        
        // Tell OCI that setup failed.
        notifySetupFailure();
    } else {

        struct in_addr tmpAddr;
        tmpAddr.s_addr = htonl(ipAddress);
        SAMPLE_OCD_PRINT_DEBUG2 (DEBUG_LEVEL_SUPER,
                    "Resolved %s to %s. Continuing tunnel setup...\n",
                    this->endPointName, inet_ntoa (tmpAddr)
                );
        this->remoteAddress.s_addr = htonl(ipAddress);
        ((SampleOCD*)this->ocd)->context->continueTunnelSetup(this);
    }
}

void SampleOCDStateInfo::refreshState() {

    //printf ("\n\n\n\n SAMPLE_OCD___REFRESH_STATE\n\n\n");
    // We need to fake a notifySetupSuccess() callback asynchronously.
    // If there was some actual refreshing to do for SampleOCD, we wouldn't
    // have done this shady stuff.
    // TODO: Come up with clean mechanism for notifySetupSuccess callbacks.
    TObjectFunctor<SampleOCDStateInfo> *tmpFakeSetupSuccessFunctor  = 
                new TObjectFunctor<SampleOCDStateInfo> (this, &SampleOCDStateInfo::fakeSetupSuccess);
	//PRINT_INFO1 (INFO_LEVEL_MINIMAL, "SampleOCDStateinfo::refreshState %p\n", tmpFakeSetupSuccessFunctor);

    TimerEvent* tmpFakeSetupSuccessTimer = TimerEvent::create (0, 1, (TFunctor *) tmpFakeSetupSuccessFunctor, true /* automatically free cbk when TimerEvent is destroyed*/);
    ocd->addTimerEvent(tmpFakeSetupSuccessTimer);
}

/**
  * This function exists only in order for notifySetupSuccess to be called asynchronously
  * by refreshState
  */
void SampleOCDStateInfo::fakeSetupSuccess() {
    printf ("\n\n\n\n SAMPLE_OCD___NOTIFY_SETUP_SUCCESS_STATE\n\n\n");
    notifySetupSuccess();
}
