# This script is used to make a new OCD based on SampleOCD.
# Suppose you want to create a new OCD called MyNewOCD.
# This script will create a new directory called MyNewOCD at the same level
# as SampleOCD.  
# It will then do the following inside the MyNewOCD directory:
# 1. Rename all SampleOCD* files to MyNewOCD*.
# 2. Appropriate change Makefile.am
# 3. Appropriately change the class names.

if [ $# -lt 1 ]
then
    echo "Usage: Please specify the name of your new OCD (eg: ./makeNewOCD MyNewOCD)"
    exit -1
fi

NEW_OCD_NAME=$1
NEW_OCD_DIR=../$NEW_OCD_NAME

# Check if a directory for the new OCD already exists.
if [ -d $NEW_OCD_DIR ]
then
    echo "$NEW_OCD_DIR already exists.  I will not overwrite it. Aborting..."
    exit -1
fi

# Create the new directory and copy the contents of SampleOCD
cp -r . $NEW_OCD_DIR

cd $NEW_OCD_DIR

if [ $? -ne 0 ]
then
    echo "Unable to change directory to $NEW_OCD_DIR. Aborting..."
    exit -1
fi

# Delete all unwanted files
rm -rf `find . -name CVS`
rm -rf `find . -name .deps`
rm -rf `find . -name .libs`
rm -rf `find . -name "*.lo"`
rm -rf `find . -name "*.o"`
rm -rf `find . -name "*.a"`
rm -rf `find . -name "*.dll"`
rm -rf Makefile
rm -rf Makefile.in
rm -rf configure
rm -rf config.h
rm -rf config.h.*
rm -rf config.log
rm -rf config.status
rm -rf autom4te.cache
rm -rf aclocal.m4
rm -rf libtool
rm -rf stamp-h1
rm -rf makeNewOCD.sh
rm -rf *.so

# Rename the files.
# sed does not create all the files.
mv SampleOCD.h ${NEW_OCD_NAME}.h
mv SampleOCDPacket.h ${NEW_OCD_NAME}Packet.h
mv SampleOCDContext.h ${NEW_OCD_NAME}Context.h
mv SampleOCDStateInfo.h ${NEW_OCD_NAME}StateInfo.h
mv SampleOCD_debug.h ${NEW_OCD_NAME}_debug.h
mv SampleOCD_Windows.h ${NEW_OCD_NAME}_Windows.h

mv SampleOCD.cpp ${NEW_OCD_NAME}.cpp
mv SampleOCDContext.cpp ${NEW_OCD_NAME}Context.cpp
mv SampleOCDStateInfo.cpp ${NEW_OCD_NAME}StateInfo.cpp
mv SampleOCD_debug.c ${NEW_OCD_NAME}_debug.c
mv SampleOCD_Windows.cpp ${NEW_OCD_NAME}_Windows.cpp
mv sampleOCD.xml ${NEW_OCD_NAME}.xml

# Change Makefile.am
sed -e "s/SampleOCD/$NEW_OCD_NAME/g" ../sampleOCD/Makefile.am > Makefile.am


# Change Makefile.apple
sed -e "s/SampleOCD/$NEW_OCD_NAME/g" ../sampleOCD/Makefile.apple > Makefile.apple

SED_CMD1=s/SampleOCD/$NEW_OCD_NAME/g
SED_CMD2=s/sampleOCD/var$NEW_OCD_NAME/g
SED_CMD3=s/SAMPLE_OCD/${NEW_OCD_NAME}/g


SED_CMD="-e $SED_CMD1 -e $SED_CMD2 -e $SED_CMD3"

# Change the .cpp and .h files
sed $SED_CMD  ../sampleOCD/SampleOCD.h > ${NEW_OCD_NAME}.h
sed $SED_CMD ../sampleOCD/SampleOCD.cpp > ${NEW_OCD_NAME}.cpp
sed $SED_CMD ../sampleOCD/SampleOCDContext.h > ${NEW_OCD_NAME}Context.h
sed $SED_CMD ../sampleOCD/SampleOCDContext.cpp > ${NEW_OCD_NAME}Context.cpp
sed $SED_CMD ../sampleOCD/SampleOCDStateInfo.h > ${NEW_OCD_NAME}StateInfo.h
sed $SED_CMD ../sampleOCD/SampleOCDStateInfo.cpp > ${NEW_OCD_NAME}StateInfo.cpp
sed $SED_CMD ../sampleOCD/SampleOCD_Windows.h > ${NEW_OCD_NAME}_Windows.h
sed $SED_CMD ../sampleOCD/SampleOCD_Windows.cpp > ${NEW_OCD_NAME}_Windows.cpp
sed $SED_CMD ../sampleOCD/SampleOCD_debug.h > ${NEW_OCD_NAME}_debug.h
sed $SED_CMD ../sampleOCD/SampleOCD_debug.c > ${NEW_OCD_NAME}_debug.c
sed $SED_CMD ../sampleOCD/SampleOCDPacket.h > ${NEW_OCD_NAME}Packet.h
sed $SED_CMD ../sampleOCD/sampleOCD.xml > ${NEW_OCD_NAME}.xml
sed $SED_CMD ../sampleOCD/debug_fns.h > debug_fns.h

# Visual Studio
mv visual_studio/SampleOCD visual_studio/$NEW_OCD_NAME
mv visual_studio/$NEW_OCD_NAME/SampleOCD.sln visual_studio/$NEW_OCD_NAME/${NEW_OCD_NAME}.sln
mv visual_studio/$NEW_OCD_NAME/SampleOCD.vcproj visual_studio/$NEW_OCD_NAME/${NEW_OCD_NAME}.vcproj

sed $SED_CMD ../sampleOCD/visual_studio/SampleOCD/SampleOCD.sln > visual_studio/$NEW_OCD_NAME/${NEW_OCD_NAME}.sln
sed $SED_CMD ../sampleOCD/visual_studio/SampleOCD/SampleOCD.vcproj > visual_studio/$NEW_OCD_NAME/${NEW_OCD_NAME}.vcproj


echo
echo "Congratulations! Your new OCD $NEW_OCD_NAME has been created in $NEW_OCD_DIR."
echo "You may now require some manual customization for your OCD."
echo "For example, you may want to change MY_OCD_TYPE_STR in ${NEW_OCD_NAME}.cpp."
echo
echo "Windows Warning: Please make sure that you never include both SampleOCD and $NEW_OCD_NAME inside the same Visual Studio solution file as both projects have the same GUIDs.  Visual Studio might get confused."
echo
echo
echo "To compile $NEW_OCD_NAME, execute ../compile inside the ${NEW_OCD_DIR} subdirectory."
echo
echo "Good luck developing $NEW_OCD_NAME."
echo "       - The OCALA team @ Berkeley."
echo
echo

